/**************************************************************************
 *                                                                        *
 * Xors3D Engine official header for C\C++. Copyright 2009 Xors3D Team    *
 * WWW: http://www.xors3d.com                                             *
 * E-Mail: support@xors3d.com                                             *
 *                                                                        *
 * Xors3D header  is distributed in hope that it will be useful, but      *
 * WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility *
 * to anyone for the  consequences of using  it or for  whether it serves *
 * any particular purpose or works at all, unless he says so in writing.  *
 *                                                                        *
 **************************************************************************/
#ifndef _XORS3D_H_
#define _XORS3D_H_

// standard headers
#include <windows.h>

/*! \mainpage
* Xors3d is a game development tool. This engine can be used for development
* of games, simulations and visualizations. Using Xors3d you're able to
* construct game of any genre. It combines simplicity, flexibility and power.
*
* Documentation is partly based on original Blitz3D help.
*
* For Xors3d 1.14 26 Jun 2009
*/

// Types definition
/*! \defgroup typedefs Object types definitions*/
/*@{*/
/*! \typedef Entity
 *  \brief Entities type definition.
 *
 *   This type uses all functions for working with entities, such as xEntityX(), xEntityFX(), etc.
 *   Also, functions such as xLoadMesh(), xCreateCube() return Entity as result.
 */
typedef int Entity;
/*! \typedef Font
*  \brief Fonts type definition.
* 
*   This type uses all functions for working with fonts for loading and drawing.
*   xLoadFont() returns Font as result, use xSetFont() to set current font.
*/
typedef int Font;
/*! \typedef Surface
*  \brief Surfaces type definition.
*
*   This type uses all functions for working with surfaces (create surfaces by xCreateSurface(), 
*   work with surface vertices and triangles by xAddVertex(), xVertexX() and other)
*   Functions for creating and finding surfaces (like xCreateSurface() or xFindSurface())
*   return Surface as result.
*/
typedef int Surface;
/*! \typedef PostEffect
*  \brief Post effect type definition.
*
*   Deprecated, see xCreatePostEffectPoly() for post-effect realization.
*/
typedef int PostEffect;
/*! \typedef VertexShader
*  \brief Vertex shader type definition.
*
*   Deprecated, use Effect for shaders.
*/
typedef int VertexShader;
/*! \typedef PixelShader
*  \brief Pixel shader type definition.
*
*   Deprecated, use Effect for shaders.
*/
typedef int PixelShader;
/*! \typedef Texture
*  \brief Texture type definition.
*
*   This type uses all functions for work with textures (creating, loading, 
*   changing blend type, scaling, etc). Functions for texture creating or loading
*   (like xCreateTexture() or xLoadTexture()), return Texture as result.
*/
typedef int Texture;
/*! \typedef Image
*  \brief Image type definition.
*
*   This type uses all functions for work with 2D images (e.g. for interface
*   realization), for its loading, drawing, collision detection.
*   Funcions like xLoadImage() return Image as result.
*/
typedef int Image;
/*! \typedef Brush
*  \brief Brush type definition.
*
*   This type uses all functions for work with brushes. Brushes collect
*   entity material parameters, such as color, FX falgs, alpha value, etc.
*   You can apply brush for entity by xPaintEntity() command, and change
*   its material in this way . Functions like xCreateBrush() will return
*   Brush as result.
*/
typedef int Brush;
/*! \typedef Buffer
*  \brief Buffer type definition.
*
*   This type uses all functions for working with buffers. Buffers collect
*   matrices or vectors for passing into shader (e.g. instansing).
*/
typedef int Buffer;
/*! \typedef Sound
*  \brief Sound type definition.
*
*   This type uses all functions for working with sounds, for loading
*   and plaing. Function like xLoadSound() returns Sound as result.
*/
typedef int Sound;
/*! \typedef Channel
*  \brief Channel type definition.
*
*   This type uses all functions for working with sound channels. If you
*   play sound with xPlaySound() it returns a new channel.
*   You can use it to control sound playback, adjusting volume, etc.
*/
typedef int Channel;
/*! \typedef Effect
*  \brief Effect type definition.
*
*   This type uses all functions for working with shader effects. You can load
*   a new effect from FX file with xLoadFXFile() and assign it to entity with
*   xSetEntityEffect(). Using shaders you can realize a lot of effects, like
*   per-pixel lighting, normal-mapping, shadows and much more.
*/
typedef int Effect;
/*! \typedef Group
*  \brief Group type definition.
*
*   This type uses all functions for working with groups. Groups collect
*   entities excluded from main rendering cycle (by xRenderWorld()) in some
*   reasons (e.g. you need to render to shadows map LOD versions of meshes).
*   Using xRenderGroup() command you can render only specified group.
*/
typedef int Group;
/*@}*/

// Log levels
/*! \defgroup loglevels Logging level*/
/*@{*/
//! On this logging level nothing will be written in log-file.
//!
#define LOG_NO							5
//! On this logging level only fatal errors will be written in log-file.
//!
#define LOG_FATAL						4
//! On this logging level all errors will be written in log-file.
//!
#define LOG_ERROR						3
//! On this logging level all errors and warnings will be written in log-file.
//!
#define LOG_WARNING						2
//! On this logging level all message types will be written in log-file,
//! except informative messages.
#define LOG_MESSAGE						1
//! On this logging level all messages types will be written in log-file.
//!
#define LOG_INFO						0
/*@}*/

// Log targets
/*! \defgroup logtargets Logging output targets*/
/*@{*/
//! Log will be written to html-file specified in xCreateLog().
//!
#define LOG_HTML						1
//! Log will be written to standard output stream (std::cout in C++)
//!
#define LOG_COUT						2
//! Log will be written to standard output stream (std::cout in C++),
//! and html-file specified in xCreateLog().
#define LOG_HTML_COUT					3
//! Log will be written to string (may be get with xGetLogString())
//!
#define LOG_STRING						4 
//! Log will be written to string (may be get with xGetLogString()),
//! and html-file specified in xCreateLog().
#define LOG_HTML_STRING					5 
//! Log will be written to string (may be get with xGetLogString()),
//! and standard output stream (std::cout in C++)
#define LOG_COUT_STRING					6 
//! Log will be written to string (may be get with xGetLogString()),
//! standard output stream (std::cout in C++) and
//! html-file specified in xCreateLog().
#define LOG_HTML_COUT_STRING			7 
/*@}*/

// skinning methods
/*! \defgroup skinmethods Skinning methods
Skinning deforms entity mesh in respect to animated skeleton.
*/
/*@{*/
//! Unvalaliable now. Skinning is computed on GPU,
//! it is faster than software skinning, but video-card needs support of
//! hardware indexed vertex blending which is avaliable only on ATi (37 bones max).
#define SKIN_HARDWARE					0
//! Skinning is computed on GPU (in vertex shaders), it is faster than
//! software skinning, but you need to add skinning code in all your
//! vertex shaders which are used on animated meshes.
#define SKIN_HARDWAREVS					1
//! Skinning is computed on CPU, it is a slowly method but vertex shader
//! will receive already skinned vertices, so your don't need to write your own
//! skinning code in all vertex shaders. This method is used in original Blitz3D
#define SKIN_SOFTWARE					2
/*@}*/

// lighting types
/*! \defgroup lighttypes Light types*/
/*@{*/
//! Directional light works like sun. It has not a position or a range, it has
//! direction of light rays and light color only.
#define LIGHT_DIRECTIONAL				1
//! Point (omni) light works like a light bulb. It has a position, a range and a color,
//! but has not a direction.
#define LIGHT_POINT						2
//! Spot light is a cone of light. It starts with an inner angle of light,
//! and then extends towards an outer angle of light (cone angles you may
//! specify by xLightConeAngles() command). This light type has a position,
//! a direction, a range and a color.
#define LIGHT_SPOT						3
/*@}*/

// types of texture filtering
/*! \defgroup texfilters Texture filtering types*/
/*@{*/
//! Texture filtering not used.
//! 
#define TF_NONE							0
//! Point filtering is used . The texel with coordinates nearest to the desired
//! pixel value is used. The texture filter to be used between mipmap levels
//! is nearest-point mipmap filtering. The rasterizer uses the color from
//! the texel of the nearest mipmap texture. 
#define TF_POINT						1
//! Bilinear interpolation filtering is used. A weighted average of a 2 x 2
//! area of texels surrounding the desired pixel is used. The texture filter
//! to use between mipmap levels is trilinear mipmap interpolation. The
//! rasterizer linearly interpolates pixel color, using the texels of the
//! two nearest mipmap textures.
#define TF_LINEAR						2
//! 2X anisotropic texture filtering is used. Compensates for distortion caused
//! by the difference in angle between the texture polygon and the plane
//! of the screen. 
#define TF_ANISOTROPIC					3
//! 4X anisotropic texture filtering is used. Compensates for distortion caused
//! by the difference in angle between the texture polygon and the plane
//! of the screen. 
#define TF_ANISOTROPICX4				4
//! 8X anisotropic texture filtering is used. Compensates for distortion caused
//! by the difference in angle between the texture polygon and the plane
//! of the screen. 
#define TF_ANISOTROPICX8				5
//! 16X anisotropic texture filtering is used. Compensates for distortion caused
//! by the difference in angle between the texture polygon and the plane
//! of the screen. 
#define TF_ANISOTROPICX16				6
/*@}*/

// PS versions
/*! \defgroup psversions Pixel Shaders versions
Deprecated. Use effects and xLoadFXFile() for shaders.
*/
/*@{*/
//! Shader will be compiled for 1.1 shader model.
//!
#define PS_1_1							0
//! Shader will be compiled for 1.2 shader model.
//!
#define PS_1_2							1
//! Shader will be compiled for 1.3 shader model.
//!
#define PS_1_3							2
//! Shader will be compiled for 1.4 shader model.
//!
#define PS_1_4							3
//! Shader will be compiled for 2.0 shader model.
//!
#define PS_2_0							4
//! Shader will be compiled for 3.0 shader model.
//!
#define PS_3_0							5
/*@}*/

// VS versions
/*! \defgroup vsversions Vertex Shaders versions
Deprecated. Use effects and xLoadFXFile() for shaders.
*/
/*@{*/
//! Shader will be compiled for 1.1 shader model.
//!
#define VS_1_1							0
//! Shader will be compiled for 2.0 shader model.
//!
#define VS_2_0							1
//! Shader will be compiled for 3.0 shader model.
//!
#define VS_3_0							2
/*@}*/

// types of matrices semantics
/*! \defgroup mtxsemantic Matrices semantics*/
/*@{*/
//! World matrix to be passed to shader constant.
//! It translates, rotates and scales mesh vertices.
//! xPositionEntity(), xRotateEntity(), xScaleEntity()
//! change this matrix.
#define WORLD							0
//! Product of entity world matrix, camera view matrix
//! and camera projection matrix to be passed to shader constant.
#define WORLDVIEWPROJ					1
//! Product of camera view matrix and camera projection matrix
//! to be passed to shader constant.
#define VIEWPROJ						2
//! View matrix of active camera to be passed to shader constant.
//! It translates and rotates all scene to place it in
//! camera view space. xPositionEntity(), xRotateEntity()
//! called for camera entity change this matrix.
#define VIEW							3
//! Projection matrix of active camera to be passed to shader constant.
//! It translates and rotates all scene to place it in
//! camera projection space (camera viewport). xCameraViewport(), xCameraRange(),
//! xCameraZoom() change this matrix.
#define PROJ							4
//! Product of entity world matrix and camera view matrix
//! to be passed to shader constant.
#define WORLDVIEW						5
//! Inverted view matrix of active camera to be passed to shader constant.
//! 
#define VIEWINVERSE						6
//! Deprecated. Now you can pass matrices of other entities to
//! shader directly.
#define ADDITIONAL1						7
//! Deprecated. Now you can pass matrices of other entities to
//! shader directly.
#define ADDITIONAL2						8
//! Deprecated. Now you can pass matrices of other entities to
//! shader directly.
#define ADDITIONAL3						9
//! Deprecated. Now you can pass matrices of other entities to
//! shader directly.
#define ADDITIONAL4						10
//! Deprecated. Now you can pass matrices of other entities to
//! shader directly.
#define ADDITIONAL5						11
//! Deprecated. Now you can pass matrices of other entities to
//! shader directly.
#define ADDITIONAL6						12
//! Deprecated. Now you can pass matrices of other entities to
//! shader directly.
#define ADDITIONAL7						13
//! Deprecated. Now you can pass matrices of other entities to
//! shader directly.
#define ADDITIONAL8						14
//! Inverted and transposed world matrix of an entity to be passed
//! to shader constant.
#define WORLDINVERSETRANSPOSE			15
//! Inverted world matrix of an entity to be passed to shader constant.
//! 
#define WORLDINVERSE					16
//! Transposed world matrix of an entity to be passed to shader constant.
//! 
#define WORLDTRANSPOSE					17
//! Inverted product of camera view matrix and camera projection matrix
//! to be passed to shader constant.
#define VIEWPROJINVERSE					18
//! Inverted and transposed product of camera view matrix and camera projection matrix
//! to be passed to shader constant.
#define VIEWPROJINVERSETRANSPOSE		19
//! Transposed view matrix of camera to be passed to shader constant.
//! 
#define VIEWTRANSPOSE					20
//! Inverted and transposed view matrix of camera to be passed to shader constant.
//! 
#define VIEWINVRSETRANSPOSE				21
//! Inverted projection matrix of camera to be passed to shader constant.
//! 
#define PROJINVERSE						22
//! Transposed projection matrix of camera to be passed to shader constant.
//! 
#define PROJTRANSPOSE					23
//! Inverted and transposed projection matrix of camera to be passed to shader constant.
//! 
#define PROJINVRSETRANSPOSE				24
//! Transposed product of entity world matrix, camera view matrix and
//! camera projection matrix to be passed to shader constant.
#define WORLDVIEWPROJTRANSPOSE			25
//! Inverted product of entity world matrix, camera view matrix and
//! camera projection matrix to be passed to shader constant.
#define WORLDVIEWPROJINVERSE			26
//! Inverted and transposed product of entity world matrix,
//! camera view matrix and camera projection matrix
//! to be passed to shader constant.
#define WORLDVIEWPROJINVERSETRANSPOSE	27
//! Transposed product of entity world matrix and camera view matrix
//! to be passed to shader constant.
#define WORLDVIEWTRANSPOSE				28
//! Inverted product of entity world matrix and camera view matrix
//! to be passed to shader constant.
#define WORLDVIEWINVERSE				29
//! Inverted and transposed product of entity world matrix and camera view matrix
//! to be passed to shader constant.
#define WORLDVIEWINVERSETRANSPOSE		30
/*@}*/

// AA types
/*! \defgroup aatypes AntiAliasing types*/
/*@{*/
//! Antialiasing will be disabled.
//!
#define AANONE							0
//! 2x antialiasing will be used.
//!
#define AA2SAMPLES						1
//! 3x antialiasing will be used.
//! Current hardware does not support this type.
#define AA3SAMPLES						2
//! 4x antialiasing will be used.
//!
#define AA4SAMPLES						3	
//! 5x antialiasing will be used.
//! Current hardware does not support this type.
#define AA5SAMPLES						4
//! 6x antialiasing will be used.
//!
#define AA6SAMPLES						5
//! 7x antialiasing will be used.
//! Current hardware does not support this type.
#define AA7SAMPLES						6
//! 8x antialiasing will be used.
//!
#define AA8SAMPLES						7
//! 9x antialiasing will be used.
//! Current hardware does not support this type.
#define AA9SAMPLES						8
//! 10x antialiasing will be used.
//! Current hardware does not support this type.
#define AA10SAMPLES						9
//! 11x antialiasing will be used.
//! Current hardware does not support this type.
#define AA11SAMPLES						10
//! 12x antialiasing will be used.
//! Current hardware does not support this type.
#define AA12SAMPLES						11
//! 13x antialiasing will be used.
//! Current hardware does not support this type.
#define AA13SAMPLES						12
//! 14x antialiasing will be used.
//! Current hardware does not support this type.
#define AA14SAMPLES						13
//! 15x antialiasing will be used.
//! Current hardware does not support this type.
#define AA15SAMPLES						14
//! 16x antialiasing will be used .
//!
#define AA16SAMPLES						15	
/*@}*/

// keyboard and mouse scancodes
//! \cond FALSE
#define MOUSE_LEFT						0
#define MOUSE_RIGHT						1
#define MOUSE_MIDDLE					2
#define MOUSE4							3
#define MOUSE5							4
#define MOUSE6							5
#define MOUSE7							6
#define MOUSE8							7
#define KEY_ESCAPE						1
#define KEY_1							2
#define KEY_2							3
#define KEY_3							4
#define KEY_4							5
#define KEY_5							6
#define KEY_6							7
#define KEY_7							8
#define KEY_8							9
#define KEY_9							10
#define KEY_0							11
#define KEY_MINUS						12
#define KEY_EQUALS						13
#define KEY_BACK						14
#define KEY_TAB							15
#define KEY_Q							16
#define KEY_W							17
#define KEY_E							18
#define KEY_R							19
#define KEY_T							20
#define KEY_Y							21
#define KEY_U							22
#define KEY_I							23
#define KEY_O							24
#define KEY_P							25
#define KEY_LBRACKET					26
#define KEY_RBRACKET					27
#define KEY_RETURN						28
#define KEY_LCONTROL					29
#define KEY_RCONTROL					157
#define KEY_A							30
#define KEY_S							31
#define KEY_D							32
#define KEY_F							33
#define KEY_G							34
#define KEY_H							35
#define KEY_J							36
#define KEY_K							37
#define KEY_L							38
#define KEY_SEMICOLON					39
#define KEY_APOSTROPHE					40
#define KEY_GRAVE						41
#define KEY_LSHIFT						42
#define KEY_BACKSLASH					43
#define KEY_Z							44
#define KEY_X							45
#define KEY_C							46
#define KEY_V							47
#define KEY_B							48
#define KEY_N							49
#define KEY_M							50
#define KEY_COMMA						51
#define KEY_PERIOD						52
#define KEY_SLASH						53
#define KEY_RSHIFT						54
#define KEY_MULTIPLY					55
#define KEY_MENU						56
#define KEY_SPACE						57
#define KEY_F1							59
#define KEY_F2							60
#define KEY_F3							61
#define KEY_F4							62
#define KEY_F5							63
#define KEY_F6							64
#define KEY_F7							65
#define KEY_F8							66
#define KEY_F9							67
#define KEY_F10							68
#define KEY_NUMLOCK						69
#define KEY_SCROLL						70
#define KEY_NUMPAD7						71
#define KEY_NUMPAD8						72
#define KEY_NUMPAD9						73
#define KEY_SUBTRACT					74
#define KEY_NUMPAD4						75
#define KEY_NUMPAD5						76
#define KEY_NUMPAD6						77
#define KEY_ADD							78
#define KEY_NUMPAD1						79
#define KEY_NUMPAD2						80
#define KEY_NUMPAD3						81
#define KEY_NUMPAD0						82
#define KEY_DECIMAL						83
#define KEY_TILD						86
#define KEY_F11							87
#define KEY_F12							88
#define KEY_NUMPADENTER					156
#define KEY_RMENU						221
#define KEY_PAUSE						197
#define KEY_HOME						199
#define KEY_UP							200
#define KEY_PRIOR						201
#define KEY_LEFT						203
#define KEY_RIGHT						205
#define KEY_END							207
#define KEY_DOWN						208
#define KEY_NEXT						209
#define KEY_INSERT						210
#define KEY_DELETE						211
#define KEY_LWIN						219
#define KEY_RWIN						220
#define KEY_BACKSPACE					KEY_BACK
#define KEY_NUMPADSTAR					KEY_MULTIPLY
#define KEY_LALT						184
#define KEY_CAPSLOCK					58
#define KEY_NUMPADMINUS					KEY_SUBTRACT
#define KEY_NUMPADPLUS					KEY_ADD
#define KEY_NUMPADPERIOD				KEY_DECIMAL
#define KEY_DIVIDE						181
#define KEY_NUMPADSLASH					KEY_DIVIDE
#define KEY_RALT						56
#define KEY_UPARROW						KEY_UP
#define KEY_PGUP						KEY_PRIOR
#define KEY_LEFTARROW					KEY_LEFT
#define KEY_RIGHTARROW					KEY_RIGHT
#define KEY_DOWNARROW					KEY_DOWN
#define KEY_PGDN						KEY_NEXT	
//! \endcond

// Camera fog mode
/*! \defgroup fogtypes Fog types*/
/*@{*/
//! Fog will be disabled.
//!
#define FOG_NONE						0
//! Linear fog will be used. Fog will be computed by this formula:
//! f = (end - d) / (end - start), where 'start' is the distance at which
//! fog effects begin, 'end' is the distance at which fog effects no longer
//! increase, 'd' represents depth, or the distance from the viewpoint.
//! Specify 'start' and 'end' with xCameraFogRange() command.
#define FOG_LINEAR						1
/*@}*/

// camera projection mode
/*! \defgroup projtypes Camera projection types*/
/*@{*/
//! Camera will be disabled.
//!
#define PROJ_DISABLE					0
//! Perspective projection matrix will be used. The two most characteristic
//! features of perspective are that objects are drawn:
//! - Smaller as their distance from the observer increases
//! - Foreshortened: the size of an object's dimensions along the line of
//! sight are relatively shorter than dimensions across the line of sight
#define PROJ_PERSPECTIVE				1
//! Orthographic projection matrix will be used. Orthographic projection
//! corresponds to a perspective projection with a hypothetical viewpoint --
//! e.g., one where the camera lies an infinite distance away from the object
//! and has an infinite focal length, or "zoom".
#define PROJ_ORTHOGRAPHIC				2
/*@}*/

// Entity FX flags
/*! \defgroup fxflags Entities FX flags*/
/*@{*/
//! No effects will be applied to an entity.
//!
#define FX_NOTHING						0 
//! Lights will not affect the entity.
//!
#define FX_FULLBRIGHT					1
//! Vertex diffuse color will be used instead of material diffuse color
//! (set by xEntityColor() command).
#define FX_VERTEXCOLOR					2
//! In the flat shading mode, the rendering pipeline renders a polygon, using
//! the color of the polygon material at its first vertex as the color
//! for the entire polygon. 3D objects that are rendered with flat shading have
//! visibly sharp edges between polygons if they are not coplanar.
#define FX_FLATSHEDED					4
//! Fog will not affect the entity.
//!
#define FX_DISABLEFOG					8
//! Entity will be rendered without backface culling (two-sided triangles).
//!
#define FX_DISABLECULLING				16
/*@}*/

// entity blend types
/*! \defgroup entblendtypes Entity blending modes*/
/*@{*/
//! Entity will be blended in respect to its aplha value. It uses this formula: 
//! RGBr = (As * RGBs) + ((1.0 - As) * RGBd), where 'RGBr' - result color,
//! 'RGBs' - source pixel color (from rendered entity), 'RGBd' - 
//! destination pixel color (from rendering buffer), 'As' - source alpha
//! value (set by xEntityAlpha() command).
#define BLEND_ALPHA						1
//! Entity will be multiplied with rendering buffer. It uses this formula: 
//! RGBr = RGBs * RGBd, where 'RGBr' - result color, 'RGBs' - source pixel
//! color (from rendered entity), 'RGBd' - destination pixel color (from
//! rendering buffer)
#define BLEND_MULTIPLY					2
//! Entity color multiplied by alpha value will be added to rendering buffer.
//! It uses this formula: RGBr = (RGBs * As) + RGBd, where 'RGBr' - result
//! color, 'RGBs' - source pixel color (from rendered entity), 'RGBd' - 
//! destination pixel color (from rendering buffer), 'As' - source alpha
//! value (set by xEntityAlpha() command).
#define BLEND_ADD						3	
/*@}*/

// compare functions
/*! \defgroup cmpfunc Compare functions*/
/*@{*/
//! Always fail the test.
//!
#define CMP_NEVER						1
//! Accept if value is less than the current value. 
//!
#define CMP_LESS						2
//! Accept if value equals the current value. 
//!
#define CMP_EQUAL						3
//! Accept if value is less than or equal to the current value. 
//!
#define CMP_LESSEQUAL					4
//! Accept if value is greater than the current value. 
//!
#define CMP_GREATER						5
//! Accept if value does not equal the current value. 
//!
#define CMP_NOTEQUAL					6
//! Accept if value is greater than or equal to the current value. 
//!
#define CMP_GREATEREQUAL				7
//! Always pass the test. 
//!
#define CMP_ALWAYS						8	
/*@}*/

// axis defines
/*! \defgroup axistypes Axis*/
/*@{*/
//! x-axis will be used (from left to right).
//!
#define AXIS_X							1
//! y-axis will be used (from bottom to top).
//!
#define AXIS_Y							2
//! z-axis will be used (from back to front).
//!
#define AXIS_Z							3
/*@}*/

// Texture loading flags
/*! \defgroup tlflags Texture loading flags*/
/*@{*/
//! Texture will be loaded with color channels.
//!
#define FLAGS_COLOR						1
//! Texture will be loaded with alpha channel.
//!
#define FLAGS_ALPHA						2
//! All black pixels (0xFFFFFFFF color) will be transparent.
//!
#define FLAGS_MASKED					4
//! Mip-map levels for texture will be generated.
//!
#define FLAGS_MIPMAPPED					8
//! Any part of a texture that lies outsides the U coordinates of range
//! [0; 1] will not be drawn. Prevents texture-wrapping. 
//!
#define FLAGS_CLAMPU					16
//! Any part of a texture that lies outsides the V coordinates of range
//! [0; 1] will not be drawn. Prevents texture-wrapping. 
//!
#define FLAGS_CLAMPV					32
//! This works by taking a single image, and then applying it to a 3D mesh
//! in such a way that the image appears to be reflected. When used with a
//! texture that contains light sources, it can give some meshes such as a
//! teapot a shiny appearance. 
#define FLAGS_SPHERICALENVMAP			64
//! Cube mapping is similar to spherical mapping, except it uses six images
//! each representing a particular 'face' of an imaginary cube, to give the
//! appearance of an image that perfectly reflects its surroundings. 
#define FLAGS_CUBICENVMAP				128
//! Floating-point texture will be created (it uses all 32 bits for a single channel,
//! usual textures use 8 bits per channel). It is necessary for writing scene depth
//! to texture, e.g. for shadow mapping or DOF.
#define FLAGS_R32F						256
//! Reserved, don't use.
//!
#define FLAGS_RESERVED			    	512
//! Texture will be loaded as volume. Volumes are organized into slices
//! and can be thought of as width x height 2D surfaces stacked to make a
//! width x height x depth volume. Each slice is a single row. Volumes can
//! have subsequent levels in which the dimensions of each level are truncated
//! to half the dimensions of the previous level. 
#define FLAGS_VOLUMETEXTURE				1024
//! Floating-point texture will be created similar to FLAGS_R32F but it will
//! reserve 16 bit per channel. It is necessary for writing some scene information
//! like pixel position, normal for some shader effects.
#define FLAGS_ARBG16F					2048
//! Floating-point texture will be created similar to FLAGS_R32F but it will
//! reserve 32 bit per channel. It is necessary for writing some scene information
//! like pixel position, normal for some shader effects.
#define FLAGS_ARBG32F					4096
/*@}*/

// texture blending modes
/*! \defgroup texblendtypes Textures blending types*/
/*@{*/
//! Texture will not be blended with other layers.
//!
#define TEXBLEND_NONE					0
//! Texture will be blended with other layers in respect to its alpha value.
//!
#define TEXBLEND_ALPHA					1	
//! Texture pixel colors will be multiplied.
//!
#define TEXBLEND_MULTIPLY				2
//! Texture pixel colors will be added.
//!
#define TEXBLEND_ADD					3
//! Emulates DOT3 bump-mapping. Modulates the components of each argument as
//! signed components, adds their products; then replicates the sum to all
//! color channels, including alpha. This operation is supported for color
//! and alpha operations. 
#define TEXBLEND_DOT3					4
//! Texture pixel colors will be multiplied and shifted the products to the
//! left 1 bit (effectively multiplying them by 2) for brightening. Used
//! for lightmaps.
#define TEXBLEND_LIGHTMAP				5
//! Texture will be blended with another one in respect to alpha value of previous layer.
//! E.g. you may set diffuse maps on layers #0 and #2 and separate them with alpha-map
//! (set texture with alpha channel on layer #1). It is useful if you need to
//! blend tiled textures, but you does not need an alpha channel to be tiled.
#define TEXBLEND_SEPARATEALPHA			6
/*@}*/

// cube map faces
/*! \defgroup facetypes Cubemap faces*/
/*@{*/
//! xTextureBuffer() will return left (negative X) face.
//!
#define FACE_LEFT						0
//! xTextureBuffer() will return front (positive Z) face.
//!
#define FACE_FORWARD					1
//! xTextureBuffer() will return right (positive X) face.
//!
#define FACE_RIGHT						2
//! xTextureBuffer() will return back (negative Z) face.
//!
#define FACE_BACKWARD					3
//! xTextureBuffer() will return top (positive Y) face.
//!
#define FACE_UP							4
//! xTextureBuffer() will return bottom (negative Y) face.
//!
#define FACE_DOWN						5
/*@}*/

// Entity Animation types
/*! \defgroup animtypes Animation playback types*/
/*@{*/
//! Stop animation.
//!
#define ANIMATION_STOP					0
//! Loop animation (default).
//!
#define ANIMATION_LOOP					1
//! Ping-pong animation (from first to last, from last to first and so on)
//!
#define ANIMATION_PINGPONG				2
//! One-shot animation.
//!
#define ANIMATION_ONE					3
/*@}*/

// Collision types
/*! \defgroup colltypes Collision types*/
/*@{*/
//! Sphere-to-sphere collision check.
//!
#define SPHERETOSHPHERE					1
//! Sphere-to-box collision check.
//!
#define SPHERETOBOX						2
//! Sphere-to-triangle-mesh collision check.
//! mesh.
#define SPHERETOTRIMESH					3
/*@}*/

// collision response types
/*! \defgroup resptypes Response types*/
/*@{*/
//! Object will stop moving if collision occurs.
//!
#define RESPONSE_STOP					1
//! Object will slide if collision occurs.
//!
#define RESPONSE_SLIDING				2
//! Object will slide if collision occurs,
//! but sliding down the slope will be prevented.
#define RESPONSE_SLIDING_DOWNLOCK		3
/*@}*/

// entities picking modes
/*! \defgroup picktypes Picking types*/
/*@{*/
//! Unpickable object.
//!
#define PICK_NONE						0
//! Bounding sphere will be used for picking.
//!
#define PICK_SPHERE						1
//! Triangle mesh will be used for picking.
//!
#define PICK_TRIMESH					2
//! Bounding box will be used for picking.
//!
#define PICK_BOX						3
/*@}*/

// Sprite view modes
/*! \defgroup sviewmodes Sprite view modes*/
/*@{*/
//! Sprite is always aligned to camera (like a billboard).
//! It changes its pitch and yaw values to face camera, but doesn't roll. 
#define SPRITE_FIXED					1
//! Sprite is not aligned to camera.
//! It does not change either its pitch, yaw or roll values. 
#define SPRITE_FREE						2
//! Sprite is always aligned to camera
//! It changes its yaw and pitch to face camera, and changes
//! its roll value to match cameras.
#define SPRITE_FREEROLL					3
//! Sprite is always aligned to camera in y-axis only.
//! It changes its yaw value to face camera, but not its pitch value,
//! and changes its roll value to match cameras. 
#define SPRITE_FIXEDYAW					4
/*@}*/

// deferred channels types
/*! \defgroup dctypes Deferred channel types*/
/*@{*/
//! Channel is disabled.
//!
#define DEFERRED_NONE    0
//! Standard texture with 8 bits per color channel (32 bit total) will be used.
//!
#define DEFERRED_ARGB8   1
//! Floating-point texture with 32 bits for single channel will be used
//! (like FLAGS_R32F for texture loading).
#define DEFERRED_R32F    2
//! Floating-point texture with 32 bits per color channel will be used
//! (128 bits total).
#define DEFERRED_ARGB32F 3
//! Floating-point texture with 16 bits per color channel will be used
//! (2 channels, 32 bits total).
#define DEFERRED_GR16F   4
//! Texture with 10 bits per color channel and 2 bits for alpha will be used
//! channel (32 bits total).
#define DEFERRED_RGB10A2 5
/*@}*/

// x-files optimization types
/*! \defgroup xopttypes .x file format optimization types*/
/*@{*/
//! Reorders faces to increase the cache hit rate of vertex caches.
//!
#define OPTIMIZE_VERTEXCACHE  1
//! Reorders faces to maximize length of adjacent triangles.
//!
#define OPTIMIZE_STRIPREORDER 2
//! Reorders faces to optimize for fewer attribute bundle state changes
//! and enhanced draw performance.
#define OPTIMIZE_ATTRSORT     3
/*@}*/

// joystick types
/*! \defgroup joytypes Joystick types*/
/*@{*/
//! Joystick is not connected to this port
//!
#define JOY_NONE    0
//! Digital joystick is connected to this port
//!
#define JOY_DIGITAL 1
//! Analog joystick is connected to this port
//!
#define JOY_ANALOG  2
/*@}*/

// functions declaration & call types
//! \cond FALSE
#define XDECL extern "C" __declspec(dllexport)
#define XCALL _stdcall 
//! \endcond

// Functions
/*! \defgroup comref Command reference*/
/*! \defgroup maincommands Graphics
\ingroup comref*/
/*! \defgroup logcommands Logging
\ingroup comref*/
/*! \defgroup sicommands System information
\ingroup comref*/
/*! \defgroup incommands Input
\ingroup comref*/
/*! \defgroup keycommands Keyboard
\ingroup incommands*/
/*! \defgroup mousecommands Mouse
\ingroup incommands*/
/*! \defgroup joycommands Joystick
\ingroup incommands*/
/*! \defgroup shadowcommands Shadows
\ingroup comref*/
/*! \defgroup dscommands Deferred render
\ingroup comref*/
/*! \defgroup shadercommands Shaders effects
\ingroup comref*/
/*! \defgroup rgcommands Rendering groups
\ingroup comref*/
/*! \defgroup textcommands Text drawing
\ingroup comref*/
/*! \defgroup imagecommands Images
\ingroup comref*/
/*! \defgroup texcommands Textures
\ingroup comref*/
/*! \defgroup brushcommands Brushes
\ingroup comref*/
/*! \defgroup meshcommands Meshes
\ingroup comref*/
/*! \defgroup surfcommands Surfaces
\ingroup comref*/
/*! \defgroup camcommands Cameras
\ingroup comref*/
/*! \defgroup lightcommands Lights
\ingroup comref*/
/*! \defgroup sprcommands Sprites
\ingroup comref*/
/*! \defgroup psyscommands Particle systems
\ingroup comref*/
/*! \defgroup terrcommands Terrains
\ingroup comref*/
/*! \defgroup audiocommands Audio
\ingroup comref*/
/*! \defgroup emcommands Entity movement
\ingroup comref*/
/*! \defgroup eacommands Entity animation
\ingroup comref*/
/*! \defgroup eccommands Entity control
\ingroup comref*/
/*! \defgroup escommands Entity state
\ingroup comref*/
/*! \defgroup ecolcommands Entity collision
\ingroup comref*/
/*! \defgroup mathcommands 3D maths
\ingroup comref*/

//! \brief Sets texture filtering
//! \details Loaded textures that contain text specified by
//! 'matchText' in file name param will have the provided flags added.
//! \param matchText Text string that, if found in texture filename,
//! will activate certain filters.
//! \param flags Texture flags to be applied, see '\ref tlflags'
//! for more information.
//! \see xClearTextureFilters()
//! \ingroup texcommands
XDECL void			XCALL xTextureFilter(const char * matchText, int flags);

//! \brief Clears the current texture filter list.
//! \see xTextureFilter()
//! \ingroup texcommands
XDECL void			XCALL xClearTextureFilters();

//! \brief Returns total number of DIP calls
//! \details Counter updates during xFlip() call.
//! \see xSurfRendered(), xEntityRendered(), xTrisRendered()
//! \ingroup maincommands
XDECL int			XCALL xDIPCounter();

//! \brief Returns total number of rendered surfaces
//! \details Counter updates during xFlip() call.
//! \see xDIPCounter(), xEntityRendered(), xTrisRendered()
//! \ingroup maincommands
XDECL int			XCALL xSurfRendered();

//! \brief Returns total number of rendered entities
//! \details Counter updates during xFlip() call.
//! \see xDIPCounter(), xSurfRendered(), xTrisRendered()
//! \ingroup maincommands
XDECL int			XCALL xEntityRendered();

//! \brief Checks enity visibility
//! \details Will return true if specified entity is visible.
//! \param entity Handle of entity to be checked.
//! \see xHideEntity(), xShowEntity()
//! \ingroup escommands
XDECL bool			XCALL xEntityVisible(Entity entity);

//! \brief Renders shadows
//! \details This function will render shadows related to every light source
//! (except light sources with disabled shadow casting). Shadows will be rendered
//! to specified texture, it will represent scene from specified camera view
//! with shadows only (no diffuse map, lighting or color). You can blend this
//! texture with colored scene and get a final image, with diffuse maps, colors, lighting etc.
//! If deferred render is enable you don't need to call this function,
//! shadows will be drawn automatically.
//! \param camera Camera which represents a scene
//! \param texture Texture for rendering of shadowed scene
//! \ingroup shadowcommands
XDECL void			XCALL xRenderShadows(Entity camera, Texture texture);

//! \brief Sets epsilon values of light source
//! \details Every light sources has 2 epsilon values for shadows rendering. First value
//! is a depth bias (it's needed to compensate inaccuracy of floating-point
//! values), second value is to prevent shadowing of angularly triangles. Epsilons
//! delete some noise from shadows, but can lead to incorrect rejection of
//! a part of shadow. Default values are optimal, but you may
//! need to adjust them for best results.
//! \param light Light source handle
//! \param eps1 First epsilon value (compensates inaccuracy of floating-point
//! values)
//! \param eps2 Second epsilon value (prevents shadowing of angularly triangles),
//! actually it is a cosine of maximum angle of shadowed triangle.
//! \ingroup shadowcommands
XDECL void			XCALL xLightShadowEpsilons(Entity light, float eps1, float eps2 = 0.16f);

//! \brief Sets path to shadow shader
//! \details Specified shader must contain code for rendring shadow maps
//! (writing depth textures), final scene rendering with shadows, and mixing
//! result from all light sources. If no shader is specified in this
//! function - default built-in shader ill be used.
//! \param path Shader file name
//! \ingroup shadowcommands
XDECL void			XCALL xSetShadowShader(const char * path);

//! \brief Initializes engine shadow system
//! \details You should call this function to use shadows in your game.
//! You can specify size of shadow maps for all light types separatelly. If
//! you set 0 shadow map will not be created for this light type, you can't
//! cast shadows from this type, but you will save some video memory.
//! \param dirSize Shadow map size for direction light type
//! \param spotSize Shadow map size for spot light type
//! \param pointSize Shadow map size for point light type (uses cube map,
//! it needs 6 renders of scene)
//! \ingroup shadowcommands
XDECL void			XCALL xInitShadows(int dirSize = 512, int spotSize = 512, int pointSize = 128);

//! \brief Sets PSSM system parameters
//! \details You can change number of PSSM split planes (PSSM technique splits
//! visible scene in some parts, each part will be rendered
//! separatelly to its own shadow map and will use all this map. It increases
//! quality of shadows near camera, but may cause loss of quality in large distance
//! from camera), and split lambda. We set optimal values, but you may change
//! them for best quality (or for increase of speed).
//! Also you may change projection type for shadow map rendering, and set
//! distance of light source from camera view point (larger distance causes
//! larger part of scene which casts shadows, but quality of shadows will be purer).
//! \param splitPlanes Number of split planes for PSSM rendering
//! \param splitLambda Value for active camera pliting frustum.
//! \param useOrtho True to use orthographical projection for shadow map
//! rendering, false - for perspective projection
//! \param lightDist Distance from light source to view point
//! \ingroup shadowcommands
XDECL void			XCALL xSetShadowParams(int splitPlanes = 4, float splitLambda = 0.95f, bool useOrtho = true, float lightDist = 500.0f);

//! \brief Enables or disables shadow casting for entity
//! \details In some cases you need some entity which doesn't cast shadows from
//! specified light source (e.g. light bulb). This function excludes
//! entity from list of shadow casters. By default entity casts shadows from all
//! light sources.
//! \param entity Entity which must be excluded from list of casters
//! \param light Light source from which entity will not cast shadows
//! \param state If true - entity will cast shadows.
//! \see xEntityIsCaster()
//! \ingroup shadowcommands
XDECL void			XCALL xEntityCastShadows(Entity entity, Entity light, bool state);

//! \brief Checks if entity casts shadows from light source
//! \param entity Entity to be checked
//! \param light Light to be checked
//! \see xEntityCastShadows()
//! \ingroup shadowcommands
XDECL bool			XCALL xEntityIsCaster(Entity entity, Entity light);

//! \brief Sets minimum priority for shadow casting
//! \details All light sources have shadow casting priority. If priority is
//! greater than zero - light source will cast shadows. But you can set global
//! priority and shadow casting will be disabled for all light sources,
//! which priority is less than the global one. You can use it for changing shadows
//! quality in your game (enable more light for powerful machines, and disable
//! it for old hardware) using one command.
//! \param prior Global shadow casting priority
//! \ingroup shadowcommands
XDECL void			XCALL xShadowPriority(int prior);

//! \brief Enables or disables shadow casting for light
//! \details You should set casting priority for light source greater than zero
//! to enable shadow casting from it. Priority allows you to change shadows
//! quality in your game, for more information see xShadowPriority()
//! \param light Light source handle
//! \param prior Light casting priority
//! \see xShadowPriority()
//! \ingroup shadowcommands
XDECL void			XCALL xLightEnableShadows(Entity light, int prior);

//! \brief Checks if light source has shadows
//! \param light Light source to be checked
//! \ingroup shadowcommands
XDECL bool			XCALL xLightShadowsEnabled(Entity light);

//! \brief Initializes deferred render
//! \details You should call this function before using any deferred shading
//! commands. This function will create a g-buffer and prepare other data
//! for deferred shading use. You may specify g-buffer channels formats
//! for your needs. By default (for default deferred shading shader)
//! ARGB8 is  used(8 bit per channel) for diffuse and normal maps, and R32F (32 bits
//! for one floating-point channel) for depth map, the fourth channel is disabled.
//! For more information about channels types see '\ref dctypes'.
//! \param tex1 Format for the first g-buffer channel
//! \param tex2 Format for the second g-buffer channel
//! \param tex3 Format for the third g-buffer channel
//! \param tex4 Format for the fourth g-buffer channel
//! \ingroup dscommands
XDECL void			XCALL xInitDeferred(int tex1 = DEFERRED_ARGB8, int tex2 = DEFERRED_ARGB8, int tex3 = DEFERRED_R32F, int tex4 = DEFERRED_NONE);

//! \brief Sets path to deferred render shaders
//! \details Specified shaders must contain code for deferred rendering.
//! Engine deferred shading system uses 2 shaders. First shader is used for writing
//! g-buffer, and second one is for final scene rendering from all light sources.
//! If no shader is specified, default built-in deferred shading shaders will be used.
//! \param mrtShader Shader for g-buffer rendering
//! \param lightShader Shader for final scene rendering
//! \ingroup dscommands
XDECL void			XCALL xSetDeferredShaders(const char * mrtShader, const char * lightShader);

//! \brief Renders the current scene with deferred shading
//! \details Renders the current scene to the specified rendering buffer
//! (by xSetBuffer() command) onto the rectangle defined for each camera
//! in xCameraViewport(). Every camera which is not hidden by xHideEntity() or which has a
//! xCameraProjMode() not set to 0 is rendered. It equals xRenderWorld(), but
//! scene will be rendered using of engine deferred shading system.
//! \param texture If valid texture handle is specified, scene will be rendered to
//! it (may be used for post-processing). Otherwise scene will be rendered
//! directly into backbuffer.
//! \ingroup dscommands
XDECL void			XCALL xRenderWorldDeferred(Texture texture = NULL);

//! \brief Clears log string
//! \details Clears string which contains log messages. This command doesn't
//! clear html log-file or standard output stream (std::cout).
//! \see xGetLogString()
//! \ingroup logcommands
XDECL void			XCALL xClearLogString();

//! \brief Returns string which contains log messages.
//! \details This function returns log meessages written since last
//! xClearLogString() or xGetLogString() call.
//! \see xClearLogString()
//! \ingroup logcommands
XDECL const char *	XCALL xGetLogString();

//! \brief Changes logging level
//! \details Logging level controls types of messages to be written to
//! log. See '\ref loglevels' for more information about logging levels.
//! \param level Logging level
//! \see xGetLogLevel(), xSetLogTarget()
//! \ingroup logcommands
XDECL void			XCALL xSetLogLevel(int level = LOG_WARNING);

//! \brief Changes logging target
//! \details This function allows you to specify where log messages should be written.
//! See '\ref logtargets' for more information about logging targets.
//! \param target Logging target
//! \see xGetLogTarget(), xSetLogLevel()
//! \ingroup logcommands
XDECL void			XCALL xSetLogTarget(int target = LOG_HTML);

//! \brief Returns current logging level
//! \see xSetLogLevel()
//! \ingroup logcommands
XDECL int			XCALL xGetLogLevel();

//! \brief Returns current logging target
//! \see xSetLogTarget()
//! \ingroup logcommands
XDECL int			XCALL xGetLogTarget();

//! \brief Starts logging
//! \details This function enables engine logging. You can specify
//! logging level, target and html-file for log output.
//! \param target Logging target. See '\ref logtargets' for more information
//! \param level Logging level. See '\ref loglevels' for more information
//! \param filename Log file for output (needed if HTML logging is enabled)
//! \ingroup logcommands
XDECL int			XCALL xCreateLog(int target = LOG_WARNING, int level = LOG_HTML, const char * filename = "xors_log.html");

//! \brief Returns joystick type
//! \details This command returns the type of joystick which is currently
//! connected to the computer. See '\ref joytypes' for return values.
//! \param portID Number of joystick port to be checked
//! \ingroup joycommands
XDECL int			XCALL xJoyType(int portID = 0);

//! \brief Detects if a joystick button is being held down.
//! \details This command (and its counterparts xKeyDown and xMouseDown) is
//! used to detect if a joystick button is being held down. You must check
//! for each joystick button independantly with its corresponding number.
//! \param key Number of joystick button to check 
//! \param portID Number of joystick port to check
//! \see xJoyHit()
//! \ingroup joycommands
XDECL int			XCALL xJoyDown(int key, int portID = 0);

//! \brief Returns the number of times a specified button has been hit.
//! \details This command returns the number of times a specified joystick
//! button has been hit since the last time you called the xJoyHit() command
//! \param key Number of joystick button to check 
//! \param portID Number of joystick port to check
//! \see xJoyDown()
//! \ingroup joycommands
XDECL int			XCALL xJoyHit(int key, int portID = 0);

//! \brief Returns pressed joystick button
//! \details Unlike the other similar commands (xJoyDown() and xJoyHit()),
//! this command doesn't need to know which button you are trying to test
//! for. It looks for any joystick button, then returns the number the user
//! pressed. Since you are polling all the buttons instead of just a
//! specific one, this may be a tad less efficient than using xJoyDown()
//! or xJoyHit(). 
//! \param portID Number of joystick port to check
//! \ingroup joycommands
XDECL int			XCALL xGetJoy(int portID = 0);

//! \brief Stops program execution unless any joystick key is not pressed
//! \details This command makes your program halt until a joystick button
//! is pressed on the joystick. Used alone, it simply halts and waits for
//! a button press. It return pressed button's code value.
//! \param portID Number of joystick port to check
//! \ingroup joycommands
XDECL int			XCALL xWaitJoy(int portID = 0);

//! \brief Returns the value of the x-axis of the joystick
//! \details This command returns the value of the x-axis of the joystick.
//! The range is [-1; 1] (full left to full right). The value returned is
//! a floating point number.
//! \param portID Number of joystick port to check
//! \ingroup joycommands
XDECL float			XCALL xJoyX(int portID = 0);

//! \brief Returns the value of the y-axis of the joystick
//! \details This command returns the value of the y-axis of the joystick.
//! The range is [-1; 1] (full left to full right). The value returned is
//! a floating point number.
//! \param portID Number of joystick port to check
//! \ingroup joycommands
XDECL float			XCALL xJoyY(int portID = 0);

//! \brief Returns the value of the z-axis of the joystick
//! \details This command returns the value of the z-axis of the joystick.
//! The range is [-1; 1] (full left to full right). The value returned is
//! a floating point number.
//! \param portID Number of joystick port to check
//! \ingroup joycommands
XDECL float			XCALL xJoyZ(int portID = 0);

//! \brief Returns the value of the u-axis of the joystick
//! \details Returns a float value in range [-1; 1] that represents the U
//! value of a joystick's output. The U value of a joystick usually
//! corresponds to a joystick's 'slider' or 'throttle' feature, although
//! this may vary depending on the joystick, and will not be available with
//! all joysticks. The value returned is a floating point number.
//! \param portID Number of joystick port to check
//! \ingroup joycommands
XDECL float			XCALL xJoyU(int portID = 0);

//! \brief Returns the value of the v-axis of the joystick
//! \details Returns a float value in range [-1; 1] that represents the V
//! value of a joystick's output. The value returned is a floating point
//! number.
//! \param portID Number of joystick port to check
//! \ingroup joycommands
XDECL float			XCALL xJoyV(int portID = 0);

//! \brief Returns the pitch value of a joystick
//! \param portID Number of joystick port to check
//! \ingroup joycommands
XDECL float			XCALL xJoyPitch(int portID = 0);

//! \brief Returns the yaw value of a joystick
//! \param portID Number of joystick port to check
//! \ingroup joycommands
XDECL float			XCALL xJoyYaw(int portID = 0);

//! \brief Returns the roll value of a joystick
//! \details The roll value of a joystick usually corresponds to a joystick's
//! 'twistable stick' or 'rudder' feature, although this may vary depending
//! on the joystick, and will not be available with all joysticks.
//! \param portID Number of joystick port to check
//! \ingroup joycommands
XDECL float			XCALL xJoyRoll(int portID = 0);

//! \brief Returns the state of a joystick 'hat' or 'pov'
//! (point-of-view) control.
//! \details If the returned value is -1, the hat is currently centred.
//! \param portID Number of joystick port to check
//! \ingroup joycommands
XDECL float			XCALL xJoyHat(int portID = 0);

//! \brief Returns the direction of the x-axis of the joystick
//! \details This command returns the direction of the x-axis of the joystick
//! being pressed. The value is -1 (left) or 1 (right). The value returned
//! is an integer number. Perfect for digital joysticks.
//! \param portID Number of joystick port to check
//! \ingroup joycommands
XDECL int			XCALL xJoyXDir(int portID = 0);

//! \brief Returns the direction of the y-axis of the joystick
//! \details This command returns the direction of the y-axis of the joystick
//! being pressed. The value is -1 (left) or 1 (right). The value returned
//! is an integer number. Perfect for digital joysticks.
//! \param portID Number of joystick port to check
//! \ingroup joycommands
XDECL int			XCALL xJoyYDir(int portID = 0);

//! \brief Returns the direction of the z-axis of the joystick
//! \details This command returns the direction of the z-axis of the joystick
//! being pressed. The value is -1 (left) or 1 (right). The value returned
//! is an integer number. Perfect for digital joysticks.
//! \param portID Number of joystick port to check
//! \ingroup joycommands
XDECL int			XCALL xJoyZDir(int portID = 0);

//! \brief Returns the direction of the u-axis of the joystick
//! \details This command returns the direction of the u-axis of the joystick
//! being pressed. The value is -1 (left) or 1 (right). The value returned
//! is an integer number. Perfect for digital joysticks.
//! \param portID Number of joystick port to check
//! \ingroup joycommands
XDECL int			XCALL xJoyUDir(int portID = 0);

//! \brief Returns the direction of the v-axis of the joystick
//! \details This command returns the direction of the v-axis of the joystick
//! being pressed. The value is -1 (left) or 1 (right). The value returned
//! is an integer number. Perfect for digital joysticks.
//! \param portID Number of joystick port to check
//! \ingroup joycommands
XDECL int			XCALL xJoyVDir(int portID = 0);

//! \brief Clears information about joystick keys hits
//! \details There are many times when you aren't interested in the dozens
//! of possible joystick button pressed the player might have made before
//! you are checking for one in particular. Or perhaps you want to pause
//! the game and wait for any joystick button to be hit, but you don't want
//! a 'queued' button press bypassing this. Use this command before you
//! specifically want to poll a joystick button hit from the user. 
//! \ingroup joycommands
XDECL void			XCALL xFlushJoy();

//! \brief Returns a pointer to engine function
//! \details You may use this command to get engine functions (like
//! xCreateCube(), xLoadMesh(), xFlip(), etc) addresses. They may be used
//! in script engines (like xScript).
//! \param name Name of function (e.g. 'xFlip')
//! \ingroup maincommands
XDECL void *		XCALL xGetFunctionAddress(const char * name);

//! \brief Loads a texture from an image file
//! \details Loads a texture from an image file and returns the texture's
//! handle like xLoadTexture(), but this command do it on separate thread, so
//! you will not have some lags in you game while texture is being loaded. Supported
//! file formats are: bmp, dds, dib, hdr, jpg, pfm, png, ppm, tga. 
//! The optional flags parameter allows you to apply certain effects to the
//! texture. Flags can be added to combine two or more effects. See
//! '\ref tlflags' for more information.
//! \param path Texture file name
//! \param flags Combination of '\ref tlflags'
//! \see xLoadTexture()
//! \ingroup texcommands
XDECL Texture		XCALL xLoadTextureInThread(const char * path, int flags = FLAGS_COLOR | FLAGS_MIPMAPPED);

//! \brief Returns true if texture was loaded in separate thread
//! \param texture Texture to be checked
//! \ingroup texcommands
XDECL bool			XCALL xTextureLoaded(Texture texture);

//! \brief Returns supported MRT number.
//! \details MRT (Multiple Render Targets) refers to the ability to render
//! to multiple textures with a single draw call. These textures can be
//! created independently of each other. It may be used for deferred shading
//! or other effects.
//! \ingroup maincommands
XDECL int			XCALL xGetNumberRT();

//! \brief Sets texture as a channel of MRT.
//! \details This function is used for MRT realization, it sets a texture as one of
//! MRT channels.
//! \param buffTex Texture which is needed to be set as MRT channel
//! \param frame Frame of texture which is needed to be set as MRT channel
//! \param index MRT channel index (from 0 to xGetNumberRT() - 1)
//! \see xUnSetMRT()
//! \ingroup maincommands
XDECL void			XCALL xSetMRT(Texture buffTex, int frame, int index);

//! \brief Unsets all MRT channels.
//! \details This function is used for clearing MRT channel and return rendering
//! to backbuffer.
//! \see xSetMRT()
//! \ingroup maincommands
XDECL void			XCALL xUnSetMRT();

//! \brief Returns x-axis scale of entity
//! \param entity Entity to be checked
//! \see xScaleEntity()
//! \ingroup escommands
XDECL float			XCALL xEntityScaleX(Entity entity);

//! \brief Returns y-axis scale of entity
//! \param entity Entity to be checked
//! \see xScaleEntity()
//! \ingroup escommands
XDECL float			XCALL xEntityScaleY(Entity entity);

//! \brief Returns z-axis scale of entity
//! \param entity Entity to be checked
//! \see xScaleEntity()
//! \ingroup escommands
XDECL float			XCALL xEntityScaleZ(Entity entity);

//! \brief Loads texture into memory.
//! \details This function loads texture file into memory and returns a pointer
//! to memory block containing it. You may use this pointer with
//! xCreateTextureFromRAM() command for texture creation.
//! \param  path Texture file name
//! \ingroup texcommands
XDECL void *		XCALL xLoadTextureInRAM(const char * path);

//! \brief Creates texture from memory buffer
//! \details Creates a new texture instance from memory buffer created by
//! xLoadTextureInRAM() function.
//! \param tRAM Pointer to memory buffer
//! \ingroup texcommands
XDECL Texture		XCALL xCreateTextureFromRAM(void * tRAM);

//! \brief Creates a new rendering group
//! \details Rendering groups may be used if you need to render some objects
//! separatelly from main scene. In original Blitz3D you could hide/show
//! objects, but in Xors3D you can create a rendering group, add necessary
//! entities to it and render it by xRenderGroup() command. This command returns
//! a new rendering group handle.
//! \see xGroupAddEntity(), xGroupRemoveEntity()
//! \ingroup rgcommands
XDECL Group			XCALL xCreateGroup();

//! \brief Adds entity to rendering group
//! \details Adds a new entity to rendering group. Entity will be excluded from
//! main rendering group (which is rendered by xRenderWorld()).
//! \param group Rendering group
//! \param entity Entity handle
//! \see xGroupRemoveEntity()
//! \ingroup rgcommands
XDECL void			XCALL xGroupAddEntity(Group group, Entity entity);

//! \brief Draw rendering group
//! \details This command is an equivalent to xRenderWorld(),
//! but it is used for rendering groups.
//! \param group Rendering group
//! \see xGroupAddEntity()
//! \ingroup rgcommands
XDECL void			XCALL xRenderGroup(Group group);

//! \brief Removes entity from rendering group
//! \details Entity will be included in main rendering group (which is rendered
//! by xRenderWorld()).
//! \param group Rendering group
//! \param entity Entity handle
//! \see xGroupAddEntity()
//! \ingroup rgcommands
XDECL void			XCALL xGroupRemoveEntity(Group group, Entity entity);

//! \brief Adds the source mesh to the destination mesh. 
//! \details xAddMesh() works best with meshes that have previously only
//! had mesh commands used with them. So if you want to manipulate a mesh
//! before adding it to another mesh, make sure you use xScaleMesh(),
//! xPositionMesh(), xPaintMesh() etc rather than xScaleEntity(),
//! xPositionEntity(), xEntityTexture() etc before using xAddMesh(). 
//! However, something to be aware of when using commands such as
//! xRotateMesh is that all mesh commands work from a global origin of
//! (0, 0, 0). Therefore it is generally a good idea to scale and rotate a
//! mesh before positioning it, otherwise your mesh could end up in
//! unexpected positions. Also, when using xAddMesh(), the origin of the new
//! all-in-one mesh will be set at (0, 0, 0). 
//! After using xAddMesh(), the original 'src' mesh will still exist,
//! therefore use xFreeEntity() to delete it if you wish to do so.
//! \param src Source mesh
//! \param dest Destination mesh
//! \ingroup meshcommands
XDECL void			XCALL xAddMesh(Entity src, Entity dest);

//! \brief Adds a triangle to a surface and returns the triangle's index
//! number, starting from 0.
//! \details The v0, v1 and v2 parameters are the index numbers of the
//! vertices created using xAddVertex(). 
//! Depending on how the vertices are arranged, then the triangle will only
//! be visible from a certain side. Imagine that a triangle's vertex points
//! are like dot-to-dot pattern, each numbered v0, v1, v2. If these dots,
//! starting from v0, through to V2, form a clockwise pattern relative to
//! the viewer, then the triangle will be visible. If these dots form an
//! anti-clockwise pattern relative to the viewer, then the triangle will
//! not be visible. 
//! The reason for having one-sided triangles is that it reduces the amount
//! of triangles that need to be rendered when one side faces the side of
//! an object which won't be seen (such as the inside of a snooker ball).
//! However, if you wish for a triangle to be two-sided, then you can either
//! create two triangles, using the same set of vertex numbers for both but
//! assigning them in opposite orders, or you can use xCopyEntity() and
//! xFlipMesh() together. 
//! \param surf Surface handle
//! \param v0 Index number of first vertex of triangle 
//! \param v1 Index number of second vertex of triangle 
//! \param v2 Index number of third vertex of triangle 
//! \see xAddVertex()
//! \ingroup surfcommands
XDECL int			XCALL xAddTriangle(Surface surf, int v0, int v1, int v2);

//! \brief Sets the ambient lighting colour
//! \details Ambient light is a light source that affects all points on a
//! 3D object equally. So with ambient light only, all 3D objects will
//! appear flat, as there will be no shading. 
//! Ambient light is useful for providing a certain level of light, before
//! adding other lights to provide a realistic lighting effect. 
//! An ambient light level of (0, 0, 0) will result in no ambient light
//! being displayed.
//! \param red Red ambient light value 
//! \param green Green ambient light value 
//! \param blue Blue ambient light value 
//! \ingroup maincommands
XDECL void			XCALL xAmbientLight(int red, int green, int blue);

//! \brief Enables or disables antialiasing
//! \details Antialiasing is a technique used to smooth out the
//! entire screen, so that jagged lines are made less noticeable. You must
//! set antialiasing level by xSetAntiAliasType() command before using
//! antialiasing.
//! \param state True to enable antialiasing, false to disable
//! \ingroup maincommands
XDECL void			XCALL xAntiAlias(bool state);

//! \brief Allows you to set the text of the program's title bar
//! \param title The text that will be displayed in the title bar of the
//! program window
//! \ingroup maincommands
XDECL void			XCALL xAppTitle(const char * title);

//! \brief Enables or disables auto appling xMidhandle() for all loading images
//! \details When an image is loaded with xLoadImage(), the image handle
//! (the location within the image where the image is 'drawn from') is
//! always defaulted to the top left corner (coordinates 0, 0). This means
//! if you draw an image that is 50x50 pixels at screen location (200, 200),
//! the image will begin to be drawn at (200, 200) and extend to (250, 250). 
//! The xMidHandle() command moves the image's handle to the middle of the
//! image. See this command for more information about the image's handle. 
//! This command eliminates the need for the xMidHandle() command by making
//! ALL subsequently loaded images default to having their image handles
//! set to mid.
//! \param state If true - images will load with auto midhandle, without if
//! false
//! \ingroup imagecommands
XDECL void			XCALL xAutoMidHandle(bool state);

//! \brief Returns a backbuffer handle
//! \details This is a value usually used with xSetBuffer() to denote the
//! secondary non-visible drawing buffer called the Back Buffer. In MOST
//! gaming situations, you will want to be using the xBackBuffer() for
//! drawing operations then using xFlip() to bring that buffer to the
//! front buffer where it can be seen. There are other uses for the command,
//! but this is the biggie. See xSetBuffer() for more info. Once again - if
//! you set drawing operations to the xBackBuffer() you will NOT see any of
//! them until you call xFlip().
//! \ingroup maincommands
XDECL Buffer		XCALL xBackBuffer();

//! \deprecated
XDECL void			XCALL xBufferDelete(Buffer buff);

//! \deprecated
XDECL void *		XCALL xBufferMatrixGetElement(Buffer buff, int index);

//! \deprecated
XDECL void			XCALL xBufferMatrixSetElement(Buffer buff, int index, void * matrix);

//! \deprecated
XDECL float			XCALL xBufferVectorsGetElement(Buffer buff, int index, int element);

//! \deprecated
XDECL void			XCALL xBufferVectorsSetElement(Buffer buff, int index, float x, float y, float z, float w);

//! \brief Sets camera clip plane
//! \param camera Camera handle
//! \param index Clip plane index
//! \param enabled True to enable clipping plane
//! \param a a component of clipping plane
//! \param b b component of clipping plane
//! \param c c component of clipping plane
//! \param d d component of clipping plane
//! \ingroup camcommands
XDECL void			XCALL xCameraClipPlane(Entity camera, int index, bool enabled, float a, float b, float c, float d);

//! \brief Sets camera background color. Defaults to 0, 0, 0.
//! \param camera Camera handle
//! \param red Red value of camera background color
//! \param green Green value of camera background color
//! \param blue Blue value of camera background color
//! \ingroup camcommands
XDECL void			XCALL xCameraClsColor(Entity camera, float red, float green, float blue);

//! \brief Sets camera clear mode
//! \param camera Camera handle
//! \param clearColor True to clear the color buffer, false not to
//! \param clearZBbuffer True to clear the color z-buffer, false not to
//! \ingroup camcommands
XDECL void			XCALL xCameraClsMode(Entity camera, bool clearColor = true, bool clearZBbuffer = true);

//! \brief Sets camera fog color
//! \param camera Camera handle
//! \param red Red value of fog
//! \param green Green value of fog
//! \param blue Blue value of fog
//! \ingroup camcommands
XDECL void			XCALL xCameraFogColor(Entity camera, float red, float green, float blue);

//! \brief Sets camera fog range. 
//! \details The 'nearRange' parameter specifies at what distance in front
//! of the camera that the fogging effect will start; all 3D object before
//! this point will not be faded. 
//! The 'farRange' parameter specifies at what distance in front of the
//! camera that the fogging effect will end; all 3D objects beyond this
//! point will be completely faded out.
//! \param camera Camera handle
//! \param nearRange Distance in front of camera that fog starts
//! \param farRange Distance in front of camera that fog ends
//! \ingroup camcommands
XDECL void			XCALL xCameraFogRange(Entity camera, float nearRange, float farRange);

//! \brief Picks the entity positioned at the specified viewport coordinates
//! \details Returns the entity picked, or 0 if none there. 
//! An entity must have its xEntityPickMode() set to a non-0 value value to
//! be 'pickable'. 
//! \param camera Camera handle
//! \param x 2D viewport x-coordinate
//! \param y 2D viewport y-coordinate
//! \ingroup camcommands
XDECL Entity		XCALL xCameraPick(Entity camera, float x, float y);

//! \brief Projects the world coordinates on to the 2D screen
//! \param camera Camera handle
//! \param x World coordinate x
//! \param y World coordinate y
//! \param z World coordinate z
//! \ingroup camcommands
XDECL void			XCALL xCameraProject(Entity camera, float x, float y, float z);

//! \brief Sets camera range
//! \details Try and keep the ratio of far/near as small as possible for
//! optimal z-buffer performance. Defaults to 1, 1000
//! \param camera Camera handle
//! \param nearPlane Distance in front of camera that 3D objects start
//! being drawn
//! \param farPlane Distance in front of camera that 3D object stop being
//! drawn
//! \ingroup camcommands
XDECL void			XCALL xCameraRange(Entity camera, float nearPlane, float farPlane);

//! \brief Sets the camera viewport position and size
//! \details The camera viewport is the area of the 2D screen that
//! the 3D graphics as viewed by the camera are displayed in. 
//! Setting the camera viewport allows you to achieve spilt-screen and
//! rear-view mirror effects
//! \param camera Camera handle
//! \param x x-coordinate of top left hand corner of viewport
//! \param y y-coordinate of top left hand corner of viewport
//! \param width Width of viewport
//! \param height Height of viewport
//! \ingroup camcommands
XDECL void			XCALL xCameraViewport(Entity camera, int x, int y, int width, int height);

//! \brief Sets zoom factor for a camera
//! \param camera Camera handle
//! \param zoom Zoom factor of camera 
//! \ingroup camcommands
XDECL void			XCALL xCameraZoom(Entity camera, float zoom = 1.0f);

//! \brief Sets channel panoram value
//! \details When you want to do real sound panning effects, this is the
//! command you'll use. This will allow you to pan the sound channels on a
//! 'per channel' basis between the left and right speakers. This command
//! makes it very easy to produce some really killer stereophonic effects. 
//! The pan value is between -1 and 1 with 0 being perfect center. -1 is
//! full left, and 1 is full right. To make it somewhere in between, try
//! -0.5 for 50% left or 0.75 for 75% right.
//! \param channel Channel handle
//! \param pan Panning value to denote channel playback
//! \ingroup audiocommands
XDECL void			XCALL xChannelPan(Channel channel, float pan);

//! \brief Sets channel frequency
//! \details You can alter the pitch of a sound channel that is playing
//! (or in use and just paused). Use the frequency of your sound as the
//! 'baseline' for pitch change. So if your sample is at 11025 hertz,
//! increase the pitch to 22050 to make the pitch twice as high, 8000 to
//! make it lower, etc. While similar to xSoundPitch(), this command let's
//! you change the pitch individually of each and every channel in use. 
//! \param channel Channel handle
//! \param hertz Pitch to apply to the channel
//! \ingroup audiocommands
XDECL void			XCALL xChannelPitch(Channel channel, int hertz);

//! \brief Returns true if specified channel is being played
//! \details Often you will need to know if a sound channel has completed
//! playing or not. This command will return 1 if the sound is still playing
//! or 0 if it has stopped. Use this to restart your background music or
//! some other sound that might have stopped unintentionally.
//! \param channel Channel handle
//! \ingroup audiocommands
XDECL bool			XCALL xChannelPlaying(Channel channel);

//! \brief Sets channel volume
//! \details While xSoundVolume() happily changes the volume of the entire
//! program, this command will let you adjust volume rates on a 'per
//! channel' basis. Extremely useful. 
//!  The volume value is a floating point value between 0.0 and 1.0 (0.0f
//! = silence, 0.5 = half volume, 1.0 = full volume).
//! \param channel Channel handle
//! \param volume Volume level floating value between 0.0 and 1.0
//! \ingroup audiocommands
XDECL void			XCALL xChannelVolume(Channel channel, float volume);

//! \brief Clears all passed constansts
//! \details This function deletes all constants passed into effect (by
//! functions like xSetEffectInt(), xSetEffectTexture() etc).
//! \param entity Entity handle
//! \ingroup shadercommands
XDECL void			XCALL xClearEffectConstants(Entity entity);

//! \brief Clears all passed constansts of surface
//! \details This function deletes all constants passed into effect (by
//! functions like xSurfaceEffectInt(), xSurfaceEffectTexture() etc).
//! \param surface Surface handle
//! \ingroup shadercommands
XDECL void			XCALL xClearSurfaceConstants(Surface surface);

//! \deprecated
XDECL void			XCALL xClearPixelShaderConstants(Entity entity);

//! \deprecated
XDECL void			XCALL xClearVertexShaderConstants(Entity entity);

//! \brief Clears rendering buffer
//! \details This command will wipe the current drawing buffer clean of any
//! graphics or text present and reset the drawing buffer back to the color
//! defined in the xClsColor() command
//! \ingroup maincommands
XDECL void			XCALL xCls();

//! \brief Set clear color for xCls() command
//! \details This function changes the color for subsequent xCls() calls. Use this
//! command when you need xCls() to 'clear' the screen with some other color
//! than black. 
//! \param red Red clear color value
//! \param green Green clear color value
//! \param blue Blue clear color value
//! \ingroup maincommands
XDECL void			XCALL xClsColor(int red, int green, int blue);

//! \brief Returns the other entity involved in a particular collision
//! \details Index should be in the range [0; xCountCollisions(entity) - 1]
//! \param entity Entity handle 
//! \param index Index of collision
//! \ingroup ecolcommands
XDECL Entity		XCALL xCollisionEntity(Entity entity, int index);

//! \brief Returns the x component of the normal of a particular collision
//! \details Index should be in the range [0; xCountCollisions(entity) - 1]
//! \param entity Entity handle 
//! \param index Index of collision
//! \ingroup ecolcommands
XDECL float			XCALL xCollisionNX(Entity entity, int index);

//! \brief Returns the y component of the normal of a particular collision
//! \details Index should be in the range [0; xCountCollisions(entity) - 1]
//! \param entity Entity handle 
//! \param index Index of collision
//! \ingroup ecolcommands
XDECL float			XCALL xCollisionNY(Entity entity, int index);

//! \brief Returns the z component of the normal of a particular collision
//! \details Index should be in the range [0; xCountCollisions(entity) - 1]
//! \param entity Entity handle 
//! \param index Index of collision
//! \ingroup ecolcommands
XDECL float			XCALL xCollisionNZ(Entity entity, int index);

//! \brief Returns the handle of the surface belonging to the specified
//! entity that was closest to the point of a particular collision
//! \details Index should be in the range [0; xCountCollisions(entity) - 1]
//! \param entity Entity handle 
//! \param index Index of collision
//! \ingroup ecolcommands
XDECL Surface		XCALL xCollisionSurface(Entity entity, int index);

//! \brief Returns the time taken to calculate a particular collision
//! \details Index should be in the range [0; xCountCollisions(entity) - 1]
//! \param entity Entity handle 
//! \param index Index of collision
//! \ingroup ecolcommands
XDECL int			XCALL xCollisionTime(Entity entity, int index);

//! \brief Returns the world x coordinate of a particular collision
//! \details Index should be in the range [0; xCountCollisions(entity) - 1]
//! \param entity Entity handle 
//! \param index Index of collision
//! \ingroup ecolcommands
XDECL float			XCALL xCollisionX(Entity entity, int index);

//! \brief Returns the world y coordinate of a particular collision
//! \details Index should be in the range [0; xCountCollisions(entity) - 1]
//! \param entity Entity handle 
//! \param index Index of collision
//! \ingroup ecolcommands
XDECL float			XCALL xCollisionY(Entity entity, int index);

//! \brief Returns the world z coordinate of a particular collision
//! \details Index should be in the range [0; xCountCollisions(entity) - 1]
//! \param entity Entity handle 
//! \param index Index of collision
//! \ingroup ecolcommands
XDECL float			XCALL xCollisionZ(Entity entity, int index);

//! \brief Enables collisions between two different entity types
//! \details Entity types are just numbers you assign to an entity using
//! xEntityType(). Then uses the entity types to check for collisions
//! between all the entities that have those entity types. 
//! Where any ways of checking for collisions, as denoted by the method
//! parameter. However, collision checking is always ellipsoid to something.
//! In order to know what size a source entity is, you must first assign an
//! entity radius to all source entities using xEntityRadius(). 
//! In the case of collision detection method SPHERETOSPHERE being selected,
//! then the destination entities concerned will need to have an
//! xEntityRadius() assigned to them too. In the case of method
//! SPHERETOBOX being selected, then the destination entities will need to
//! have an xEntityBox() assigned to them. Method SPHERETOTRIMESH requires
//! nothing to be assigned to the destination entities. 
//! Engine not only check for collisions, but it acts upon them when it
//! detects them too, as denoted by the response parameter. You have three
//! options in this situation. You can either choose to make the source
//! entity stop, slide or only slide upwards. 
//! All collision checking occurs, and collision responses are acted out,
//! when xUpdateWorld() is called. 
//! Finally, every time the xCollisions() command is used, collision
//! information is added to the collision information list. This can be
//! cleared at any time using the xClearCollisions() command.
//! \param srcType Entity type to be checked for collisions
//! \param destType Entity type to be collided with
//! \param method Collision detection method. See '\ref colltypes' for more
//! information
//! \param response What the source entity does when a collision occurs.
//! See '\ref resptypes' for more information
//! \ingroup ecolcommands
XDECL void			XCALL xCollisions(int srcType, int destType, int method = SPHERETOSHPHERE, int response = RESPONSE_STOP);

//! \brief Returns the index number of the triangle belonging to the
//! specified entity that was closest to the point of a particular collision
//! \details Index should be in the range [0; xCountCollisions(entity) - 1]
//! \param entity Entity handle 
//! \param index Index of collision
//! \ingroup ecolcommands
XDECL int			XCALL xCollisionTriangle(Entity entity, int index);

//! \brief This command sets the drawing color (using RGB values) for all
//! subsequent drawing commands (xLine(), xRect(), xText(), etc.)
//! \param red Red value of drawing color
//! \param green Green value of drawing color
//! \param blue Blue value of drawing color
//! \ingroup maincommands
XDECL void			XCALL xColor(int red, int green, int blue);

//! \brief Return the blue component of the RGB color of the current drawing
//! color.
//! \ingroup maincommands
XDECL int			XCALL xColorBlue();

//! \brief Return the green component of the RGB color of the current drawing
//! color.
//! \ingroup maincommands
XDECL int			XCALL xColorGreen();

//! \brief Return the red component of the RGB color of the current drawing
//! color.
//! \ingroup maincommands
XDECL int			XCALL xColorRed();

//! \brief Copies a rectangle of graphics from one buffer to another.
//! \details If a buffer is omitted, the current buffer is used
//! \param sx Source top left x location to begin copying from
//! \param sy Source top left y location to begin copying from 
//! \param swidth Width of source area to copy 
//! \param sheight Height of source area to copy 
//! \param dx Destination top left x location to copy to 
//! \param dy Destination top left y location to copy to 
//! \param srcBuff Handle to the source buffer
//! \param destBuff Handle to the destination buffer
//! \ingroup maincommands
XDECL void			XCALL xCopyRect(int sx, int sy, int swidth, int sheight, int dx, int dy, Buffer srcBuff = -1, Buffer destBuff = -1);

//! \brief Returns the number of children of an entity
//! \param entity Entity handle
//! \ingroup escommands
XDECL int			XCALL xCountChildren(Entity entity);

//! \brief Returns how many collisions an entity was involved in during the
//! last xUpdateWorld()
//! \param entity Entity handle
//! \ingroup ecolcommands
XDECL int			XCALL xCountCollisions(Entity entity);

//! \brief Returns the number of surfaces in a mesh
//! \details Surfaces are sections of mesh. A mesh may contain only one
//! section, or very many
//! \param entity Entity handle
//! \ingroup meshcommands
XDECL int			XCALL xCountSurfaces(Entity entity);

//! \brief Returns the number of triangles in a surface
//! \param surface Surface handle
//! \ingroup surfcommands
XDECL int			XCALL xCountTriangles(Surface surface);

//! \brief Returns the number of vertices in a surface
//! \param surface Surface handle
//! \ingroup surfcommands
XDECL int			XCALL xCountVertices(Surface surface);

//! \brief Returns a number which shows CPU family
//! \ingroup sicommands
XDECL int			XCALL xCPUFamily();

//! \brief Returns a number which shows CPU model
//! \ingroup sicommands
XDECL int			XCALL xCPUModel();

//! \brief Returns a string which shows CPU name
//! \ingroup sicommands
XDECL const char *	XCALL xCPUName();

//! \brief Returns a number which shows CPU frequency in MHz
//! \ingroup sicommands
XDECL int			XCALL xCPUSpeed();

//! \brief Returns a number which shows CPU stepping
//! \ingroup sicommands
XDECL int			XCALL xCPUStepping();

//! \brief Returns a string which shows CPU vendor name
//! \ingroup sicommands
XDECL const char *	XCALL xCPUVendor();

//! \brief Creates 3D line object
//! \param parent Parent entity for line
//! \ingroup meshcommands
XDECL Entity		XCALL xCreate3DLine(Entity parent = NULL);

//! \brief Sets initial and ending points of line
//! \param line Line handle
//! \param x0 Starts x coordinate
//! \param y0 Starts y coordinate
//! \param z0 Starts z coordinate
//! \param x1 Starts x coordinate
//! \param y1 Starts y coordinate
//! \param z1 Starts z coordinate
//! \ingroup meshcommands
XDECL void			XCALL x3DLinePoints(Entity line, float x0, float y0, float z0, float x1, float y1, float z1);

//! \deprecated
XDECL void *		XCALL xCreateBufferMatrix(int index);

//! \deprecated
XDECL void *		XCALL xCreateBufferVectors(int index);

//! \brief Creates a depth-stencil surface
//! \details Depth-stencil surface is used for z-buffer technique. It's used to write
//! scene depth for clipping invisible pixels. By default DSS is used with
//! backbuffer size, but if you want to render to larger textures you must
//! create DSS with an appropriate size. E.g. if you use 800x600 mode, default DSS will be 
//! 800x600 too. And you want to render to 1024x1024 texture (e.g. for shadows),
//! you must call xCreateDSS(1024, 1024) for it.
//! \param width Depth-stencil surface width
//! \param height Depth-stencil surface height
//! \ingroup maincommands
XDECL void			XCALL xCreateDSS(int width, int height);

//! \brief Creates a polygon
//! \param sides If true two-sided polygon will be created
//! \param parent Parent entity handle
//! \ingroup meshcommands
XDECL Entity		XCALL xCreatePoly(bool sides, Entity parent = NULL);

//! \brief Creates a quad for post-processing
//! \param camera Camera handle which this quad be attached to
//! \param mode Quad mode, 1 for screen aligned quad, 0 for simple quad
//! (equals xCreatePoly())
//! \ingroup shadercommands
XDECL Entity		XCALL xCreatePostEffectPoly(Entity camera, int mode = 1);

//! \brief Deletes a depth-stencil surface
//! \ingroup maincommands
XDECL void			XCALL xDeleteDSS();

//! \brief Deletes a specified effect constant
//! \param entity Entity handle
//! \param name Constant name
//! \ingroup shadercommands
XDECL void			XCALL xDeleteEffectConstant(Entity entity, const char * name);

//! \brief Deletes a specified surface's effect constant
//! \param surface Surface handle
//! \param name Constant name
//! \ingroup shadercommands
XDECL void			XCALL xDeleteSurfaceConstant(Surface surface, const char * name);

//! \brief Deletes an allocated array for mesh indecies
//! \param entity Entity handle
//! \ingroup meshcommands
XDECL void			XCALL xDeleteMeshIB(Entity entity);

//! \brief Deletes an allocated array for mesh verticies
//! \param entity Entity handle
//! \ingroup meshcommands
XDECL void			XCALL xDeleteMeshVB(Entity entity);

//! \deprecated
XDECL void			XCALL xDeletePixelShaderConstant(Entity entity, const char * name);

//! \deprecated
XDECL void			XCALL xDeleteVertexShaderConstant(Entity entity, const char * name);

//! \brief Returns the pitch angle, that first entity should be rotated by
//! in order to face second.
//! \details This command can be used to be point one entity at another,
//! rotating on the x axis only
//! \param entity1 First entity handle
//! \param entity2 Second entity handle
//! \ingroup escommands
XDECL float			XCALL xDeltaPitch(Entity entity1, Entity entity2);

//! \brief Returns the yaw angle, that first entity should be rotated by
//! in order to face second.
//! \details This command can be used to be point one entity at another,
//! rotating on the y axis only
//! \param entity1 First entity handle
//! \param entity2 Second entity handle
//! \ingroup escommands
XDECL float			XCALL xDeltaYaw(Entity entity1, Entity entity2);

//! \brief Enables or disables hardware dithering
//! \details Hardware dithering is useful when running games in 16-bit
//! colour mode. Due to the fact that 16-bit mode offers less colours than
//! the human eye can detect, separate bands of colour are often noticeable
//! on shaded objects. However, hardware dithering will dither the entire
//! screen to give the impression that more colours are being used than is
//! actually the case. 
//! Due to the fact that 32-bit offer more colours than the human eye can
//! detect, hardware dithering is made pretty much redundant in this mode
//! \param state True to enable dithering, false to disable
//! \ingroup maincommands
XDECL void			XCALL xDither(bool state);

//! \brief Enables or disables rendering of entitys' bounding boxes
//! \param state True to enable bounding boxes rendering, false to disable
//! \param zenable True to enable z-buffeing when drawing bounding boxes,
//! false to disable
//! \param red Red value of bounding boxes color
//! \param green Green value of bounding boxes color
//! \param blue Blue value of bounding boxes color
//! \param alpha Alpha value of bounding boxes color
//! \ingroup maincommands
XDECL void			XCALL xDrawBBox(bool state, bool zenable, int red, int green, int blue, int alpha);

//! \brief Emits a sound attached to the specified entity and returns a
//! sound channel
//! \details The sound must have been loaded using xLoad3DSound() for 3D
//! effects
//! \param sound Sound handle
//! \param entity Entity handle
//! \ingroup audiocommands
XDECL Channel		XCALL xEmitSound(Sound sound, Entity entity);

//! \brief Enables or disables shaders for an entity
//! \param entity Entity handle
//! \param enable True to enable shaders, false to disable
//! \ingroup shadercommands
XDECL void			XCALL xEnableEntityShader(Entity entity, bool enable);

//! \deprecated
XDECL void			XCALL xEnablePostProcessing(bool state);

//! \brief Sets the alpha level of an entity. 
//! \details The 'alpha' value should be in a floating point value in the
//! range [0.0; 1.0]. The default entity alpha setting is 1.0. 
//! The alpha level is how transparent an entity is. A value of 1.0 will
//! mean the entity is opaque. A value of 0.0 will mean the entity is
//! completely transparent, i.e. invisible. Values between 0.0 and 1.0
//! will cause varying amount of transparency. This is useful for imitating
//! the look of objects such as glass and other translucent materials.
//! \param entity Entity handle
//! \param alpha Alpha level of entity 
//! \ingroup eccommands
XDECL void			XCALL xEntityAlpha(Entity entity, float alpha);

//! \brief Sets the ambient color of an entity 
//! \details The red, green and blue values should be in the range [0; 255]
//! with 0 being darkest and 255 brightest. The default entity ambient
//! color is 255, 255, 255 (white). xEntityColor() also change ambient color
//! \param entity Entity handle
//! \param red Red value of entity ambient color
//! \param green Green value of entity ambient color
//! \param blue Blue value of entity ambient color
//! \ingroup eccommands
XDECL void			XCALL xEntityAmbientColor(Entity entity, float red, float green, float blue);


//! \brief Enables auto fading for an entity.
//! \details This will cause an entity's alpha level to be adjusted at
//! distances between near and far to create a 'fade-in' effect. 
//! \param entity Entity handle
//! \param nearRange Distance in front of the camera at which entity's will
//! start being faded 
//! \param farRange Distance in front of the camera at which entity's will
//! stop being faded (and will be invisible) 
//! \ingroup eccommands
XDECL void			XCALL xEntityAutoFade(Entity entity, float nearRange, float farRange);

//! \brief Sets the blending mode of an entity.
//! \details This blending mode determines the way in which the new RGBA of
//! the pixel being rendered is combined with the RGB of the background. 
//! To calculate the new RGBA of the pixel being rendered, the texture RGBA
//! for the pixel is taken, its alpha component multiplied by the
//! entities/brushes (where applicable) alpha value and its color compentent
//! multiplied by the entities/brushes colour. This is the RGBA which will
//! then be blended into the background pixel, and how this is done depends
//! on the xEntityBlend() value.
//! \param entity Entity handle
//! \param mode Blend mode. See '\ref entblendtypes' for more information
//! \ingroup eccommands
XDECL void			XCALL xEntityBlend(Entity entity, int mode = BLEND_ALPHA);

//! \brief Sets the dimensions of an entity's collision box
//! \param entity Entity handle
//! \param x x position of entity's collision box 
//! \param y x position of entity's collision box 
//! \param z x position of entity's collision box 
//! \param width Width of entity's collision box
//! \param height Height of entity's collision box
//! \param depth Depth of entity's collision box
//! \ingroup ecolcommands
XDECL void			XCALL xEntityBox(Entity entity, float x, float y, float z, float width, float height,float depth);

//! \brief Returns a string containing the class of the specified entity
//! \param entity Entity handle
//! \ingroup escommands
XDECL const char *	XCALL xEntityClass(Entity entity);

//! \brief Sets the diffuse color of an entity. 
//! \details The red, green and blue values should be in the range [0; 255]
//! with 0 being darkest and 255 brightest. The default entity color is
//! 255, 255, 255 (white).
//! \param entity Entity handle
//! \param red Red value of entity color
//! \param green Green value of entity color
//! \param blue Blue value of entity color
//! \ingroup eccommands
XDECL void			XCALL xEntityColor(Entity entity, float red, float green, float blue);

//! \brief Returns the distance between two entities
//! \param entity1 First entity handle
//! \param entity2 Second entity handle
//! \ingroup escommands
XDECL float			XCALL xEntityDistance(Entity entity1, Entity entity2);

//! \brief Sets the emissive color of an entity. 
//! \details The red, green and blue values should be in the range [0; 255]
//! with 0 being darkest and 255 brightest. The default emissive entity
//! color is 0, 0, 0 (black).
//! \param entity Entity handle
//! \param red Red value of entity emissive color
//! \param green Green value of entity emissive color
//! \param blue Blue value of entity emissive color
//! \ingroup eccommands
XDECL void			XCALL xEntityEmissiveColor(Entity entity, float red, float green, float blue);

//! \brief Sets miscellaneous effects for an entity. 
//! \details Flags can be added to combine two or more effects.
//! \param entity Entity handle
//! \param fxFlags Effects flags. See '\ref fxflags' for more information
//! \ingroup eccommands
XDECL void			XCALL xEntityFX(Entity entity, int fxFlags = FX_NOTHING);

//! \brief Returns true if the specified entity is visible to the specified
//! camera. 
//! \details If the entity is a mesh, its bounding box will be checked for
//! visibility. 
//! For all other types of entities, only their centre position will be
//! checked
//! \param entity Entity handle
//! \param camera Camera handle
//! \ingroup camcommands
XDECL bool			XCALL xEntityInView(Entity entity, Entity camera);

//! \brief Returns the name of an entity.
//! An entity's name may be set in a modelling program, or manually set
//! using xNameEntity()
//! \param entity Entity handle
//! \ingroup escommands
XDECL const char *	XCALL xEntityName(Entity entity);

//! \brief Sets the drawing order for an entity. 
//! \details An order value of 0 will mean the entity is drawn normally. A
//! value greater than 0 will mean that entity is drawn first, behind
//! everything else. A value less than 0 will mean the entity is drawn
//! last, in front of everything else. 
//! Setting an entity's order to non-0 also disables z-buffering for the
//! entity, so should be only used for simple, convex entities like
//! skyboxes, sprites etc. 
//! xEntityOrder() affects the specified entity but none of its child
//! entities, if any exist. 
//! \param entity Entity handle
//! \param order Order that entity will be drawn in 
//! \ingroup eccommands
XDECL void			XCALL xEntityOrder(Entity entity, int order = 0);

//! \brief Returns the nearest entity 'ahead' of the specified entity.
//! \details An entity must have a non-zero xEntityPickMode() to be pickable. 
//! \param entity Entity handle
//! \param range Range of pick area around entity 
//! \ingroup escommands
XDECL int 			XCALL xEntityPick(Entity entity, float range);

//! \brief Sets the radius of an entity's collision sphere. 
//! \details An entity radius should be set for all entities involved in
//! spherical collisions, which is all source entities (as collisions are
//! always sphere-to-something), and whatever destination entities are
//! involved in sphere-to-sphere collisions 
//! \param entity Entity handle
//! \param radius Radius of entity's collision sphere 
//! \ingroup ecolcommands
XDECL void			XCALL xEntityRadius(Entity entity, float radius);

//! \brief Returns true if entity shaders is enabled
//! \param entity Entity handle
//! \ingroup escommands
XDECL bool			XCALL xEntityShader(Entity entity);

//! \brief Sets the specular color of an entity. 
//! \details The red, green and blue values should be in the range [0; 255]
//! with 0 being darkest and 255 brightest. The default specular entity
//! color is 0, 0, 0 (black). xEntityShininess() also change specular color.
//! \param entity Entity handle
//! \param red Red value of entity specular color
//! \param green Green value of entity specular color
//! \param blue Blue value of entity specular color
//! \ingroup eccommands
XDECL void			XCALL xEntitySpecularColor(Entity entity,float red, float green, float blue);

//! \brief Sets the collision type for an entity. 
//! A collision type value of 0 indicates that no collision checking will
//! occur with that entity. A collision value of [1; 999] will mean
//! collision checking will occur. 
//! \param entity Entity handle
//! \param cType Collision type of entity. Must be in the range [0; 999]
//! \param recurse True to apply collision type to entity's children
//! \ingroup ecolcommands
XDECL void			XCALL xEntityType(Entity entity, int cType, bool recurse = false);

//! \brief Returns the first child of the specified entity with matching name
//! \param entity Entity handle
//! \param name Child name to find within entity 
//! \ingroup escommands
XDECL Entity		XCALL xFindChild(Entity entity, const char * name);

//! \brief Attempts to find a surface attached to the specified mesh and
//! created with the specified brush.
//! \details Returns the surface handle if found or 0 if not. 
//! \param entity Entity handle
//! \param brush Brush handle
//! \ingroup meshcommands
XDECL Surface		XCALL xFindSurface(Entity entity, Brush brush);

//! \brief Switches the front buffer and back buffer. 
//! \details This command should be used when you are using double buffering.
//! Double buffering is a technique used to ensure that screen updates are
//! not visible to the user. If you draw directly to the front buffer, the
//! display may appear flickery as the updates are drawn directly to the
//! screen. If you draw to the back buffer, the updates are drawn in memory.
//! xFlip() is then used to make the back buffer the front buffer, and
//! hence show the updates on screen in one go. At the same time, the
//! front buffer becomes the back buffer, allowing you to draw the next
//! screen update on the back buffer before flipping again.
//! \ingroup maincommands
XDECL void			XCALL xFlip();

//! \brief Flips all the triangles in a mesh.
//! \details This is useful for a couple of reasons. Firstly though, it is
//! important to understand a little bit of the theory behind 3D graphics.
//! A 3D triangle is represented by three points; only when these points
//! are presented to the viewer in a clockwise-fashion is the triangle
//! visible. So really, triangles only have one side. 
//! Normally, for example in the case of a sphere, a model's triangles face
//! the inside of the model, so it doesn't matter that you can't see them.
//! However, what about if you wanted to use the sphere as a huge sky for
//! your world, i.e. so you only needed to see the inside? In this case you
//! would just use xFlipMesh(). 
//! Another use for xFlipMesh() is to make objects two-sided, so you can
//! see them from the inside and outside if you can't already. In this
//! case, you can copy the original mesh using xCopyEntity(), specifying
//! the original mesh as the parent, and flip it using xFlipMesh(). You
//! will now have two meshes occupying the same space - this will make it
//! double-sided, but beware, it will also double the polygon count.
//! The above technique is worth trying when an external modelling program
//! has exported a model in such a way that some of the triangles appear
//! to be missing.
//! \param entity Entity handle
//! \ingroup meshcommands
XDECL void			XCALL xFlipMesh(Entity entity);

//! \brief Returns the height, in pixels, of the currently selected font
//! \details Use currently setted font by xSetFont() command.
//! \ingroup textcommands
XDECL int			XCALL xFontHeight();

//! \brief Returns the width, in pixels, of the currently selected font
//! \details Use currently setted font by xSetFont() command.
//! \ingroup textcommands
XDECL int			XCALL xFontWidth();

//! \brief Frees up a brush
//! \param brush Brush handle
//! \ingroup brushcommands
XDECL void			XCALL xFreeBrush(Brush brush);

//! \brief Frees up an effect
//! \param effect Effect handle
//! \ingroup shadercommands
XDECL void			XCALL xFreeEffect(Effect effect);

//! \brief Frees up an entity
//! \param entity Entity handle
//! \ingroup eccommands
XDECL void			XCALL xFreeEntity(Entity entity);

//! \brief Frees up a font
//! \param font Font handle
//! \ingroup textcommands
XDECL void			XCALL xFreeFont(Font font);

//! \brief Frees up an image
//! \param image Image handle
//! \ingroup imagecommands
XDECL void			XCALL xFreeImage(Image image);

//! \brief Frees up a particle
//! \param entity Particle system entity handle
//! \param pid Particle index
//! \ingroup psyscommands
XDECL void			XCALL xFreeParticle(Entity  entity, int pid);

//! \deprecated
XDECL void			XCALL xFreePostEffect(PostEffect effect);

//! \brief Frees up a sound
//! \param sound Sound handle
//! \ingroup audiocommands
XDECL void			XCALL xFreeSound(Sound sound);

//! \brief Frees up a texture
//! \param texture Texture handle
//! \ingroup texcommands
XDECL void			XCALL xFreeTexture(Texture texture);

//! \brief Returns an entity's alpha testing function
//! \param entity Entity handle
//! \ingroup escommands
XDECL int			XCALL xGetAlphaFunc(Entity entity);

//! \brief Returns an entity's alpha testing reference
//! \param entity Entity handle
//! \ingroup escommands
XDECL int			XCALL xGetAlphaRef(Entity entity);

//! \brief Returns amount of available page memory in KB (i.e. virtual memory, swap, pagefile)
//! \ingroup sicommands
XDECL float			XCALL xGetAvailPageMem();

//! \brief Returns amount of available physical memory in KB
//! \ingroup sicommands
XDECL float			XCALL xGetAvailPhysMem();

//! \brief Returns amount of available local video memory in KB (i.e. onboard memory)
//! \ingroup sicommands
XDECL float			XCALL xGetAvailVidLocalMem();

//! \brief Returns amount of available video memory in KB (i.e. local + nonlocal)
//! \ingroup sicommands
XDECL float			XCALL xGetAvailVidMem();

//! \brief Returns amount of available nonlocal video memory in KB (i.e. AGP memory)
//! \ingroup sicommands
XDECL float			XCALL xGetAvailVidNonlocalMem();

//! \brief Returns a child of an entity 
//! \param entity Entity handle
//! \param index Index of child entity. Should be in the range
//! [0; xCountChildren(entity) - 1] 
//! \ingroup escommands
XDECL Entity		XCALL xGetChild(Entity entity, int index);

//! \brief This command works like a 'color picker' in paint program.
//! \param x x coordinate of pixel 
//! \param y y coordinate of pixel 
//! \ingroup maincommands
XDECL int			XCALL xGetColor(int x, int y);

//! \brief Returns a pointer to Direct3D device used by Xors3D
//! \ingroup maincommands
XDECL void *		XCALL xGetDevice();

//! \brief Returns an entity's alpha value
//! \param entity Entity handle
//! \ingroup escommands
XDECL float			XCALL xGetEntityAlpha(Entity entity);

//! \brief Returns an entity's brush
//! \param entity Entity handle
//! \ingroup escommands
XDECL Brush			XCALL xGetEntityBrush(Entity entity);

//! \brief Returns a pointer entity's world matrix
//! \param entity Entity handle
//! \ingroup escommands
XDECL void *		XCALL xGetEntityMatrix(Entity entity);

//! \brief Returns the value of an element from within an entity's world matrix
//! \param entity Entity handle
//! \param row Matrix row index 
//! \param cell Matrix cell index 
//! \ingroup escommands
XDECL float			XCALL xGetEntityMatrixElement(Entity entity, int row, int cell);

//! \brief Returns entity collision type
//! \param entity Entity handle
//! \ingroup escommands
XDECL int			XCALL xGetEntityType(Entity entity);

//! \brief Returns FPS (frames per second) counter
//! \ingroup maincommands
XDECL int			XCALL xGetFPS();

//! \brief This command will check to see if a key has been pressed and
//! will return its ASCII value
//! \ingroup keycommands
XDECL int			XCALL xGetKey();

//! \brief Returns the value of an element from within an entity's world matrix
//! \param entity Entity handle
//! \param row Matrix row index 
//! \param cell Matrix cell index 
//! \ingroup escommands
XDECL float			XCALL xGetMatElement(Entity entity, int row, int cell);

//! \brief Returns max supported antialiasing level
//! \ingroup maincommands
XDECL int			XCALL xGetMaxAntiAlias();

//! \deprecated
XDECL int			XCALL xGetMaxPixelShaderVersion();

//! \deprecated
XDECL int			XCALL xGetMaxVertexShaderVersion();

//! \brief Returns a pointer to mesh indeices
//! \details Actually it's only a copy of original index buffer, you can use it
//! for phisics engines, such as PhysX for making trimesh and so one. You
//! must delete created buffers when it's no more needed by xDeteleMeshIB()
//! \param entity Entity handle
//! \ingroup meshcommands
XDECL int *			XCALL xGetMeshIB(Entity entity);

//! \brief Returns a size of mesh indeices buffer
//! \param entity Entity handle
//! \ingroup meshcommands
XDECL int			XCALL xGetMeshIBSize(Entity entity);

//! \brief Returns pointer to mesh vertices
//! \details Actually it's a only copy of original vertex buffer, you can use it
//! for phisics engines, such as PhysX for making trimesh and so one. You
//! must delete created buffers when it's no more needed by xDeteleMeshVB()
//! \param entity Entity handle
//! \ingroup meshcommands
XDECL float *		XCALL xGetMeshVB(Entity entity);

//! \brief Returns a size of mesh vertices buffer
//! \param entity Entity handle
//! \ingroup meshcommands
XDECL int			XCALL xGetMeshVBSize(Entity entity);

//! \brief Returns a parent of an entity
//! \param entity Entity handle
//! \ingroup escommands
XDECL Entity		XCALL xGetParent(Entity entity);

//! \brief Returns a pointer to Xors3D's rendering window
//! \ingroup maincommands
XDECL void *		XCALL xGetRenderWindow();

//! \brief Returns the handle of the surface attached to the specified mesh
//! and with the specified index number. 
//! \details Index should be in the range [0; CountSurfaces(entity) - 1] 
//! You need to 'get a surface', i.e. get its handle, in order to be able
//! to then use that particular surface with other commands.
//! \param entity entity handle
//! \param index Index of surface 
//! \ingroup meshcommands
XDECL Surface		XCALL xGetSurface(Entity entity, int index);

//! \brief Returns a brush with the same properties as is applied to the
//! specified mesh surface. 
//! \details If this command does not appear to be returning a valid brush,
//! try using xGetEntityBrush() instead. 
//! Remember, xGetSurfaceBrush() actually creates a new brush so don't
//! forget to free it afterwards using xFreeBrush() to prevent memory leaks. 
//! Once you have got the brush handle from a surface, you can use
//! xGetBrushTexture() and xTextureName() to get the details of what
//! texture(s) are applied to the brush. 
//! \param surface Surface handle
//! \ingroup surfcommands
XDECL Brush			XCALL xGetSurfaceBrush(Surface surface);

//! \brief Return surface texture from specified index
//! \param surface Surface handle
//! \param index Texture layer index in range [0; 7]
//! \ingroup surfcommands
XDECL Texture		XCALL xGetSurfaceTexture(Surface surface, int index);

//! \brief Returns a count of texture frames
//! \param texture Texture handle
//! \ingroup texcommands
XDECL int			XCALL xGetTextureFrames(Texture texture);

//! \brief Returns a pointer to Direct3D texture surface
//! \param texture Texture handle
//! \param frame Frame index
//! \ingroup texcommands
XDECL void *		XCALL xGetTextureSurface(Texture texture, int frame = 0);

//! \brief Returns amount of total page memory in KB (i.e. virtual memory, swap, pagefile)
//! \ingroup sicommands
XDECL float			XCALL xGetTotalPageMem();

//! \brief Returns amount of total physical memory in KB
//! \ingroup sicommands
XDECL float			XCALL xGetTotalPhysMem();

//! \brief Returns amount of total local video memory in KB (i.e. onboard memory)
//! \ingroup sicommands
XDECL float			XCALL xGetTotalVidLocalMem();

//! \brief Returns amount of total video memory in KB (i.e. local + nonlocal)
//! \ingroup sicommands
XDECL float			XCALL xGetTotalVidMem();

//! \brief Returns amount of total nonlocal video memory in KB (i.e. AGP memory)
//! \ingroup sicommands
XDECL float			XCALL xGetTotalVidNonlocalMem();

//! \brief Returns a pointer to camera's projection matrix
//! \param camera Camera handle
//! \ingroup camcommands
XDECL void *		XCALL xGetProjectionMatrix(Entity camera);

//! \brief Returns a pointer to camera's view matrix
//! \param camera Camera handle
//! \ingroup camcommands
XDECL void *		XCALL xGetViewMatrix(Entity camera);

//! \brief Returns handle of current rendering buffer
//! \ingroup maincommands
XDECL Buffer		XCALL xGraphicsBuffer();

//! \brief Returns depth of current rendering buffer
//! \ingroup maincommands
XDECL int			XCALL xGraphicsDepth();

//! \brief Returns height of current rendering buffer
//! \ingroup maincommands
XDECL int			XCALL xGraphicsHeight();

//! \brief Returns width of current rendering buffer
//! \ingroup maincommands
XDECL int			XCALL xGraphicsWidth();

//! \brief Sets image drawing handle position
//! \details When an image is loaded with xLoadImage(), the image handle (the
//! location within the image where the image is 'drawn from') is always
//! defaulted to the top left corner (coordinates 0, 0). This means if you
//! draw an image that is 50x50 pixels at screen location 200, 200, the
//! image will begin to be drawn at 200, 200 and extend to 250, 250. 
//! This command moves the image handle from the 0, 0 coordinate of the
//! image to the specified x and y location in the image. You can retrieve
//! an image's current location handle using the xImageXHandle() and
//! xImageYHandle(). Finally, you can make all images automatically load
//! with the image handle set to middle using the xAutoMidHandle() command. 
//! \param image Image handle
//! \param x x coordinate of the new image drawing handle location 
//! \param y y coordinate of the new image drawing handle location 
//! \ingroup imagecommands
XDECL void			XCALL xHandleImage(Image image, int x, int y);

//! \brief Sets a sprite handle. Defaults to 0, 0. 
//! \details By default sprite extends from -1, -1 to +1, +1. 
//! \param entity Sprite handle
//! \param x x coordinate of the new sprite drawing handle location 
//! \param y y coordinate of the new sprite drawing handle location 
//! \ingroup sprcommands
XDECL void			XCALL xHandleSprite(Entity entity, float x, float y);

//! \brief Hides an entity, so that it is no longer visible, and is no
//! longer involved in collisions. 
//! \details The main purpose of hide entity is to allow you to create
//! entities at the beginning of a program, hide them, then copy them and
//! show as necessary in the main game. This is more efficient than creating
//! entities mid-game. 
//! If you wish to hide an entity so that it is no longer visible but still
//! involved in collisions, then use EntityAlpha 0 instead. This will make
//! an entity completely transparent. 
//! xHideEntity() affects the specified entity and all of its child
//! entities, if any exist.
//! \param entity Entity handle
//! \ingroup eccommands
XDECL void			XCALL xHideEntity(Entity entity);

//! \brief Hides mouse pointer
//! \details xHidePointer() is for use in windowed display modes, and
//! simply hides the Windows pointer when it is moved over your game's
//! window. You can bring it back via xShowPointer(). It has no effect in
//! full-screen modes
//! \ingroup maincommands
XDECL void			XCALL xHidePointer();

//! \brief Returns the height of the given image in pixels. 
//! \param image Image handle
//! \ingroup imagecommands
XDECL int			XCALL xImageHeight(Image image);

//! \brief Checks if image is collided with a rectangle on the screen
//! \details There are many times when you need to see if an image has
//! collided with (or is touching) a specific rectangular area of the
//! screen. This command performs pixel perfect accurate collision
//! detection between the image of your choice and a specified rectangle on 
//! the screen. 
//! Howevever, should your program just need to detect a graphic (like a
//! mouse pointer) over at a particular location/region of the screen
//! (often called a 'hot spot'), this command works great.
//! As with any collision, you will need to know the PRECISE location of
//! the graphic you wish to test collision with, as well as the x, y,
//! width, and height of the screen area (rect) you wish to test.
//! \param image Image handle
//! \param x Image's x location
//! \param y Image's y location
//! \param frame Image's frame
//! \param rectx x location start of the rect
//! \param recty y location start of the rect 
//! \param rectWidth Width of the rect 
//! \param rectHeight Height of the rect 
//! \ingroup imagecommands
XDECL bool			XCALL xImageRectCollide(Image image, int x, int y, int frame, int rectx, int recty, int rectWidth, int rectHeight);

//! \brief Checks if image is overlapped with a rectangle on the screen
//! \param image Image handle
//! \param x Image's x location
//! \param y Image's y location
//! \param rectx x location start of the rect
//! \param recty y location start of the rect 
//! \param rectWidth Width of the rect 
//! \param rectHeight Height of the rect 
//! \ingroup imagecommands
XDECL bool			XCALL xImageRectOverlap(Image image, int x, int y, int rectx, int recty, int rectWidth, int rectHeight);

//! \brief Returns the width of the given image in pixels
//! \param image Image handle
//! \ingroup imagecommands
XDECL int			XCALL xImageWidth(Image image);

//! \brief Returns x location of an image's drawing handle
//! \param image Image handle
//! \ingroup imagecommands
XDECL int			XCALL xImageXHandle(Image image);

//! \brief Returns y location of an image's drawing handle
//! \param image Image handle
//! \ingroup imagecommands
XDECL int			XCALL xImageYHandle(Image image);

//! \brief Checks if images are collided
//! \details This is the command to get pixel-perfect collisions between
//! images. It will not consider transparent pixels during the collision
//! check (basically, only the 'meat' of the image will invoke a collision).
//! This makes it perfect for most situations where you have odd-shaped
//! graphics to text against. 
//! The xImagesOverlap() command is mesh faster, however, but can only
//! determine if any of the two images have overlapped (this includes
//! transparent pixels). This method works if you have graphics that
//! completely fill their container and/or you don't plan on needing
//! pinpoint accuracy. 
//! \param image1 First image handle
//! \param x1 First image x location
//! \param y1 First image y location
//! \param frame1 First image frame
//! \param image2 Second image handle
//! \param x2 Second image x location
//! \param y2 Second image y location
//! \param frame2 Second image frame
//! \ingroup imagecommands
XDECL bool			XCALL xImagesCollide(Image image1, int x1, int y1, int frame1, Image image2, int x2, int y2, int frame2);

//! \brief Checks if images are overlapped
//! \details This is a very fast, simple collision type command that will
//! allow you to determine whether or not two images have overlapped each
//! other. This does not take into account any transparent pixels (see 
//! xImagesCollide()). 
//! In many cases, you might be able to get away with using this more
//! crude, yet quite fast method of collision detection. For games where
//! your graphics are very squared off and pixel-perfect accuracy isn't a
//! must, you can employ this command to do quick and dirty overlap checking. 
//! \param image1 First image handle
//! \param x1 First image x location
//! \param y1 First image y location
//! \param image2 Second image handle
//! \param x2 Second image x location
//! \param y2 Second image y location
//! \ingroup imagecommands
XDECL bool			XCALL xImagesOverlap(Image image1, int x1, int y1, Image image2, int x2, int y2);

//! \brief Activates full Xors3D version
//! \param key License key of Xors3D
//! \ingroup maincommands
XDECL void			XCALL xKey(const char * key);

//! \brief Checks if specified key is pressed
//! \details This command (similar to its counterparts xMouseDown() and 
//! xJoyDown()) is used to detect if a key is being held down. This command
//! returns a 0 if the key is not held down, a 1 if the key is held down. 
//! \param key Corresponding key scancode 
//! \ingroup keycommands
XDECL int			XCALL xKeyDown(int key);

//! \brief Returns the number of times a specified key has been hit since
//! the last time you called the xKeyHit() command
//! \param key Corresponding key scancode 
//! \ingroup keycommands
XDECL int			XCALL xKeyHit(int key);

//! \brief Checks if specified key is unpressed
//! \details This command returns 1 if the key is not held down, 0 if
//! the key is held down.
//! \param key Corresponding key scancode 
//! \ingroup keycommands
XDECL int			XCALL xKeyUp(int key);

//! \brief Sets the color of a light.
//! \param light Light source handle
//! \param red Red alue of light color
//! \param green Green alue of light color
//! \param blue Blue alue of light color
//! \ingroup lightcommands
XDECL void			XCALL xLightColor(Entity light, float red, float green, float blue);

//! \brief Sets the cone angle for a spot light. 
//! \details The default light cone angles setting is 0, 90. Affect only for
//! spot light sources.
//! \param light Light source handle
//! \param inner Inner angle of cone
//! \param outer Outer angle of cone
//! \ingroup lightcommands
XDECL void			XCALL xLightConeAngles(Entity light, float inner, float outer);

//! \brief Sets the range of a light. 
//! \details The range of a light is how far it reaches. Everything outside
//! the range of the light will not be affected by it. 
//! The value is very approximate, and should be experimented with for best
//! results. Affect only for spot and point light sources.
//! \param light Light source handle
//! \param range Range of light
//! \ingroup lightcommands
XDECL void			XCALL xLightRange(Entity light, float range);

//! \brief Draws a line, in the current drawing color
//! \param x1 Starting x location of the line 
//! \param y1 Starting y location of the line 
//! \param x2 Ending x location of the line 
//! \param y2 Ending y location of the line
//! \ingroup maincommands
XDECL void			XCALL xLine(int x1, int y1, int x2, int y2);

//! \brief Draws a 3D line
//! \param x1 Starting x location of the line 
//! \param y1 Starting y location of the line 
//! \param z1 Starting z location of the line 
//! \param x2 Ending x location of the line 
//! \param y2 Ending y location of the line
//! \param z2 Ending z location of the line
//! \param red Red value of drawing color
//! \param green Green value of drawing color
//! \param blue Blue value of drawing color
//! \param alpha Alpha value of drawing color
//! \param zenable Enable or disable z-buffering for line
//! \ingroup maincommands
XDECL void			XCALL xLine3D(int x1, int y1, int z1, int x2, int y2, int z2, int red, int green, int blue, int alpha, bool zenable = true);

//! \brief Loads a sound and returns its handle for use with xEmitSound()
//! \param path Filename of sound file to be loaded and used as 3D sound 
//! \ingroup audiocommands
XDECL Sound			XCALL xLoad3DSound(const char * path);

//! \brief Load's and animated image
//! \details While similar to xLoadImage(), the xLoadAnimImage() loads a
//! single image that is made up of 'frames' of seperate images (presumably
//! to be used as frames of a graphic animation). 
//! The imagestrip itself consists of 2 or more frames, arranged in a single
//! graphic image. There is no spaces between the frames, and each frame
//! must be the same width and height. When loaded, the frames will be
//! indexed in a left-to-right, top-to-bottom fashion, starting in the top
//! left corner.
//! When drawing the image to the screen with the xDrawImage() command, you
//! specify which frame to draw with the frame parameter. 
//! To actually make your image animate, you'll need to cycle through the
//! frames (like a flip book, cartoon, or any other video) quickly to give
//! the illusion of motion. Our example will show you how to use one of the
//! sample imagestrips and make it animate. While it may seem confusing, we
//! are going to do some timer work as well as a little weird math. 
//! \param path String designating full path and filename to image
//! \param width Width in pixels of each frame in the image
//! \param height Height in pixels of each frame in the image
//! \param startFrame The frame to start with (usually 0) 
//! \param frames How many frames you are using of the imagestrip 
//! \ingroup imagecommands
XDECL Image			XCALL xLoadAnimImage(const char * path, int width, int height, int startFrame, int frames);

//! \brief Loads an animated texture
//! \details The 'flags' parameter allows you to apply certain effects to the
//! texture. Flags can be added to combine two or more effects. See '\ref 
//! tlflags' for more infromation.
//! \param path Name of image file
//! \param flags Loading flags
//! \param width Width in pixels of each frame in the texture
//! \param height Height in pixels of each frame in the texture
//! \param startFrame The frame to start with (usually 0) 
//! \param frames How many frames you are using of the imagestrip 
//! \ingroup texcommands
XDECL Texture		XCALL xLoadAnimTexture(const char * path, int flags, int width, int height, int startFrame, int frames);

//! \brief Loads and image into specified buffer
//! \details There are a hundred and one uses for this command, but
//! probably most often used would be to display a title screen or some
//! other 'one time viewing only' image to the back buffer 
//! \param buffer Buffer handle (xbackBuffer(), xImageBuffer() or xTextureBuffer())
//! \param path String containing filename of image 
//! \ingroup maincommands
XDECL void			XCALL xLoadBuffer(Buffer buffer, const char * path);

//! \brief Loads a static .x files
//! \details Loads a static .x file as optimized geomentry. You can't use shaders
//! or other things with this meshes but it may work more faster than xLoadMesh()
//! \param path String containing filename of mesh file
//! \param opimizeType Mesh optimization type. See '\ref xopttypes' for
//! more information
//! \param parent Parent entity handle
//! \ingroup meshcommands
XDECL Entity		XCALL xLoadDXmesh(const char * path, int opimizeType, Entity parent = NULL);

//! \brief Loads an effect from file
//! \details Xors3D supports DirectX effects files (it uses HLSL as shaders
//! language), it's more powerful and easy to use than regular shaders, you
//! can change render states manually, make multi-pass effects and much more.
//! \param path String containing filename of effect file
//! \ingroup shadercommands
XDECL Effect		XCALL xLoadFXFile(const char * path);

//! \brief This command loads an image
//! \details You will use the xDrawImage() command to display the graphic later.
//! \param path String containing filename of imge file
//! \ingroup imagecommands
XDECL Image			XCALL xLoadImage(const char * path);

// ?!
XDECL Entity		XCALL xLoadMeshAsync(const char * path, Entity parent = NULL);

// ?!
XDECL void			XCALL xLoadMeshInThread(const char * path, Entity parent = NULL);

//! \brief Loads a static mesh with saving hierarchy
//! \param path String containing filename of mesh file
//! \param parent Parent entity handle
//! \ingroup meshcommands
XDECL Entity		XCALL xLoadMeshWithChild(const char * path, Entity parent = NULL);

//! \deprecated
XDECL PixelShader	XCALL xLoadPixelShader(const char * path, const char * entryPoint, int version = PS_2_0);

//! \deprecated
XDECL PostEffect	XCALL xLoadPostEffect(const char * path, const char * entryPoint, int version = PS_2_0);

//! \brief Loads a sound file into memory
//! \param path Name of sound file. Formats supported: raw/wav/mp3/ogg 
//! \ingroup audiocommands
XDECL Sound			XCALL xLoadSound(const char * path);

//! \deprecated
XDECL VertexShader	XCALL xLoadVertexShader(const char * path, const char * entryPoint, int version =  VS_2_0);

//! \brief Sets up play back a sound file in an endless loop (like for
//! background music)
//! \details This command doesn't actually play the sound loop, just sets
//! it up for looping. You still need to execute the xPlaySound() command
//! to hear the sound
//! \param sound Sound handle
//! \ingroup audiocommands
XDECL void			XCALL xLoopSound(Entity sound);

//! \brief Returns true if Direct3D device lost
//! \ingroup maincommands
XDECL bool			XCALL xLostDevice();

//! \brief Converts an entity with several surfaces into entity
//! with a single surface. All textures of this entities became ignorable.
//! \param entity Entity handle
//! \ingroup surfcommands
XDECL Entity		XCALL xMakeSingleSurface(Entity entity);

//! \brief Returns max number of suppoted clip planes
//! \ingroup maincommands
XDECL int			XCALL xMaxClipPlanes();

//! \brief Returns the depth of a mesh.
//! \details This is calculated by the actual vertex positions and so the
//! scale of the entity (set by xScaleEntity()) will not have an effect on
//! the resultant depth. Mesh operations, on the other hand, will effect the
//! result.
//! \param entity Entity handle
//! \ingroup meshcommands
XDECL float			XCALL xMeshDepth(Entity entity);

//! \brief Returns the height of a mesh.
//! \details This is calculated by the actual vertex positions and so the
//! scale of the entity (set by xScaleEntity()) will not have an effect on
//! the resultant height. Mesh operations, on the other hand, will effect the
//! result.
//! \param entity Entity handle
//! \ingroup meshcommands
XDECL float			XCALL xMeshHeight(Entity entity);

//! \brief Returns the width of a mesh.
//! \details This is calculated by the actual vertex positions and so the
//! scale of the entity (set by xScaleEntity()) will not have an effect on
//! the resultant width. Mesh operations, on the other hand, will effect the
//! result.
//! \param entity Entity handle
//! \ingroup meshcommands
XDECL float			XCALL xMeshWidth(Entity entity);

//! \brief Returns true if the specified meshes are currently intersecting
//! \param entity1 First entity handle
//! \param entity2 Second entity handle
//! \ingroup meshcommands
XDECL bool			XCALL xMeshesIntersect(Entity entity1, Entity entity2);

//! \brief Centers image drawing handle
//! \details When an image is loaded with xLoadImage(), the image handle
//! (the location within the image where the image is 'drawn from') is
//! always defaulted to the top left corner (coordinates 0, 0). This means
//! if you draw an image that is 50x50 pixels at screen location 200, 200,
//! the image will begin to be drawn at 200, 200 and extend to 250, 250. 
//! This command moves the image handle from the 0, 0 coordinate of the
//! image to the exact middle of the image. Therefore, in the same scenario
//! above, if you were to draw a 50x50 pixel image at screen location 200, 
//! 200 with its image handle set to Mid with this command, the image would
//! start drawing at 175, 175 and extend to 225, 225. 
//! You can manual set the location of the image's handle using the
//! xHandleImage() command. You can retrieve an image's handle using the
//! xImageXHandle() and xImageYHandle(). Finally, you can make all images
//! automatically load with the image handle set to middle using the
//! xAutoMidHandle() command. 
//! \param image Image handle
//! \ingroup imagecommands
XDECL void			XCALL xMidHandle(Image image);

//! \brief Detects if a mouse button is being held down
//! \details You must check for each mouse button independantly with its
//! corresponding number. Also see xMouseHit().
//! \param key Mouse button for check
//! \ingroup mousecommands
XDECL int			XCALL xMouseDown(int key);

//! \brief Returns the number of times a specified mouse button has been
//! hit since the last time you called the xMouseHit() command
//! \param key Mouse button for check
//! \ingroup mousecommands
XDECL int			XCALL xMouseHit(int key);

//! \brief Returns the X location of the mouse on the screen
//! \details This position is always from the range [0; xGraphicsWidth() - 1].
//! You can use this command in combination with xDrawImage() to make a
//! custom mouse pointer, or to control something on the screen directly
//! with the mouse.
//! \ingroup mousecommands
XDECL int			XCALL xMouseX();

//! \brief Returns mouse movement on x-axis since last call
//! \ingroup mousecommands
XDECL int			XCALL xMouseXSpeed();

//! \brief Returns the Y location of the mouse on the screen
//! \details This position is always from the range [0; xGraphicsHeight() - 1].
//! You can use this command in combination with xDrawImage() to make a
//! custom mouse pointer, or to control something on the screen directly
//! with the mouse.
//! \ingroup mousecommands
XDECL int			XCALL xMouseY();

//! \brief Returns mouse movement on y-axis since last call
//! \ingroup mousecommands
XDECL int			XCALL xMouseYSpeed();

//! \brief Returns the current position of the mouse wheel on a suitable mouse.
//! It starts off at zero when your program begins. The value of xMouseZ()
//! increases as you move the wheel away from you and decreases as you move
//! it towards you.
//! \ingroup mousecommands
XDECL int			XCALL xMouseZ();

//! \brief Returns mouse movement on z-axis since last call
//! \ingroup mousecommands
XDECL int			XCALL xMouseZSpeed();

//! \brief Moves the mouse to a designated location
//! \details Although the mouse isn't visible on the screen, the mouse
//! location is still being tracked and you can attach a graphic to it.
//! However, there are times when you want to put the pointer to a specific
//! location on the screen.
//! \param x The x coordinate on the screen to move the mouse 
//! \param y The y coordinate on the screen to move the mouse 
//! \ingroup mousecommands
XDECL void			XCALL xMoveMouse(int x, int y);

//! \brief Moves particle (in local particle system's coordinates)
//! \param entity Particle system handle
//! \param index Particle's index
//! \param x New x coordinate of particle
//! \param y New y coordinate of particle
//! \param z New z coordinate of particle
//! \ingroup psyscommands
XDECL void			XCALL xMoveParticle(Entity entity, int index, float x, float y, float z);

//! \brief Sets an entity's name
//! \param entity Entity handle
//! \param name Name of entity 
//! \ingroup eccommands
XDECL void			XCALL xNameEntity(Entity entity, const char * name);

//! \brief Sets a point of origin for all subsequent drawing commands
//! \param x x offset value
//! \param y y offset value
//! \ingroup maincommands
XDECL void			XCALL xOrigin(int x, int y);

//! \brief Paints a entity with a brush. 
//! \details The reason for using xPaintEntity() to apply specific
//! properties to a entity using a brush rather than just using 
//! xEntityTexture(), xEntityColor(), xEntityShininess() etc, is that you
//! can pre-define one brush, and then paint entities over and over again
//! using just the one command rather than lots of separate ones. 
//! \param entity Entity handle
//! \param brush Brush handle
//! \ingroup eccommands
XDECL void			XCALL xPaintEntity(Entity entity, Brush brush);

//! \brief Paints a mesh with a brush. 
//! \details This has the effect of instantly altering the visible
//! appearance of the mesh, assuming the brush's properties are different
//! to what was was applied to the surface before. 
//! The reason for using xPaintMesh() to apply specific properties to a
//! mesh using a brush rather than just using xEntityTexture(), xEntityColor(),
//! xEntityShininess() etc, is that you can pre-define one brush, and then
//! paint meshes over and over again using just the one command rather than
//! lots of separate ones.
//! \param entity Entity handle
//! \param brush Brush handle
//! \ingroup meshcommands
XDECL void			XCALL xPaintMesh(Entity entity, Brush brush);

//! \brief Paints a surface with a brush. 
//! \details This has the effect of instantly altering the visible appearance
//! of that particular surface, i.e. section of mesh, assuming the brush's
//! properties are different to what was applied to the surface before.
//! \param surface Surface handle
//! \param brush Brush handle
//! \ingroup surfcommands
XDECL void			XCALL xPaintSurface(Surface surface, Brush brush);

//! \brief Returns particle's acceleration
//! \param entity Particle system handle
//! \param index Particle's index
//! \ingroup psyscommands
XDECL float			XCALL xParticleAcceleration(Entity entity, int index);

//! \brief Returns particle's age
//! \param entity Particle system handle
//! \param index Particle's index
//! \ingroup psyscommands
XDECL int			XCALL xParticleAge(Entity entity, int index);

//! \brief Returns the blue value of particle's color
//! \param entity Particle system handle
//! \param index Particle's index
//! \ingroup psyscommands
XDECL float			XCALL xParticleBlue(Entity entity, int index);

//! \brief Sets particle's acceleration
//! \param entity Particle system handle
//! \param index Particle's index
//! \param red Red value of particle color
//! \param green Green value of particle color
//! \param blue Blue value of particle color
//! \ingroup psyscommands
XDECL void			XCALL xParticleColor(Entity entity, int index, float red, float green, float blue);

//! \brief Returns the x value of particle's direction
//! \param entity Particle system handle
//! \param index Particle's index
//! \ingroup psyscommands
XDECL float			XCALL xParticleDX(Entity entity, int index);

//! \brief Returns the y value of particle's direction
//! \param entity Particle system handle
//! \param index Particle's index
//! \ingroup psyscommands
XDECL float			XCALL xParticleDY(Entity entity, int index);

//! \brief Returns the z value of particle's direction
//! \param entity Particle system handle
//! \param index Particle's index
//! \ingroup psyscommands
XDECL float			XCALL xParticleDZ(Entity entity, int index);

//! \brief Sets particle's direction
//! \param entity Particle system handle
//! \param index Particle's index
//! \param x x-value of particle direction
//! \param y y-value of particle direction
//! \param z z-value of particle direction
//! \ingroup psyscommands
XDECL void			XCALL xParticleDirection(Entity entity, int index, float x, float y, float z);

//! \brief Returns the green value of particle's color
//! \param entity Particle system handle
//! \param index Particle's index
//! \ingroup psyscommands
XDECL float			XCALL xParticleGreen(Entity entity, int index);

//! \brief Returns the red value of particle's color
//! \param entity Particle system handle
//! \param index Particle's index
//! \ingroup psyscommands
XDECL float			XCALL xParticleRed(Entity entity, int index);

//! \brief Returns particle's velocity
//! \param entity Particle system handle
//! \param index Particle's index
//! \ingroup psyscommands
XDECL float			XCALL xParticleVelocity(Entity entity, int index);

//! \brief Returns the x location of particle
//! \param entity Particle system handle
//! \param index Particle's index
//! \ingroup psyscommands
XDECL float			XCALL xParticleX(Entity entity, int index);

//! \brief Returns the y location of particle
//! \param entity Particle system handle
//! \param index Particle's index
//! \ingroup psyscommands
XDECL float			XCALL xParticleY(Entity entity, int index);

//! \brief Returns the z location of particle
//! \param entity Particle system handle
//! \param index Particle's index
//! \ingroup psyscommands
XDECL float			XCALL xParticleZ(Entity entity, int index);

//! \brief Returns number of particles in particle system
//! \param entity Particle system handle
//! \ingroup psyscommands
XDECL int			XCALL xParticlesCounter(Entity entity);

//! \brief Pauses channel playing
//! \details When you are playing a sound channel, there may come a time
//! you wish to pause the sound for whatever reason (like to play another
//! sound effect). This command does this - and the channel can be resumed
//! with the xResumeChannel() command. You can use xStopChannel() to
//! actually halt the sound.
//! \param channel Channel handle
//! \ingroup audiocommands
XDECL void			XCALL xPauseChannel(Channel channel);

//! \brief Returns the entity 'picked' by the most recently executed pick
//! command.
//! \details This might have been xCameraPick(), xEntityPick() or xLinePick(). 
//! \ingroup camcommands
XDECL Entity		XCALL xPickedEntity();

//! \brief Returns the x component of the normal of the most recently
//! executed pick command
//! \details This might have been xCameraPick(), xEntityPick() or xLinePick(). 
//! \ingroup camcommands
XDECL float			XCALL xPickedNX();

//! \brief Returns the y component of the normal of the most recently
//! executed pick command
//! \details This might have been xCameraPick(), xEntityPick() or xLinePick(). 
//! \ingroup camcommands
XDECL float			XCALL xPickedNY();

//! \brief Returns the z component of the normal of the most recently
//! executed pick command
//! \details This might have been xCameraPick(), xEntityPick() or xLinePick(). 
//! \ingroup camcommands
XDECL float			XCALL xPickedNZ();

//! \brief Returns the handle of the surface that was 'picked' by the most
//! recently executed pick command
//! \details This might have been xCameraPick(), xEntityPick() or xLinePick(). 
//! \ingroup camcommands
XDECL Surface		XCALL xPickedSurface();

//! \brief Returns the time taken to calculate the most recently executed
//! pick command
//! \details This might have been xCameraPick(), xEntityPick() or xLinePick(). 
//! \ingroup camcommands
XDECL int			XCALL xPickedTime();

//! \brief Returns the index number of the triangle that was 'picked' by
//! the most recently executed pick command
//! \details This might have been xCameraPick(), xEntityPick() or xLinePick(). 
//! \ingroup camcommands
XDECL int			XCALL xPickedTriangle();

//! \brief Returns the world x coordinate of the most recently executed
//! pick command
//! \details This might have been xCameraPick(), xEntityPick() or xLinePick(). 
//! The coordinate represents the exact point of where something was picked
//! \ingroup camcommands
XDECL float			XCALL xPickedX();

//! \brief Returns the world y coordinate of the most recently executed
//! pick command
//! \details This might have been xCameraPick(), xEntityPick() or xLinePick(). 
//! The coordinate represents the exact point of where something was picked
//! \ingroup camcommands
XDECL float			XCALL xPickedY();

//! \brief Returns the world z coordinate of the most recently executed
//! pick command
//! \details This might have been xCameraPick(), xEntityPick() or xLinePick(). 
//! The coordinate represents the exact point of where something was picked
//! \ingroup camcommands
XDECL float			XCALL xPickedZ();

//! \brief This command will load and play a music file. 
//! \details You can't 'preload' the audio like you can a sound sample via
//! the xLoadSound() command. Every time you call the xPlayMusic() command,
//! the file is reloaded and played. This means that if you use the command
//! while some graphics are moving on-screen, you may get a slight pause
//! when the hard drive seeks and grabs the music file. To avoid this, you
//! might want to use the xPlaySound()/xLoopSound() commands instead
//! \param path Name of music file
//! \ingroup audiocommands
XDECL Channel		XCALL xPlayMusic(const char * path);

//! \brief Plays a sound previously loaded using the xLoadSound() command.
//! \param sound Sound handle 
//! \ingroup audiocommands
XDECL Channel		XCALL xPlaySound(Sound sound);

//! \brief Moves all vertices of a mesh
//! \details Unlike xPositionEntity(), xPositionMesh() actually modifies
//! the actual mesh structure. 
//! So whereas using xPositionEntity(0, 0, 1) would only move an entity by
//! one unit the first time it was  used, xPositionMesh(0, 0, 1) will move
//! the mesh by one unit every time it is used. 
//! This is because xPositionEntity() positions an entity based on a fixed
//! mesh structure, whereas xPositionMesh() actually modifies the mesh
//! structure itself.
//! \param entity Entity handle
//! \param x x position of mesh 
//! \param y y position of mesh 
//! \param z z position of mesh 
//! \ingroup meshcommands
XDECL void			XCALL xPositionMesh(Entity entity, float x, float y, float z);

//! \brief Sets particle position (in local particle system's coordinates)
//! \param entity Entity handle
//! \param index Particle index
//! \param x x position of particle 
//! \param y y position of particle 
//! \param z z position of particle 
//! \ingroup psyscommands
XDECL void			XCALL xPositionParticle(Entity entity, int index, float x, float y, float z);

//! \brief Positions a texture at an absolute position. 
//! \details This will have an immediate effect on all instances of the
//! texture being used. 
//! Positioning a texture is useful for performing scrolling texture
//! effects, such as for water etc.
//! \param texture Texture handle
//! \param u u position of texture 
//! \param v v position of texture 
//! \ingroup texcommands
XDECL void			XCALL xPositionTexture(Texture texture, float u, float v);

//! \brief Returns the viewport x coordinate of the most recently executed 
//! xCameraProject()
//! \ingroup camcommands
XDECL float			XCALL xProjectedX();

//! \brief Returns the viewport y coordinate of the most recently executed 
//! xCameraProject()
//! \ingroup camcommands
XDECL float			XCALL xProjectedY();

//! \brief Returns the viewport z coordinate of the most recently executed 
//! xCameraProject()
//! \ingroup camcommands
XDECL float			XCALL xProjectedZ();

// ?!
XDECL void			XCALL xPSSMDebugHud(bool mode = false);

//! \deprecated
XDECL void			XCALL xPSSMInit(Entity camera, const char * path, int mapSize);

//! \deprecated
XDECL void			XCALL xPSSMLightRange(float nearRange, float farRange);

//! \deprecated
XDECL void			XCALL xPSSMLightSource(float x, float y, float z);

//! \deprecated
XDECL void			XCALL xPSSMLightTarget(float x, float y, float z);

//! \deprecated
XDECL void			XCALL xPSSMLightZoom(float zoom);

//! \deprecated
XDECL void			XCALL xPSSMRender(Entity camera, float splitLambda, int splitPlanes, Texture texture);

//! \deprecated
XDECL void			XCALL xPSSMSetEntity(Entity entity, bool caster = true, bool receiver = true);

//! \brief Releases Xors3D engine, clears all resources which are used by it
//! \ingroup maincommands
XDECL void			XCALL xReleaseXors();

//! \brief Renders a post-processing quad with assigned effect
//! \ingroup shadercommands
XDECL void			XCALL xRenderPostEffect(Entity entity);

//! \brief Continues the playing of a sound sample or music track on the
//! given channel after you have temporarily halted playback on that
//! channel (via xPauseChannel()). 
//! \param channel Channel handle
//! \ingroup audiocommands
XDECL void			XCALL xResumeChannel(Channel channel);

//! \brief Rotates images
//! \details The purpose of this command is to rotate an image a specified
//! number of degrees. You my use it in realtime.
//! \param image Image handle
//! \param angle Floating number from 0 to 360 degrees 
//! \ingroup imagecommands
XDECL void			XCALL xRotateImage(Image image, float angle);

//! \brief Rotates all vertices of a mesh by the specified rotation
//! \details Unlike xRotateEntity(), xRotateMesh() actually modifies the
//! actual mesh structure. 
//! So whereas using xRotateEntity(0, 45, 0) would only rotate an entity by
//! 45 degrees the first time it was used, xRotateMesh(0, 45, 0) will
//! rotate the mesh every time it is used. 
//! This is because xRotateEntity() rotates an entity based on a fixed mesh
//! structure, whereas xRotateMesh() actually modifies the mesh structure
//! itself
//! \param entity Entity handle
//! \param x Pitch of mesh
//! \param y Yaw of mesh
//! \param z Roll of mesh
//! \ingroup meshcommands
XDECL void			XCALL xRotateMesh(Entity entity, float x, float y, float z);

//! \brief Rotates a sprite
//! \param entity Sprite handle
//! \param angle Absolute angle of sprite roll rotation 
//! \ingroup sprcommands
XDECL void			XCALL xRotateSprite(Entity entity, float angle);

//! \brief Rotates a texture. 
//! \details This will have an immediate effect on all instances of the
//! texture being used. 
//! Rotating a texture is useful for performing swirling texture effects,
//! such as for smoke etc
//! \ingroup texcommands
XDECL void			XCALL xRotateTexture(Texture texture, float angle);

//! \brief Save buffer into file
//! \details Typically, this is used to take a screen snapshot. This will
//! save the spcified buffer you specify to a .bmp file you specify.
//! \param buffer Buffer handle (xBackBuffer(), xImageBuffer(), xTextureBuffer())
//! \param path File name for buffer saving
//! \ingroup maincommands
XDECL void			XCALL xSaveBuffer(Buffer buffer, const char * path);

//! \brief Resizes an image to a new size using a floating point percentage 
//! \details Using a negative value perform image flipping. You may use this
//! in realtime.
//! \param image Image handle
//! \param x The amount to scale the image horizontally 
//! \param y The amount to scale the image vertically
//! \ingroup imagecommands
XDECL void			XCALL xScaleImage(Image image, float x, float y);

//! \brief Scales all vertices of a mesh by the specified scaling factors
//! \details Unlike xScaleEntity(), xScaleMesh() actually modifies the
//! actual mesh structure. 
//! So whereas using xScaleEntity(2, 2, 2) would only double the size of an
//! entity the first time it was used, xScaleMesh(2, 2, 2) will double the
//! size of the mesh every time it is used. 
//! This is because xScaleEntity() scales an entity based on a fixed mesh
//! structure, whereas xScaleMesh() actually modifies the mesh structure
//! itself.
//! \param entity Entity handle
//! \param x x scale of mesh 
//! \param y y scale of mesh 
//! \param z z scale of mesh 
//! \ingroup meshcommands
XDECL void			XCALL xScaleMesh(Entity entity, float x, float y, float z);

//! \brief Scales a sprite
//! \param entity Sprite handle
//! \param x x scale of sprite 
//! \param y y scale of sprite 
//! \ingroup sprcommands
XDECL void			XCALL xScaleSprite(Entity entity, float x, float y);

//! \brief Scales a texture by an absolute amount. 
//! \details This will have an immediate effect on all instances of the
//! texture being used
//! \param texture Texture handle
//! \param x x scale of texture 
//! \param y y scale of texture 
//! \ingroup texcommands
XDECL void			XCALL xScaleTexture(Texture texture, float x, float y);

//! \deprecated
XDECL void			XCALL xSetAdditionalMatrix(Entity entity, int index,int type);

//! \brief Sets the alpha test function of an entity. 
//! \details The 'value' value should be an integer in the
//! range [1; 8]. The default value is 5 (CMP_GREATER). 
//! Alpha test enables an application to accept or reject a pixel,
//! based on its alpha value.
//! \param entity Entity handle
//! \param value Alpha test function. See '\ref cmpfunc'  for more infomation.
//! \ingroup eccommands
XDECL void			XCALL xSetAlphaFunc(Entity entity, int value);

//! \brief Sets the reference alpha value of an entity. 
//! \details The 'value' value should be an integer in the
//! range [0; 255]. The default value is 0. 
//! It specifies a reference alpha value against which pixels
//! are tested when alpha testing is enabled (See xSetAlphaFunc()). 
//! This is useful for varying smoothness of edges of entities with
//! masked textures (e.g. grass, leaves, clouds).
//! \param entity Entity handle
//! \param value Reference alpha value 
//! \ingroup eccommands
XDECL void			XCALL xSetAlphaRef(Entity entity, int value);

//! \brief Sets anitaliasing type
//! \details This command must be called before xGaphics3D()
//! \param aaType Antialias type. See '\ref aatypes' for more information.
//! \ingroup maincommands
XDECL void			XCALL xSetAntiAliasType(int aaType = AA2SAMPLES);

//! \brief Sets a name of shader constant to pass bones for skinning computing
//! \details If you use hardware skinning, you must pass bones transformation
//! matrices in shader and transform vertices in it.
//! \param entity Entity handle
//! \param arrayName Name of shader constant for passing bones matrices
//! \ingroup shadercommands
XDECL void			XCALL xSetBonesArrayName(Entity entity, const char * arrayName);

//! \brief Sets the current drawing buffer.
//! \details If not used the default buffer, back buffer is used. 
//! \param buffer Buffer handle
//! \ingroup maincommands
XDECL void			XCALL xSetBuffer(Buffer buffer);

//! \brief Sets a boolean constant value for entity shaders
//! \param entity Entity handle
//! \param name Constant name
//! \param value Constant value
//! \ingroup shadercommands
XDECL void			XCALL xSetEffectBool(Entity entity, const char * name, bool value);

//! \brief Sets a boolean constant value for surface shaders
//! \param surface Surface handle
//! \param name Constant name
//! \param value Constant value
//! \ingroup shadercommands
XDECL void			XCALL xSurfaceEffectBool(Surface surface, const char * name, bool value);

//! \brief Sets a matrix constant value for entity shaders
//! \details You may use with it pointer returned by xGetProjectionMatrix(),
//! xGetViewMatrix() and xGetEntityMatrix().
//! \param entity Entity handle
//! \param name Constant name
//! \param matrix Pointer to matrix
//! \ingroup shadercommands
XDECL void			XCALL xSetEffectEntityMatrix(Entity entity, const char * name, void * matrix);

//! \brief Sets a matrix constant value for surface shaders
//! \details You may use with it pointer returned by xGetProjectionMatrix(),
//! xGetViewMatrix() and xGetEntityMatrix().
//! \param surface Surface handle
//! \param name Constant name
//! \param matrix Pointer to matrix
//! \ingroup shadercommands
XDECL void			XCALL xSurfaceEffectEntityMatrix(Surface surface, const char * name, void * matrix);

//! \brief Sets a floating constant value for entity shaders
//! \param entity Entity handle
//! \param name Constant name
//! \param value Constant value
//! \ingroup shadercommands
XDECL void			XCALL xSetEffectFloat(Entity entity, const char * name, float value);

//! \brief Sets a floating constant value for surface shaders
//! \param surface Surface handle
//! \param name Constant name
//! \param value Constant value
//! \ingroup shadercommands
XDECL void			XCALL xSurfaceEffectFloat(Surface surface, const char * name, float value);

//! \brief Sets a floating array constant for entity shaders
//! \param entity Entity handle
//! \param name Constant name
//! \param value Pointer to floating array
//! \param size Number of elements in array
//! \ingroup shadercommands
XDECL void			XCALL xSetEffectFloatArray(Entity entity, const char * name, float * value, int size);

//! \brief Sets a floating array constant for surface shaders
//! \param surface Surface handle
//! \param name Constant name
//! \param value Pointer to floating array
//! \param size Number of elements in array
//! \ingroup shadercommands
XDECL void			XCALL xSurfaceEffectFloatArray(Surface surface, const char * name, float * value, int size);

//! \brief Sets an integer constant value for entity shaders
//! \param entity Entity handle
//! \param name Constant name
//! \param value Constant value
//! \ingroup shadercommands
XDECL void			XCALL xSetEffectInt(Entity entity, const char * name, int value); 

//! \brief Sets shaders integer constant value
//! \param surface Surface handle
//! \param name Constant name
//! \param value Constant value
//! \ingroup shadercommands
XDECL void			XCALL xSurfaceEffectInt(Surface surface, const char * name, int value); 

//! \brief Sets an integer array constant for entity shaders
//! \param entity Entity handle
//! \param name Constant name
//! \param value Pointer to integer array
//! \param size Number of elements in array
//! \ingroup shadercommands
XDECL void			XCALL xSetEffectIntArray(Entity entity, const char * name, int * value, int size); 

//! \brief Sets an integer array constant for surface shaders
//! \param surface Surface handle
//! \param name Constant name
//! \param value Pointer to integer array
//! \param size Number of elements in array
//! \ingroup shadercommands
XDECL void			XCALL xSurfaceEffectIntArray(Surface surface, const char * name, int * value, int size); 

//! \deprecated
XDECL void			XCALL xSetEffectMatrix(Entity entity, const char * name, float f11, float f12, float f13, float f14, float f21, float f22, float f23, float f24, float f31, float f32, float f33, float f34, float f41, float f42, float f43, float f44);

//! \deprecated
XDECL void			XCALL xSurfaceEffectMatrix(Surface surface, const char * name, float f11, float f12, float f13, float f14, float f21, float f22, float f23, float f24, float f31, float f32, float f33, float f34, float f41, float f42, float f43, float f44);

//! \brief Sets a matrices array constant for shaders
//! \param entity Entity handle
//! \param name Constant name
//! \param value Pointer to matrices array
//! \param size Number of elements in array
//! \ingroup shadercommands
XDECL void			XCALL xSetEffectMatrixArray(Entity entity, const char * name, void * value, int size);

//! \brief Sets a matrices array constant for surface shaders
//! \param surface Surface handle
//! \param name Constant name
//! \param value Pointer to matrices array
//! \param size Number of elements in array
//! \ingroup shadercommands
XDECL void			XCALL xSurfaceEffectMatrixArray(Surface surface, const char * name, void * value, int size);

//! \brief Sets a matrix constant for entity shaders by matrix semantic
//! \details See '\ref mtxsemantic' for more infromation.
//! \param entity Entity handle
//! \param name Constant name
//! \param semantic Matrix semantic
//! \ingroup shadercommands
XDECL void			XCALL xSetEffectMatrixSemantic(Entity entity, const char * name, int semantic);

//! \brief Sets a matrix constant for surface shaders by matrix semantic
//! \details See '\ref mtxsemantic' for more infromation.
//! \param surface Surface handle
//! \param name Constant name
//! \param semantic Matrix semantic
//! \ingroup shadercommands
XDECL void			XCALL xSurfaceEffectMatrixSemantic(Surface surface, const char * name, int semantic);

//! \brief Sets active effect rendering technique
//! \details You may check support of technique by hardware using 
//! xValidateEffectTechnique()
//! \param entity Entity handle
//! \param name Technique name
//! \ingroup shadercommands
XDECL void			XCALL xSetEffectTechnique(Entity entity, const char * name);

//! \brief Sets active surface's effect rendering technique
//! \details You may check support technique on hadware by 
//! xValidateEffectTechnique()
//! \param surface Surface handle
//! \param name Technique name
//! \ingroup shadercommands
XDECL void			XCALL xSurfaceTechnique(Surface surface, const char * name);

//! \brief Sets a vectors array constant for entity shaders
//! \param entity Entity handle
//! \param name Constant name
//! \param value Pointer to vectors array
//! \param size Number of elements in array
//! \ingroup shadercommands
XDECL void			XCALL xSetEffectVectorArray(Entity entity, const char * name, void * value, int size);

//! \brief Sets a vectors array constant for surface shaders
//! \param surface Surface handle
//! \param name Constant name
//! \param value Pointer to vectors array
//! \param size Number of elements in array
//! \ingroup shadercommands
XDECL void			XCALL xSurfaceEffectVectorArray(Surface surface, const char * name, void * value, int size);

//! \brief Assings shader effect to entity
//! \param entity Entity handle
//! \param effect Effect handle
//! \ingroup shadercommands
XDECL void			XCALL xSetEntityEffect(Entity entity, Effect effect);

//! \brief Sets entity world matrix
//! \param entity Entity handle
//! \param matrix Pointer to matrix
//! \ingroup eccommands
XDECL void			XCALL xSetEntityMatrix(Entity entity, void * matrix);

//! \brief Sets the value of an element of entity's world matrix
//! \param entity Entity handle
//! \param row Matrix row index 
//! \param cell Matrix cell index 
//! \param value New matrix element value
//! \ingroup eccommands
XDECL void			XCALL xSetEntityMatrixElement(Entity entity, int row, int cell, float value);

//! \deprecated
XDECL void			XCALL xSetEntityPixelShader(Entity entity, PixelShader shader);

//! \deprecated
XDECL void			XCALL xSetEntityVertexShader(Entity entity, VertexShader shader);

//! \brief Sets entity rotation quaternion
//! \param entity Entity handle
//! \param quat Pointer to quaternion
//! \ingroup eccommands
XDECL void			XCALL xSetEntityQuaternion(Entity entity, void * quat);

//! \brief Activates a font previously loaded into memory (though the
//! xLoadFont() command) for future use with printing commands such as xText(). 
//! \param font Font handle
//! \ingroup textcommands
XDECL void			XCALL xSetFont(Font font);

//! \brief Sets entity's bounding sphere for frustum culling
//! \param entity Entity handle
//! \param x x coordinate of bounding sphere center
//! \param y y coordinate of bounding sphere center
//! \param z z coordinate of bounding sphere center
//! \param radius Radius of bounding sphere
//! \ingroup camcommands
XDECL void			XCALL xSetFrustrumSphere(Entity entity, float x, float y, float z, float radius);

//! \brief Updates vertex and index buffers from user arrays
//! \details It may be used to get deformation data from physics
//! engines.
//! \param entity Entity handle
//! \param nVB Pointer to new vertices positions buffer
//! \param nNB Pointer to new vertices normals buffer
//! \param nIB Pointer to new indices buffer
//! \param nPIB Pointer to old indices buffer (nned for correct remaping)
//! \param numVert New vertices count
//! \param numTris New triangles count
//! \ingroup meshcommands
XDECL void			XCALL xSetMeshVNIB(Entity entity, float * nVB, float * nNB, int * nIB, int * nPIB, int numVert, int numTris);

// ?!
XDECL void			XCALL xSetNeedUpdateBoundingSphere(Entity entity, bool need = true);

//! \brief Sets particle's acceleration
//! \param entity Particle system handle
//! \param index Particle's index
//! \param accel New particle's acceleration
//! \ingroup psyscommands
XDECL void			XCALL xSetParticleAcceleration(Entity entity, int index, float accel);

//! \brief Sets particle's age
//! \param entity Particle system handle
//! \param index Particle's index
//! \param age New particle's age
//! \ingroup psyscommands
XDECL void			XCALL xSetParticleAge(Entity entity, int index, int age);

//! \brief Sets particle's velocity
//! \param entity Particle system handle
//! \param index Particle's index
//! \param velocity New particle's velocity
//! \ingroup psyscommands
XDECL void			XCALL xSetParticleVelocity(Entity entity, int index, float velocity);

//! \deprecated
XDECL void			XCALL xSetPixelShaderBool(Entity entity, const char * name, bool value);

//! \deprecated
XDECL void			XCALL xSetPixelShaderDefaults(PixelShader shader);

//! \deprecated
XDECL void			XCALL xSetPixelShaderFloat(Entity entity, const char * name, float value);

//! \deprecated
XDECL void			XCALL xSetPixelShaderInt(Entity entity, const char * name, int value);

//! \deprecated
XDECL void			XCALL xSetPixelShaderMatrix(Entity entity, const char * name, float f11, float f12, float f13, float f14, float f21, float f22, float f23, float f24, float f31, float f32, float f33, float f34, float f41, float f42, float f43, float f44);

//! \deprecated
XDECL void			XCALL xSetPixelShaderMatrixSemantic(Entity entity, const char * name, int semantic);

//! \deprecated
XDECL void			XCALL xSetPostEffectBaseSampler(PostEffect postEffect, const char * name);

//! \deprecated
XDECL void			XCALL xSetPostEffectBaseSamplerScale(PostEffect postEffect, float x, float y);

//! \deprecated
XDECL void			XCALL xSetPostEffectBool(PostEffect postEffect, const char * name, bool value);

//! \deprecated
XDECL void			XCALL xSetPostEffectDefaults(PostEffect postEffect);

//! \deprecated
XDECL void			XCALL xSetPostEffectFloat(PostEffect postEffect, const char * name, float value);

//! \deprecated
XDECL void			XCALL xSetPostEffectInt(PostEffect postEffect, const char * name, int value);

//! \deprecated
XDECL void			XCALL xSetPostEffectSceneSampler(PostEffect postEffect, const char * sampler);

//! \deprecated
XDECL void			XCALL xSetPostEffectTexture(PostEffect postEffect, const char * name, Texture value);

//! \brief Sets window for rendering
//! \details It may be used by map editor or other tools to render in
//! some gadgets.
//! \param newWND Pointer to rendering window
//! \ingroup maincommands
XDECL void			XCALL xSetRenderWindow(void * newWND);

//! \brief Sets texture filtering method
//! \details During texture loading the method set by this command will be
//! assigned to this texture. If you use shaders you can change filtering type in shader's code.
//! \param filter Filtering type. See '\ref texfilters' for more information
//! \ingroup maincommands
XDECL void			XCALL xSetTextureFiltering(int filter = TF_LINEAR);

//! \deprecated
XDECL void			XCALL xSetVertexShaderBool(Entity entity, const char * name, bool value);

//! \deprecated
XDECL void			XCALL xSetVertexShaderDefaults(VertexShader shader);

//! \deprecated
XDECL void			XCALL xSetVertexShaderFloat(Entity entity, const char * name, float value);

//! \deprecated
XDECL void			XCALL xSetVertexShaderInt(Entity entity, const char * name, int value);

//! \deprecated
XDECL void			XCALL xSetVertexShaderMatrix(Entity entity, const char * name, float f11, float f12, float f13, float f14, float f21, float f22, float f23, float f24, float f31, float f32, float f33, float f34, float f41, float f42, float f43, float f44);

//! \deprecated
XDECL void			XCALL xSetVertexShaderMatrixSemantic(Entity entity, const char * name, int semantic);

//! \brief For Blitz3D only. Pass Blitz's rendering window to destroy it
//! \param hwnd Blitz3D window, may be get by SystemProperty("AppHWND")
//! \ingroup maincommands
XDECL void			XCALL xSetWND(void * hwnd);

//! \brief Shows an entity. 
//! \details Once an entity has been hidden using xHideEntity(), use show
//! entity to make it visible and involved in collisions again.
//! Entities are shown by default after creating/loading them, so you
//! should only need to use xShowEntity() after using xHideEntity().
//! \param entity Entity handle
//! \ingroup eccommands
XDECL void			XCALL xShowEntity(Entity entity);

//! \brief Shows the windows pointer after it's been hidden (via
//! xHidePointer()). It has no effect in full-screen modes
//! \ingroup maincommands
XDECL void			XCALL xShowPointer();

//! \brief Pans sound effect between the left and right speakers 
//! \param sound Sound handle
//! \param pan Floating point number from -1.0 (left) to 0.0 (center) to
//! 1.0 (right) 
//! \ingroup audiocommands
XDECL void			XCALL xSoundPan(Sound sound, float pan);

//! \brief Alters the pitch of a sound
//! \details By changing the pitch, you can often reuse sounds for different
//! uses or to simulate a 'counting up/down' sound. To make the sound
//! 'higher pitched', increase the hertz. Conversely, decreasing the hertz
//! will 'lower' the pitch. Note: this is in relation to the original hertz
//! frequency of the sound. 
//! \param sound Sound handle
//! \param hertz Valid playback hertz speed (up to 44000 hertz)
//! \ingroup audiocommands
XDECL void			XCALL xSoundPitch(Sound sound, int hertz);

//! \brief Alters the playback volume of sound effect
//! \details This command uses a floating point number from 0.0 to 1.0
//! to control the volume level
//! \param sound Sound handle
//! \param volume Floating point number from 0.0 (silence) to 1.0 (full volume) 
//! \ingroup audiocommands
XDECL void			XCALL xSoundVolume(Sound sound, float volume);

//! \brief Returns true if sphere passed frustum culling check
//! \param camera Camera handle
//! \param x x coordinate of sphere center
//! \param y y coordinate of sphere center
//! \param z z coordinate of sphere center
//! \param radius Radius of sphere
//! \ingroup camcommands
XDECL bool			XCALL xSphereInFrustum(Entity camera, float x, float y, float z, float radius);

//! \brief Sets the view mode of a sprite. 
//! \details The view mode determines how a sprite alters its orientation
//! in respect to the camera. This allows the sprite to in some instances
//! give the impression that it is more than two dimensional. 
//! See '\ref sviewmodes' for more infromation.
//! Note that if you use sprite view mode SPRITE_FREE, then because it is
//! independent from the camera, you will only be able to see it from one
//! side unless you use xEntityFX() flag FX_DISABLECULLING with it to
//! disable backface culling. 
//! \param entity Sprite handle
//! \param mode View mode
//! \ingroup sprcommands
XDECL void			XCALL xSpriteViewMode(Entity entity, int mode);

//! \brief Stops channel playing
//! \param channel Channel handle
//! \ingroup audiocommands
XDECL void			XCALL xStopChannel(Channel channel);

//! \brief Copys data from back buffer to texture using hardware accelerated
//! method
//! \param texture Texture handle
//! \param x x coordinate of top left corner of copied rectangle
//! \param y y coordinate of top left corner of copied rectangle
//! \param width Width of copied rectangle
//! \param height Height of copied rectangle
//! \param filter Filtering method
//! \ingroup maincommands
XDECL void			XCALL xStretchBackBuffer(Texture texture, int x, int y, int width, int height, int filter = 0);

//! \brief Copys data from one texture to another using hardware accelerated
//! method
//! \param texture1 Source texture handle
//! \param x1 x coordinate of top left corner of source rectangle
//! \param y1 y coordinate of top left corner of source rectangle
//! \param width1 Width of source rectangle
//! \param height1 Height of source rectangle
//! \param texture2 Destination texture handle
//! \param x2 x coordinate of top left corner of destination rectangle
//! \param y2 y coordinate of top left corner of destination rectangle
//! \param width2 Width of destination rectangle
//! \param height2 Height of destination rectangle
//! \param filter Filtering method
//! \ingroup texcommands
XDECL void			XCALL xStretchRect(Texture texture1, int x1, int y1, int width1, int height1, Texture texture2, int x2, int y2, int width2, int height2,int filter = 0);


//! \brief Returns the size, in pixels, the height of the indicated string.
//! \details This is useful for determining screen layout, scrolling of 
//! text, and more. This is calculated based on the size of the currently
//! loaded font. 
//! \param text Any valid string 
//! \ingroup textcommands
XDECL int			XCALL xStringHeight(const char * text);

//! \brief Returns the size, in pixels, the width of the indicated string.
//! \details This is useful for determining screen layout, scrolling of 
//! text, and more. This is calculated based on the size of the currently
//! loaded font. 
//! \param text Any valid string 
//! \ingroup textcommands
XDECL int			XCALL xStringWidth(const char * text);

//! \brief Transforms vector between coordinate systems.
//! \details After using xTFormNormal() the new components can be read with
//! xTFormedX(), xTFormedY() and xTFormedZ(). 
//! This is exactly the same as xTFormVector() but with one added feature. 
//! After the transformation the new vector is 'normalized', meaning it 
//! is scaled to have length 1. 
//! For example, suppose the result of xTFormVector is (1.0, 2.0, 2.0). 
//! This vector has length sqrt(1.0 * 1.0 + 2.0 * 2.0 + 2.0 * 2.0) =
//! sqrt(9.0) = 3.0
//! This means xTFormNormal() would produce (1.0 / 3.0, 2.0 / 3.0, 2.0 / 3.0). 
//! \param x x component of a vector in 3D space 
//! \param y y component of a vector in 3D space 
//! \param z z component of a vector in 3D space 
//! \param src Handle of source entity, or 0 for 3d world 
//! \param dest Handle of dstination entity, or 0 for 3d world 
//! \ingroup mathcommands
XDECL void			XCALL xTFormNormal(float x, float y, float z, Entity src, Entity dest);

//! \brief Transforms point between coordinate systems.
//! \details After using xTFormPoint() the new components can be read with
//! xTFormedX(), xTFormedY() and xTFormedZ(). 
//! See xEntityX() for details about local coordinates. 
//! Consider a sphere built with xCreateSphere(). The 'north pole' is at
//! (0.0, 1.0, 0.0). 
//! At first, local and global coordinates are the same. As the sphere is
//! moved, turned and scaled the global coordinates of the point change. 
//! But it is always at (0.0, 1.0, 0.0) in the sphere's local space. 
//! \param x x component of a vector in 3D space 
//! \param y y component of a vector in 3D space 
//! \param z z component of a vector in 3D space 
//! \param src Handle of source entity, or 0 for 3d world 
//! \param dest Handle of dstination entity, or 0 for 3d world 
//! \ingroup mathcommands
XDECL void			XCALL xTFormPoint(float x, float y, float z, Entity src, Entity dest);

//! \brief Transforms vector between coordinate systems.
//! \details After using xTFormVector() the new components can be read with
//! xTFormedX(), xTFormedY() and xTFormedZ(). 
//! See xEntityX() for details about local coordinates. 
//! Similar to xTFormPoint(), but operates on a vector. A vector can be
//! thought of as 'displacement relative to current location'. 
//! For example, vector (1.0, 2.0, 3.0) means one step to the right, two
//! steps up and three steps forward. 
//! \param x x component of a vector in 3D space 
//! \param y y component of a vector in 3D space 
//! \param z z component of a vector in 3D space 
//! \param src Handle of source entity, or 0 for 3d world 
//! \param dest Handle of dstination entity, or 0 for 3d world 
//! \ingroup mathcommands
XDECL void			XCALL xTFormVector(float x, float y, float z, Entity src, Entity dest);

//! \brief Returns the X component of the last xTFormPoint(), xTFormVector()
//! or xTFormNormal() operation
//! \ingroup mathcommands
XDECL float			XCALL xTFormedX();

//! \brief Returns the Y component of the last xTFormPoint(), xTFormVector()
//! or xTFormNormal() operation
//! \ingroup mathcommands
XDECL float			XCALL xTFormedY();

//! \brief Returns the Z component of the last xTFormPoint(), xTFormVector()
//! or xTFormNormal() operation
//! \ingroup mathcommands
XDECL float			XCALL xTFormedZ();

//! \brief Returns the height of the terrain at terrain grid coordinates x, z.
//! \details The value returned is in the range [0.0; 1.0].
//! \param entity Entity handle
//! \param x Grid x coordinate of terrain
//! \param y Grid y coordinate of terrain
//! \ingroup terrcommands
XDECL float			XCALL xTerrainHeight(Entity entity, int x, int y);

//! \brief Enables or disables terrain shading. 
//! \details Shaded terrains are a little slower than non-shaded terrains,
//! and in some instances can increase the visibility. However, the option
//! is there to have shaded terrains if you wish to do so. 
//! \param entity Entity handle
//! \param state True to enable terrain shading, false to to disable it.
//! The default mode is false
//! \ingroup terrcommands
XDECL void			XCALL xTerrainShading(Entity entity, bool state);

//! \brief Returns the grid size in x-axis used to create a terrain
//! \param entity Entity handle
//! \ingroup terrcommands
XDECL int			XCALL xTerrainSizeX(Entity entity);

//! \brief Returns the grid size in z-axis used to create a terrain
//! \param entity Entity handle
//! \ingroup terrcommands
XDECL int			XCALL xTerrainSizeZ(Entity entity);

//! \brief Returns the interpolated x coordinate on a terrain
//! \param entity Entity handle
//! \param x World x coordinate
//! \param y World y coordinate
//! \param z World z coordinate
//! \ingroup terrcommands
XDECL float			XCALL xTerrainX(Entity entity, float x, float y, float z);

//! \brief Returns the interpolated y coordinate on a terrain
//! \param entity Entity handle
//! \param x World x coordinate
//! \param y World y coordinate
//! \param z World z coordinate
//! \ingroup terrcommands
XDECL float			XCALL xTerrainY(Entity entity, float x, float y, float z);

//! \brief Returns the interpolated z coordinate on a terrain
//! \param entity Entity handle
//! \param x World x coordinate
//! \param y World y coordinate
//! \param z World z coordinate
//! \ingroup terrcommands
XDECL float			XCALL xTerrainZ(Entity entity, float x, float y, float z);

//! \brief Sets the blending mode for a texture. 
//! \details The texture blend mode determines how the texture will blend
//! with the texture or polygon which is 'below' it. Texture 0 will blend
//! with the polygons of the entity it is applied to. Texture 1 will blend
//! with texture 0. Texture 2 will blend with texture 1. And so on. 
//! Texture blending effectively takes the highest order texture (the one
//! with the highest index) and it blends with the texture below it, then
//! that result to the texture directly below again, and so on until
//! texture 0 which is blended with the polygons of the entity it is
//! applied to and thus the world, depending on the xEntityBlend() of the
//! object. 
//! Each of the blend modes are identical to their xEntityBlend()
//! counterparts.
//! \param texture Texture handle
//! \param blend Blending mode. See '\ref texblendtypes' for more infromation
//! \ingroup texcommands
XDECL void			XCALL xTextureBlend(Texture texture, int blend = TEXBLEND_NONE);

//! \brief Sets the texture coordinate mode for a texture. 
//! \details This determines where the UV values used to look up a texture
//! come from
//! \param texture Texture handle
//! \param coords UV coordinates number (0 or 1)
//! \ingroup texcommands
XDECL void			XCALL xTextureCoords(Texture texture, int coords);

//! \brief Returns the height of a texture
//! \param texture Texture handle
//! \ingroup texcommands
XDECL int			XCALL xTextureHeight(Texture texture);

//! \brief Returns the height of a texture
//! \param texture Texture handle
//! \ingroup texcommands
XDECL const char *	XCALL xTextureName(Texture texture);

//! \brief Returns a texture's absolute filename. 
//! \details To find out just the name of the texture, you will need to
//! parse the string returned by xTextureName().
//! \param texture Texture handle
//! \ingroup texcommands
XDECL int			XCALL xTextureWidth(Texture texture);

//! \brief Returns the vertex of a triangle corner
//! \param surface Surface handle
//! \param index Triangle index
//! \param coner Corner of triangle. Should be 0, 1 or 2.
//! \ingroup surfcommands
XDECL int			XCALL xTriangleVertex(Surface surface, int index, int coner);

//! \brief Returns the number of triangles rendered during last rendering cycle
//! \ingroup maincommands
XDECL int			XCALL xTrisRendered();

//! \brief Updates input information
//! \ingroup maincommands
XDECL void			XCALL xUpdateIM();

//! \brief Updates mesh index buffer by user indices array
//! \details You may use it to get deformations data from physics 
//! engines
//! \param entity Entity handle
//! \param indexArray Pointer to indices
//! \ingroup meshcommands
XDECL void			XCALL xUpdateMeshIB(Entity entity, int * indexArray);

//! \brief Updates mesh vertex buffers normals by user normals array
//! \details You may use it to get deformations data from physics 
//! engines
//! \param entity Entity handle
//! \param normArray Pointer to normals
//! \ingroup meshcommands
XDECL void			XCALL xUpdateMeshVBNormals(Entity entity, float * normArray);

//! \brief Updates mesh vertex buffers positions by user normals array
//! \details You may use it to get deformations data from physics 
//! engines
//! \param entity Entity handle
//! \param posArray Pointer to positions
//! \ingroup meshcommands
XDECL void			XCALL xUpdateMeshVBPositions(Entity entity, float * posArray);

//! \brief Recalculates all normals in a mesh.
//! \details This is necessary for correct lighting if you have not set
//! surface normals using xVertexNormals() commands. 
//! \param entity Entity handle
//! \ingroup meshcommands
XDECL void			XCALL xUpdateNormals(Entity entity);

//! \deprecated
XDECL void			XCALL xUpdatePhysxPose(Entity entity);

//! \brief Returns true if hardware supports specified technique
//! \param effect Effect handle
//! \param name Technique name
//! \ingroup shadercommands
XDECL bool			XCALL xValidateEffectTechnique(Effect effect, const char * name);

//! \brief Returns the pitch value of a vector. 
//! \details Using this command will return the same result as using
//! xEntityPitch() to get the pitch value of an entity that is pointing in
//! the vector's direction. 
//! \param x x vector length 
//! \param y y vector length 
//! \param z z vector length 
//! \ingroup mathcommands
XDECL float			XCALL xVectorPitch(float x, float y, float z);

//! \brief Returns the yaw value of a vector. 
//! \details Using this command will return the same result as using
//! xEntityYaw() to get the yaw value of an entity that is pointing in the
//! vector's direction. 
//! \param x x vector length 
//! \param y y vector length 
//! \param z z vector length 
//! \ingroup mathcommands
XDECL float			XCALL xVectorYaw(float x, float y, float z);

//! \brief Returns the alpha component of a vertices color, set using xVertexColor()
//! \param surface Surface handle
//! \param vertex Vertex index
//! \ingroup surfcommands
XDECL float			XCALL xVertexAlpha(Surface surface, int vertex);

//! \brief Returns the blue component of a vertices color
//! \param surface Surface handle
//! \param vertex Vertex index
//! \ingroup surfcommands
XDECL float			XCALL xVertexBlue(Surface surface, int vertex);

//! \brief Sets the geometric coordinates of an existing vertex. 
//! \details This is the command used to perform what is commonly referred
//! to as 'dynamic mesh deformation'. It will reposition a vertex so that
//! all the triangle edges connected to it, will move also. This will give
//! the effect of parts of the mesh suddenly deforming. 
//! \param surface Surface handle
//! \param vertex Vertex index
//! \param x x position of vertex
//! \param y y position of vertex
//! \param z z position of vertex
//! \ingroup surfcommands
XDECL void			XCALL xVertexCoords(Surface surface, int vertex, float x, float y, float z);

//! \brief Returns the green component of a vertices color
//! \param surface Surface handle
//! \param vertex Vertex index
//! \ingroup surfcommands
XDECL float			XCALL xVertexGreen(Surface surface, int vertex);

//! \brief Returns the x component of a vertices normal
//! \param surface Surface handle
//! \param vertex Vertex index
//! \ingroup surfcommands
XDECL float			XCALL xVertexNX(Surface surface, int vertex);

//! \brief Returns the y component of a vertices normal
//! \param surface Surface handle
//! \param vertex Vertex index
//! \ingroup surfcommands
XDECL float			XCALL xVertexNY(Surface surface, int vertex);

//! \brief Returns the z component of a vertices normal
//! \param surface Surface handle
//! \param vertex Vertex index
//! \ingroup surfcommands
XDECL float			XCALL xVertexNZ(Surface surface, int vertex);

//! \brief Sets the normal of an existing vertex
//! \param surface Surface handle
//! \param vertex Vertex index
//! \param nx Normal x of vertex
//! \param ny Normal y of vertex
//! \param nz Normal z of vertex
//! \ingroup surfcommands
XDECL void			XCALL xVertexNormal(Surface surface, int vertex, float nx, float ny, float nz);

//! \brief Returns the red component of a vertices color
//! \param surface Surface handle
//! \param vertex Vertex index
//! \ingroup surfcommands
XDECL float			XCALL xVertexRed(Surface surface, int vertex);

//! \brief Returns the x coordinate of a vertex
//! \param surface Surface handle
//! \param vertex Vertex index
//! \ingroup surfcommands
XDECL float			XCALL xVertexX(Surface surface, int vertex);

//! \brief Returns the y coordinate of a vertex
//! \param surface Surface handle
//! \param vertex Vertex index
//! \ingroup surfcommands
XDECL float			XCALL xVertexY(Surface surface, int vertex);

//! \brief Returns the z coordinate of a vertex
//! \param surface Surface handle
//! \param vertex Vertex index
//! \ingroup surfcommands
XDECL float			XCALL xVertexZ(Surface surface, int vertex);

//! \brief Returns a string which shows description of video card being used
//! \ingroup sicommands
XDECL const char *	XCALL xVideoInfo();

//! \brief Stops programm until any key is pressed
//! \details This command makes your program halt until a key is pressed
//! on the keyboard. Used alone, it simply halts and waits for a key press.
//! \ingroup keycommands
XDECL void			XCALL xWaitKey();

//! \brief Returns true if specified windows message occurs.
//! \param msgWin Windows message to be checked. "WM_CLOSE" is the only proper message presently.
//! \ingroup maincommands
XDECL int			XCALL xWinMessage(const char * msgWin);

//! \brief Enables or disables wireframe rendering. 
//! This will show the outline of each polygon on the screen, with no
//! shaded-in areas. 
//! \param state True to enable wireframe rendering, false to disable
//! \ingroup maincommands
XDECL void			XCALL xWireframe(bool state);

//! \brief Adds a vertex to the specified surface and returns the vertices
//! index number, starting from 0. 
//! \details 'x', 'y', 'z' are the geometric coordinates of the vertex, and
//! 'u', 'v', 'w' are texture mapping coordinates. 
//! A vertex is a point in 3D space which is used to connect edges of a
//! triangle together. Without any vertices, you can't have any triangles.
//! At least three vertices are needed to create one triangle; one for each
//! corner. 
//! The optional 'u', 'v' and 'w' parameters allow you to specify texture
//! coordinates for a vertex, which will determine how any triangle created
//! using those vertices will be texture mapped. The 'u', 'v' and 'w'
//! parameters specified will take effect on both texture coordinate sets
//! (0 and 1). This works on the following basis:
//!
//! The top left of an image has the uv coordinates 0.0, 0.0. 
//!
//! The top right has coordinates 1.0, 0.0 
//!
//! The bottom right is 1.0, 1.0 
//!
//! The bottom left 0.0, 1.0 
//!
//! Thus, uv coordinates for a vertex correspond to a point in the image.
//! For example, coordinates 0.9, 0.1 would be near the upper right corner
//! of the image. 
//! So now imagine you have a normal equilateral triangle. By assigning the
//! bottom left vertex a uv coordinate of 0.0, 0.0, the bottom right a
//! coordinate of 1.0, 0.0 and the top centre 0.5, 1.0, this will texture
//! map the triangle with an image that fits it. 
//! When adding a vertex its default color is 255, 255, 255, 255.
//! \param surface Surface handle
//! \param x x coordinate of vertex
//! \param y y coordinate of vertex
//! \param z z coordinate of vertex
//! \param u u texture coordinate of vertex
//! \param v v texture coordinate of vertex
//! \param w w texture coordinate of vertex - not used, included for future
//! expansion 
//! \ingroup surfcommands
XDECL int			XCALL xAddVertex(Surface surface, float x, float y, float z, float u, float v, float w);

//! \brief Aligns an entity axis to a vector
//! \param entity Entity handle
//! \param x Vector x
//! \param y Vector y
//! \param z Vector z
//! \param axis Axis of entity that will be aligned to vector. See '\ref
//! axistypes' for more information
//! \param factor Rate at which entity is aligned from current orientation
//! to vector orientation. Should be in the range [0.0; 1.0], 0.0 for
//! smooth transition and 1.0 for 'snap' transition.
//! \ingroup emcommands
XDECL void			XCALL xAlignToVector(Entity entity, float x, float y, float z, int axis, float factor);

//! \brief Returns the length of the specified entity's current animation
//! sequence
//! \details This function must be used only with real skinned mesh, or it
//! will return wrong values
//! \param entity Entity handle
//! \param rootBone Root bone name in animation track. See xAnimate() for
//! more infromation
//! \ingroup eacommands
XDECL float			XCALL xAnimLength(Entity entity, const char * rootBone = "");

//! \brief Returns the specified entity's current animation sequence
//! \details This function must be used only with real skinned mesh, or it
//! will return wrong values
//! \param entity Entity handle
//! \param rootBone Root bone name in animation track. See xAnimate() for
//! more infromation
//! \ingroup eacommands
XDECL int			XCALL xAnimSeq(Entity entity, const char * rootBone = "");

//! \brief Returns the specified entity's current animation speed
//! \details This function must be used only with real skinned mesh, or it
//! will return wrong values
//! \param entity Entity handle
//! \param rootBone Root bone name in animation track. See xAnimate() for
//! more infromation
//! \ingroup eacommands
XDECL float			XCALL xAnimSpeed(Entity entity, const char * rootBone = "");

//! \brief Returns the current animation time of an entity
//! \details This function must be used only with real skinned mesh, or it
//! will return wrong values
//! \param entity Entity handle
//! \param rootBone Name of root bone in animation track. See xAnimate() for
//! more infromation
//! \ingroup eacommands
XDECL float			XCALL xAnimTime(Entity entity, const char * rootBone = "");

//! \brief Animates an entity
//! \param entity Entity handle
//! \param mode Animating mode. See '\ref animtypes' for more information
//! \param speed Animation speed. A negative speed will play the animation
//! backwards
//! \param sequence Initially, an entity loaded with xLoadAnimMesh() will
//! have a single animation sequence. More sequences can be added using
//! either xLoadAnimSeq() or xAddAnimSeq(). Animation sequences are numbered
//! 0, 1, 2, etc
//! \param translate A value of 0 will cause an instant 'leap' to the first
//! frame, while values greater than 0 will cause a smooth transition
//! \param rootBone Root bone in hierarhy, from that animation will be
//! applied. In Blitz3D you may find bone by FindChild() command and
//! animate it. In Xors3D you must simply pass its name in xAnimate()
//! command. For hierarhy a new animation track will be created, so then
//! you will need to get its info (like time, speed, etc) you must also pass
//! a name of its root bone for correct results.
//! \ingroup eacommands
XDECL void			XCALL xAnimate(Entity entity, int mode = ANIMATION_STOP, float speed = 1.0f, int sequence = 0, int translate = 0, const char * rootBone = "");

//! \brief Returns true if the specified entity is currently animating
//! \details This function must be used only with real skinned mesh, or it
//! will return wrong values
//! \param entity Entity handle
//! \param rootBone Root bone name in animation track. See xAnimate() for
//! more infromation
//! \ingroup eacommands
XDECL int			XCALL xAnimating(Entity entity, const char * rootBone = "");

//! \brief Sets the alpha level of a brush. 
//! \details The alpha value should be in the range [0.0; 1.0]. The default
//! brush alpha setting is 1.0. 
//! The alpha level is how transparent an entity is. A value of 1.0 will
//! mean the entity is non-transparent, i.e. opaque. A value of 0.0 will
//! mean the entity is completely transparent, i.e. invisible. Values
//! between 0.0 and 1.0 will cause varying amount of transparency
//! accordingly, useful for imitating the look of objects such as glass
//! and ice.
//! \param brush Brush handle
//! \param alpha Alpha level of brush
//! \ingroup brushcommands
XDECL void			XCALL xBrushAlpha(Brush brush, float alpha);

//! \brief Sets the blending mode for a brush
//! \param brush Brush handle
//! \param blend Blending type. See '\ref entblendtypes' for more infromation
//! \ingroup brushcommands
XDECL void			XCALL xBrushBlend(Brush brush, int blend = BLEND_ALPHA);

//! \brief Sets the colour of a brush. 
//! \details The green, red and blue values should be in the range [0; 255].
//! The default brush color is 255, 255, 255. 
//! Please note that if xEntityFX() or xBrushFX() flag FX_VERTEXCOLOR is
//! being used, brush colour will have no effect and vertex colours will
//! be used instead. 
//! \param brush Brush handle
//! \param red Red value of brush
//! \param green Green value of brush
//! \param blue Blue value of brush
//! \ingroup brushcommands
XDECL void			XCALL xBrushColor(Brush brush, float red, float green, float blue);

//! \brief Sets miscellaneous effects for a brush. 
//! \details Flags can be added to combine two or more effects.
//! \param brush Brush handle
//! \param fxFlags ffects flags See '\ref fxflags' for more infromation
//! \ingroup brushcommands
XDECL void			XCALL xBrushFX(Brush brush, int fxFlags = FX_NOTHING);

//! \brief Sets the specular shininess of a brush. 
//! \details The shininess value should be in the range [0.0; 1.0]. The
//! default shininess setting is 0.0 
//! Shininess is how much brighter certain areas of an object will appear
//! to be when a light is shone directly at them. 
//! Setting a shininess value of 1.0 for a medium to high poly sphere,
//! combined with the creation of a light shining in the direction of it,
//! will give it the appearance of a shiny snooker ball
//! \param brush Brush handle
//! \param shininess Shininess of brush 
//! \ingroup brushcommands
XDECL void			XCALL xBrushShininess(Brush brush, float shininess);

//! \brief Assigns a texture to a brush. 
//! \details The optional frame parameter specifies which animation frame,
//! if any exist, should be assigned to the brush. 
//! The optional index parameter specifies texture layer that the texture
//! should be assigned to.
//! \param brush Brush handle
//! \param texture Texture handle
//! \param frame Texture frame
//! \param index Texture layer
//! \ingroup brushcommands
XDECL void			XCALL xBrushTexture(Brush brush, Texture texture, int frame = 0, int index = 0);

//! \brief Sets the camera fog mode. 
//! \details This will enable/disable fogging, a technique used to
//! gradually fade out graphics the further they are away from the camera.
//! This can be used to avoid 'pop-up', the moment at which 3D objects
//! suddenly appear on the horizon. 
//! The default fog colour is black and the default fog range is 1-1000,
//! although these can be changed by using xCameraFogColor() and
//! xCameraFogRange() respectively. 
//! Each camera can have its own fog mode, for multiple on-screen fog
//! effects.
//! \param camera Camera handle
//! \param fogMode For mode. See '\ref fogtypes' for more information
//! \ingroup camcommands
XDECL void			XCALL xCameraFogMode(Entity camera, int fogMode = FOG_NONE);

//! \brief Sets the camera projection mode
//! \param camera Camera handle
//! \param projMode Projection mode. See '\ref projtypes' for more information
//! \ingroup camcommands
XDECL void			XCALL xCameraProjMode(Entity camera, int projMode = PROJ_PERSPECTIVE);

//! \brief Removes all vertices and/or triangles from a surface. 
//! \details This is useful for clearing sections of mesh. The results will
//! be instantly visible. 
//! After clearing a surface, you may wish to add vertices and triangles to
//! it again but with a slightly different polygon count for dynamic level
//! of detail (LOD). 
//! \param surface Surface handle
//! \param clearVertices True to remove all vertices from the specified
//! surface, false not to
//! \param clearTrianlges True to remove all triangles from the specified
//! surface, false not to
//! \ingroup surfcommands
XDECL void			XCALL xClearSurface(Surface surface, bool clearVertices = true, bool clearTrianlges = true);

//! \brief Clears all entities, brushes and/or textures from the screen and
//! from memory. 
//! \details As soon as you clear something, you will not be able to use it
//! again until you reload it. Trying to do so will cause a runtime error. 
//! This command is useful for when a level has finished and you wish to
//! load a different level with new entities, brushes and textures. 
//! \param clearEntities True to clear all entities, false not to
//! \param clearBrushes True to clear all brushes, false not to
//! \param clearTexture True to clear all textures, false not to
//! \ingroup maincommands
XDECL void			XCALL xClearWorld(bool clearEntities = true, bool clearBrushes = true, bool clearTexture = true);

//! \brief Creates a copy of an entity and returns the handle of the newly
//! created copy.
//! This is a new entity instance of an existing entity's mesh. Anything
//! you do to the original mesh (such as xRotateMesh()) will effect all the
//! copies. Other properties (such as xEntityColor(), xPositionEntity(),
//! etc.) since they are 'Entity' properties, will be individual to the copy. 
//! If a parent entity is specified, the copied entity will be created at
//! the parent entity's position. Otherwise, it will be created at 0, 0, 0. 
//! \param entity Entity handle
//! \param parent Parent entity handle
//! \param cloneBuff If true vertex and indices buffers will be cloned
//! \ingroup eccommands
XDECL Entity		XCALL xCopyEntity(Entity entity, Entity parent = NULL, bool cloneBuff = true);

//! \brief Creates a copy of a mesh and returns the newly-created mesh's handle. 
//! \details The difference between xCopyMesh() and xCopyEntity() is that
//! xCopyMesh() performs a 'deep' copy of a mesh. 
//! \param entity Entity handle
//! \param parent Parent entity handle
//! \ingroup meshcommands
XDECL Entity		XCALL xCopyMesh(Entity entity, Entity parent = NULL);

//! \brief Copys a pixel from one buffer to other
//! \param sx x-coordinate of source pixel to copy from 
//! \param sy y-coordinate of source pixel to copy from 
//! \param src Source buffer handle
//! \param dx x-coordinate of destination pixel to copy to
//! \param dy y-coordinate of destination pixel to copy to 
//! \param dest Destination buffer handle
//! \ingroup maincommands
XDECL void			XCALL xCopyPixel(int sx, int sy, Buffer src, int dx, int dy, Buffer dest);

//! \brief Copys a pixel from one buffer to other
//! \details You must use this command on a locked buffer, otherwise the
//! command will fail. See xLockBuffer().
//! \param sx x-coordinate of source pixel to copy from 
//! \param sy y-coordinate of source pixel to copy from 
//! \param src Source buffer handle
//! \param dx x-coordinate of destination pixel to copy to
//! \param dy y-coordinate of destination pixel to copy to 
//! \param dest Destination buffer handle
//! \ingroup maincommands
XDECL void			XCALL xCopyPixelFast(int sx, int sy, Buffer src, int dx, int dy, Buffer dest);

//! \brief Creates a brush and returns a brush handle. 
//! \details The green, red and blue values allow you to set the colour of
//! the brush. Values should be in the range [0; 255]. If omitted the
//! values default to 255. 
//! A brush is a collection of properties such as color, alpha, shininess, 
//! textures, etc that are all stored as part of the brush. Then, all these
//! properties can be applied to an entity, mesh or surface at once just by
//! using xPaintEntity(), xPaintMesh() or xPaintSurface(). 
//! When creating your own mesh, if you wish for certain surfaces to look
//! differently from one another, then you will need to use brushes to paint
//! individual surfaces. Using commands such as xEntityColor(),
//! xEntityAlpha() will apply the effect to all surfaces at once, which may
//! not be what you wish to achieve. 
//! \param red Brush red value
//! \param green Brush green value
//! \param blue Brush blue value
//! \ingroup brushcommands
XDECL Brush			XCALL xCreateBrush(float red = 255, float green = 255, float blue = 255);

//! \brief Creates a camera entity and returns its handle. 
//! \details Without at least one camera, you won't be able to see anything
//! in your 3D world. With more than one camera, you will be to achieve
//! effect such as split-screen modes and rear-view mirrors. 
//! In Xors3D camera can render in any rendering buffer (xBackBuffer(),
//! xImageBuffer(), xTextureBuffer()).
//! The optional parent parameter allow you to specify a parent entity for
//! the camera so that when the parent is moved the child camera will move
//! with it. However, this relationship is one way; applying movement
//! commands to the child will not affect the parent. 
//! Specifying a parent entity will still result in the camera being
//! created at position 0, 0, 0 rather than at the parent entity's position. 
//! \param parent Parent entity handle
//! \ingroup camcommands
XDECL Entity		XCALL xCreateCamera(Entity parent = NULL);

//! \brief Creates a cone mesh/entity and returns its handle. 
//! \details The cone will be centred at 0, 0, 0 and the base of the cone
//! will have a radius of 1. 
//! The segments value must be in the range 3-100 inclusive. 
//! Example segments values (solid mesh): 
//! 
//! 4: 6 polygons - a pyramid 
//! 
//! 8: 14 polygons - bare minimum amount of polygons for a cone 
//!
//! 16: 30 polygons - smooth cone at medium-high distances 
//!
//! 32: 62 polygons - smooth cone at close distances 
//!
//! The optional parent parameter allow you to specify a parent entity for
//! the cone so that when the parent is moved the child cone will move with
//! it. However, this relationship is one way; applying movement commands
//! to the child will not affect the parent. 
//! Specifying a parent entity will still result in the cone being created
//! at position 0, 0, 0 rather than at the parent entity's position. 
//! \param segments Cone detail
//! \param solid True for a cone with a base, false for a cone without a base
//! \param parent Parent entity handle
//! \ingroup meshcommands
XDECL Entity		XCALL xCreateCone(int segments, bool solid = true, Entity parent = NULL);

//! \brief Creates a cube mesh/entity and returns its handle. 
//! \details The cube will extend from -1, -1, -1 to +1, +1, +1. 
//! The optional parent parameter allow you to specify a parent entity for
//! the cube so that when the parent is moved the child cube will move with
//! it. However, this relationship is one way; applying movement commands
//! to the child will not affect the parent. 
//! Specifying a parent entity will still result in the cube being created
//! at position 0, 0, 0 rather than at the parent entity's position. 
//! Creation of cubes, cylinders and cones are a great way of getting
//! scenes set up quickly, as they can act as placeholders for more complex
//! pre-modeled meshes later on in program development. 
//! \param parent Parent entity handle
//! \ingroup meshcommands
XDECL Entity		XCALL xCreateCube(Entity parent = NULL);

//! \brief Creates a cylinder mesh/entity and returns its handle. 
//! \details The cylinder will be centred at 0, 0, 0 and will have a radius
//! of 1. 
//! The segments value must be in the range 3-100 inclusive.
//! Example segments values (solid mesh): 
//!
//! 3: 8 polygons - a prism 
//!
//! 8: 28 polygons - bare minimum amount of polygons for a cylinder 
//! 
//! 16: 60 polygons - smooth cylinder at medium-high distances 
//! 
//! 32: 124 polygons - smooth cylinder at close distances 
//! 
//! The optional parent parameter allow you to specify a parent entity for
//! the cylinder so that when the parent is moved the child cylinder will
//! move with it. However, this relationship is one way; applying movement
//! commands to the child will not affect the parent. 
//! Specifying a parent entity will still result in the cylinder being
//! created at position 0, 0, 0 rather than at the parent entity's position. 
//! \param segments Cylinder detail
//! \param solid True for a cone with a base, false for a cone without a base
//! \param parent Parent entity handle
//! \ingroup meshcommands
XDECL Entity		XCALL xCreateCylinder(int segments, bool solid = true, Entity parent = NULL);

//! \brief Creates a new image with a single frame or multiple frames for
//! animation
//! \param width Width of the new image
//! \param height Height of the new image
//! \param frame Number of frames
//! \ingroup imagecommands
XDECL Image			XCALL xCreateImage(int width, int height, int frame = 1);

//! \brief Creates a light. 
//! \details Lights work by affecting the colour of all vertices within the
//! light's range. You need at to create at least one light if you wish to
//! use 3D graphics otherwise everything will appear flat. 
//! \param lType Type of light. See '\ref lighttypes' for more information
//! \ingroup lightcommands
XDECL Entity		XCALL xCreateLight(int lType = LIGHT_DIRECTIONAL);

//! \brief Creates a listener entity and returns its handle.
//! \details Currently, only a single listener is supported. 
//! \param parent Parent entity handle
//! \param rolloffFactor The rate at which volume diminishes with distance
//! \param dopplerScale The severity of the doppler effect
//! \param distanceScale Artificially scales distances
//! \ingroup audiocommands
XDECL Entity		XCALL xCreateListener(Entity parent = NULL, float rolloffFactor = 1.0f, float dopplerScale = 1.0f, float distanceScale = 1.0f);

//! \brief Creates a 'blank' mesh entity and returns its handle. 
//! \details When a mesh is first created it has no surfaces, vertices or
//! triangles associated with it
//! \param parent Parent entity handle
//! \ingroup meshcommands
XDECL Entity		XCALL xCreateMesh(Entity parent = NULL);\

//! \brief Creates a new particle in particle system and returns its index
//! \param entity Particle system handle
//! \param lifeTime Particles lifetime
//! \ingroup psyscommands
XDECL int			XCALL xCreateParticle(Entity entity, int lifeTime);

//! \brief Creates new particle system and return it handle
//! \details You may use particle system for creating effects, such as
//! fireworks, rain, snow, fire, etc.
//! \param texture Texture handle, that will be assigned to each particle
//! \param frame Texture frame
//! \param minps Min particle size
//! \param maxps max particle size
//! \param parent Parent entity handle
//! \ingroup psyscommands
XDECL Entity		XCALL xCreateParticleSystem(Texture texture, int frame = 0, float minps = 1.0f, float maxps = 100.0f, Entity parent = NULL);

//! \brief Sets particle size
//! \param entity particle system handle
//! \param index Particle index
//! \param size New particle size
//! \ingroup psyscommands
XDECL void			XCALL xSetParticleSize(Entity entity, int index, float size);

//! \brief Returns particle size
//! \param entity particle system handle
//! \param index Particle index
//! \ingroup psyscommands
XDECL float			XCALL xParticleSize(Entity entity, int index);

//! \brief Creates a pivot entity. 
//! \details A pivot entity is an invisible point in 3D space that's main
//! use is to act as a parent entity to other entities. The pivot can then
//! be used to control lots of entities at once, or act as new centre of
//! rotation for other entities. 
//! To enforce this relationship; use xEntityParent() or make use of the
//! optional parent entity parameter available with all entity
//! load/creation commands. 
//! Indeed, this parameter is also available with the xCreatePivot()
//! command if you wish for the pivot to have a parent entity itself. 
//! \param parent Parent entity handle
//! \ingroup meshcommands
XDECL Entity		XCALL xCreatePivot(Entity parent = NULL);

//! \brief Creates a sphere mesh/entity and returns its handle. 
//! \details The sphere will be centred at 0, 0, 0 and will have a radius
//! of 1. 
//! The segments value must be in the range 2-100 inclusive
//! 
//! Example segments values: 
//! 
//! 8: 224 polygons - bare minimum amount of polygons for a sphere 
//! 
//! 16: 960 polygons - smooth looking sphere at medium-high distances 
//! 
//! 32: 3968 polygons - smooth sphere at close distances 
//! 
//! The parent parameter allow you to specify a parent entity for
//! the sphere so that when the parent is moved the child sphere will move
//! with it. However, this relationship is one way; applying movement
//! commands to the child will not affect the parent. 
//! Specifying a parent entity will still result in the sphere being
//! created at position 0, 0, 0 rather than at the parent entity's position. 
//! \param segments Sphere detail
//! \param parent Parent entity handle
//! \ingroup meshcommands
XDECL Entity		XCALL xCreateSphere(int segments, Entity parent = NULL);

//! \brief Creates a sprite entity and returns its handle.
//! Sprites are simple flat (usually textured) rectangles made from two
//! triangles.
//! The sprite will be positioned at 0, 0, 0 and extend from 1, -1 to +1, +1. 
//! Sprites have two real strengths. The first is that they consist of only
//! two polygons; meaning you can use many of them at once. This makes them
//! ideal for particle effects and 2D-using-3D games where you want lots of
//! sprites on-screen at once. 
//! Secondly, sprites can be assigned a view mode using cSpriteViewMode().
//! By default this view mode is set to SPRITE_FIXED, which means the
//! sprite will always face the camera. So no matter what the orientation
//! of the camera is relative to the sprite, you will never actually notice
//! that they are flat; by giving them a spherical texture, you can make
//! them appear to look no different than a normal sphere. 
//! The parent parameter allow you to specify a parent entity for the
//! sprite so that when the parent is moved the child sprite will move with
//! it. However, this relationship is one way; applying movement commands
//! to the child will not affect the parent. 
//! Specifying a parent entity will still result in the sprite being created
//! at position 0, 0, 0 rather than at the parent entity's position. 
//! Note: Sprites have their own commands for rotation and scaling.
//! \param parent Parent entity handle
//! \ingroup sprcommands
XDECL Entity		XCALL xCreateSprite(Entity parent = NULL);

//! \brief Creates a surface attached to a mesh and returns the surface's handle. 
//! \details Surfaces are sections of mesh which are then used to attach
//! triangles to. You must have at least one surface per mesh in order to
//! create a visible mesh, however you can use as many as you like.
//! Splitting a mesh up into lots of sections allows you to affect those
//! sections individually, which can be a lot more useful than if all the
//! surfaces are combined into just one. 
//! \param entity Entity handle
//! \param brush Brush handle
//! \ingroup surfcommands
XDECL Surface		XCALL xCreateSurface(Entity entity, Brush brush);

//! \brief Creates a terrain entity and returns its handle. 
//! \details The terrain extends from 0, 0, 0 to size, 1, size. 
//! The parent parameter allows you to specify a parent entity for the
//! terrain so that when the parent is moved the child terrain will move
//! with it. However, this relationship is one way; applying movement
//! commands to the child will not affect the parent. 
//! Specifying a parent entity will still result in the terrain being
//! created at position 0, 0, 0 rather than at the parent entity's position.
//! \param size Number of grid squares along each side of terrain, and must
//! be a power of 2 value, e.g. 32, 64, 128, 256, 512, 1024
//! \param parent Parent entity handle
//! \ingroup terrcommands
XDECL Entity		XCALL xCreateTerrain(int size, Entity parent = NULL);

//! \brief Creates a texture and returns its handle. 
//! \details Width and height are the size of the texture. Note that the
//! actual texture size may be different from the width and height
//! requested, as different types of 3D hardware support different sizes
//! of texture. 
//! The optional flags parameter allows you to apply certain effects to the
//! texture. Flags can be added to combine two or more effects. See '\ref
//! tlflags' for more information
//! Once you have created a texture, use xSetBuffer(xTextureBuffer()) to
//! draw to it. Xors3D supports direct rendering into textures.
//! \param width Width of texture
//! \param height Height of texture
//! \param flags Texture creation flags
//! \param frames Number of frames texture will have
//! \ingroup texcommands
XDECL Texture		XCALL xCreateTexture(int width, int height, int flags = FLAGS_COLOR | FLAGS_MIPMAPPED, int frames = 1);

//! \brief Draws a previously loaded image.
//! \details This command draws both single image graphics (loaded with the
//! xLoadImage() command) as well as animated images (loaded with the
//! xLoadAnimImage() command). 
//! You specify where on the screen you wish the image to appear. You can
//! actually 'draw' off the screen as well by using negative values or
//! positive values that are not visible 'on the screen'. 
//! Finally, if you are using an animated image (loaded with the 
//! xLoadAnimImage()), you can specify which frame of the imagestrip is
//! displayed with the xDrawImage() command. 
//! \param image Image handle
//! \param x The x location of the screen to display the image 
//! \param y The y location of the screen to display the image 
//! \param frame The frame number of the animated image to display 
//! \ingroup imagecommands
XDECL void			XCALL xDrawImage(Image image, int x, int y, int frame = 0);

//! \brief Draws a rectangular portion of an image to the designated location
//! on the screen.
//! \details The transparent/masked portions of the original image will be
//! drawn transparent, just as you normally would draw an image.
//! \param image Image handle
//! \param x x location on the screen to draw the image
//! \param y y location on the screen to draw the image
//! \param rx Starting x location within the image to draw
//! \param ry Starting y location within the image to draw
//! \param width Width of the area to draw
//! \param height Height of the area to draw
//! \param frame Frame number of image to draw
//! \ingroup imagecommands
XDECL void			XCALL xDrawImageRect(Image image, int x, int y, int rx, int ry, int width, int height, int frame = 0);

//! \brief Attaches an entity to a parent. 
//! \details Parent may be 0, in which case the entity will have no parent
//! \param entity Entity handle
//! \param parent Parent entity handle
//! \param isGlobal True for the child entity to retain its global position
//! and orientation
//! \ingroup eccommands
XDECL void			XCALL xEntityParent(Entity entity, Entity parent = NULL, bool isGlobal = false);

//! \brief Sets the pick mode for an entity. 
//! \param entity Entity handle
//! \param mode Entity picking mode. See '\ref picktypes' for more infromation
//! \param obsc True to determine that the entity 'obscures' other entities
//! during an xEntityVisible() call
//! \ingroup ecolcommands
XDECL void			XCALL xEntityPickMode(Entity entity, int mode = PICK_NONE, bool obsc = false);

//! \brief Returns the pitch angle of an entity
//! \param entity Entity handle
//! \param isGlobal True if the pitch angle returned should be
//! relative to 0 rather than a parent entity's pitch angle
//! \ingroup escommands
XDECL float			XCALL xEntityPitch(Entity entity, bool isGlobal = false);

//! \brief Returns the roll angle of an entity
//! \param entity Entity handle
//! \param isGlobal True if the roll angle returned should be
//! relative to 0 rather than a parent entity's roll angle
//! \ingroup escommands
XDECL float			XCALL xEntityRoll(Entity entity, bool isGlobal = false);

//! \brief Sets the specular shininess of an entity. 
//! \details The shininess value should be a floting point number in the
//! range [0.0; 1.0]. The default shininess setting is 0.0 
//! Shininess is how much brighter certain areas of an object will appear
//! to be when a light is shone directly at them. 
//! Setting a shininess value of 1.0 for a medium to high poly sphere,
//! combined with the creation of a light shining in the direction of it,
//! will give it the appearance of a shiny snooker ball.
//! \param entity Entity handle
//! \param shininess Shininess of entity
//! \ingroup eccommands
XDECL void			XCALL xEntityShininess(Entity entity, float shininess);

//! \brief Applies a texture to an entity. 
//! \details The optional frame parameter specifies which texture animation
//! frame should be used as the texture. 
//! The optional index parameter specifies which index number should be
//! assigned to the texture. Index numbers are used for the purpose of
//! multitexturing. See xTextureBlend(). 
//! \param entity Entity handle
//! \param texture Texture handle
//! \param frame Frame of texture
//! \param index Index number of texture. Should be in the range to 0-7
//! \ingroup eccommands
XDECL void			XCALL xEntityTexture(Entity entity, Texture texture, int frame = 0, int index = 0);

//! \brief Returns x-coordinate of the entity. 
//! \details If the 'isGlobal' flag is set to ffalse then the parent's
//! local coordinate system is used. 
//! NOTE: If the entity has no parent then local and global coordinates are
//! the same. In this case you can think of the 3d world as the parent. 
//! Global coordinates refer to the 3d world. Xors3D uses a left-handed system: 
//!
//! X+ is to the right 
//!
//! Y+ is up 
//!
//! Z+ is forward (into the screen) 
//!
//! Every entity also has its own ;ocal coordinate system.
//! The global system never changes. 
//! But the local system is carried along as an entity moves and turns.
//! \param entity Entity handle
//! \param isGlobal True for global coordinates, false for local
//! \ingroup escommands
XDECL float			XCALL xEntityX(Entity entity, bool isGlobal = false);

//! \brief Returns y-coordinate of the entity. 
//! \details If the 'isGlobal' flag is set to ffalse then the parent's
//! local coordinate system is used. 
//! NOTE: If the entity has no parent then local and global coordinates are
//! the same. In this case you can think of the 3d world as the parent. 
//! Global coordinates refer to the 3d world. Xors3D uses a left-handed system: 
//!
//! X+ is to the right 
//!
//! Y+ is up 
//!
//! Z+ is forward (into the screen) 
//!
//! Every entity also has its own ;ocal coordinate system.
//! The global system never changes. 
//! But the local system is carried along as an entity moves and turns.
//! \param entity Entity handle
//! \param isGlobal True for global coordinates, false for local
//! \ingroup escommands
XDECL float			XCALL xEntityY(Entity entity, bool isGlobal = false);

//! \brief Returns the yaw angle of an entity
//! \param entity Entity handle
//! \param isGlobal True if the yaw angle returned should be
//! relative to 0 rather than a parent entity's yaw angle
//! \ingroup escommands
XDECL float			XCALL xEntityYaw(Entity entity, bool isGlobal = false);

//! \brief Returns z-coordinate of the entity. 
//! \details If the 'isGlobal' flag is set to ffalse then the parent's
//! local coordinate system is used. 
//! NOTE: If the entity has no parent then local and global coordinates are
//! the same. In this case you can think of the 3d world as the parent. 
//! Global coordinates refer to the 3d world. Xors3D uses a left-handed system: 
//!
//! X+ is to the right 
//!
//! Y+ is up 
//!
//! Z+ is forward (into the screen) 
//!
//! Every entity also has its own ;ocal coordinate system.
//! The global system never changes. 
//! But the local system is carried along as an entity moves and turns.
//! \param entity Entity handle
//! \param isGlobal True for global coordinates, false for local
//! \ingroup escommands
XDECL float			XCALL xEntityZ(Entity entity, bool isGlobal = false);

//! \brief Extracts part of specified sequence to a new one and returns its
//! index/
//! \details B3D format suppots only one sequence per file. You can arrange
//! all animations into one in editor and then extact them in Xors3D
//! \param enity Entity handle
//! \param firstFrame First frame of anim sequence to extract
//! \param lastFrame Last frame of anim sequence to extract
//! \param sequence Animation sequence to extract from
//! \ingroup eacommands
XDECL int			XCALL xExtractAnimSeq(Entity enity, int firstFrame, int lastFrame, int sequence = 0);

//! \brief Returns texture assigned to brush
//! \param brush Brush handle
//! \param index Texture layer index. Must be in range 0-7
//! \ingroup brushcommands
XDECL Texture		XCALL xGetBrushTexture(Brush brush, int index = 0);

//! \brief Sets 3D Graphics mode.
//! \details This command must be executed before any other 3D command,
//! otherwise programs will return an error. 
//! Width and height set the resolution of the screen. The resolution must be
//! compatible with the 3D card and monitor being used. If this value is set to 0,
//! then current system resolution will be used.
//! Depth sets the colour mode of the screen. If this value is omitted or
//! set to 0, then current system colour depth available will be
//! used. Other values usually available are 16 and 32. 16-bit colour mode
//! displays the least amount of colours, 65536. 32-bit colour modes 
//! display over 16 million colours and as a result offer a better quality
//! picture, although may result in slower programs than 16-bit.
//! \param width Width of screen resolution
//! \param height Height of screen resolution
//! \param depth Colour depth of screen
//! \param fullScreen True to enable fullscreen mode, false for windowed mode
//! \param vsync True to enable vertical synchronization, false to disable
//! \ingroup maincommands
XDECL bool			XCALL xGraphics3D(int width = 640, int height = 480, int depth = 16, bool fullScreen = false, bool vsync = false);

//! \brief Returns specified image buffer handle
//! \details You may use it for directly drawing into image
//! \param image Image handle
//! \param frame Frame of animated image
//! \ingroup imagecommands
XDECL Buffer		XCALL xImageBuffer(Image image, int frame = 0);

//! \brief Returns the first entity between x, y, z to dx, dy, dz
//! \param x x coordinate of start of line pick
//! \param y y coordinate of start of line pick
//! \param z z coordinate of start of line pick
//! \param dx x coordinate of end of line pick
//! \param dy y coordinate of end of line pick
//! \param dz z coordinate of end of line pick
//! \param distance Radius of line pick
//! \ingroup escommands
XDECL Entity		XCALL xLinePick(float x, float y, float z, float dx, float dy, float dz, float distance = 0.0f);

//! \brief Similar to xLoadMesh(), loads a mesh from .B3D file and returns
//! a mesh handle. 
//! \details The difference between xLoadMesh() and xLoadAnimMesh() is that
//! any hierarchy and animation information present in the file is retained.
//! You can then either activate the animation by using the xAnimate()
//! command or find child entities within the hierarchy by using the
//! xFindChild(), xGetChild() functions. 
//! The parent parameter allows you to specify a parent entity for the mesh
//! so that when the parent is moved the child mesh will move with it.
//! However, this relationship is one way; applying movement commands to
//! the child will not affect the parent. 
//! Specifying a parent entity will still result in the mesh being created
//! at position 0, 0, 0 rather than at the parent entity's position.
//! \param path Name of the file containing the model to load
//! \param parent Parent entity handle
//! \ingroup meshcommands
XDECL Entity		XCALL xLoadAnimMesh(const char * path, Entity parent = NULL);

//! \brief Appends an animation sequence from a file to an entity. 
//! Returns the animation sequence number added
//! \param entity Entity handle
//! \param path Filename of animated 3D object
//! \ingroup eacommands
XDECL int			XCALL xLoadAnimSeq(Entity entity, const char * path);

//! \brief Creates a brush, loads and assigns a texture to it, and returns
//! a brush handle.
//! \param path Filename of texture
//! \param flags Loading flags. See '\ref tlflags' for more information
//! \param xs Brush u scale
//! \param ys Brush v scale
//! \ingroup brushcommands
XDECL Brush			XCALL xLoadBrush(const char * path, int flags = FLAGS_COLOR | FLAGS_MIPMAPPED, float xs = 1.0f, float ys = 1.0f);

//! \brief Loads a font and returns a font handle
//! \param path Name of font to be loaded, e.g. "arial"
//! \param height Height of font in points
//! \param bold True to load bold version of font, false not to
//! \param italic True to load italic version of font, false not to
//! \param underline True to load underlined version of font, false not to
//! \ingroup textcommands
XDECL Font			XCALL xLoadFont(const char * path, int height, bool bold = false, bool italic = false, bool underline = false);

//! \brief Loads a mesh from an .B3D file and returns the mesh handle. 
//! \details Any hierarchy and animation information in the file will be
//! ignored. Use xLoadAnimMesh() to maintain hierarchy and animation
//! information. 
//! The parent parameter allows you to specify a parent entity for
//! the mesh so that when the parent is moved the child mesh will move with
//! it. However, this relationship is one way; applying movement commands
//! to the child will not affect the parent. 
//! Specifying a parent entity will still result in the mesh being created
//! at position 0, 0, 0 rather than at the parent entity's position. 
//! \param path Name of the file containing the model to load
//! \param parent Parent entity handle
//! \ingroup meshcommands
XDECL Entity		XCALL xLoadMesh(const char * path, Entity parent = NULL);

//! \brief Creates a sprite entity, assigns a texture to it, and retuns
//! sprite handle
//! \param path Filename of image file to be used as sprite
//! \param flags Texture loading flags. See '\ref tlflags' for more information
//! \param parent Parent entity handle
//! \ingroup sprcommands
XDECL Entity		XCALL xLoadSprite(const char * path, int flags = FLAGS_COLOR | FLAGS_MIPMAPPED, Entity parent = NULL);

//! \brief Loads a terrain from an image file and returns the terrain's handle. 
//! \details The image's red channel is used to determine heights. Terrain
//! is initially the same width and depth as the image, and 1.0 unit high. 
//! 
//! Tips on generating nice terrain: 
//! 
//! * Smooth or blur the height map 
//! 
//! * Reduce the y scale of the terrain 
//! 
//! * Increase the x/z scale of the terrain 
//! 
//! * Reduce the camera range 
//! 
//! When texturing an entity, a texture with a scale of 1, 1, 1 (default)
//! will be the same size as one of the terrain's grid squares. A texture
//! that is scaled to the same size as the size of the bitmap used to load
//! it or the no. of grid square used to create it, will be the same size
//! as the terrain. 
//! The parent parameter allows you to specify a parent entity for the
//! terrain so that when the parent is moved the child terrain will move
//! with it. However, this relationship is one way; applying movement
//! commands to the child will not affect the parent. 
//! Specifying a parent entity will still result in the terrain being
//! created at position 0, 0, 0 rather than at the parent entity's position. 
//! A heightmaps dimensions (width and height) must be the same and must be
//! a power of 2, e.g. 32, 64, 128, 256, 512, 1024.
//! \param path Filename of image file to be used as height map
//! \param parent Parent entity handle
//! \ingroup terrcommands
XDECL Entity		XCALL xLoadTerrain(const char * path, Entity parent = NULL);

//! \brief Loads a texture from an image file and returns the texture's handle
//! \details The optional flags parameter allows you to apply certain effects
//! to the texture. Flags can be added to combine two or more effects. See
//! '\ref tlflags' for more infromation. Supported file formats are:
//! bmp, dds, dib, hdr, jpg, pfm, png, ppm, tga.
//! \param path Filename of image file to be used as texture
//! \param flag Loading flags
//! \ingroup texcommands
XDECL Texture		XCALL xLoadTexture(const char * path, int flag = FLAGS_COLOR | FLAGS_MIPMAPPED);

//! \brief Locks specified buffer
//! \details You must UnlockBuffer before using other graphics commands or
//! API calls, and you are advised to only keep the buffer locked for as
//! long as it is needed.
//! \param buffer Buffer handle
//! \ingroup maincommands
XDECL void			XCALL xLockBuffer(Buffer buffer = -1);

//! \brief Sets the height of a point on a terrain
//! \param entity Entity handle
//! \param x Grid x coordinate of terrain
//! \param y Grid y coordinate of terrain
//! \param height Height of point on terrain. Should be in the range [0.0; 1.0]
//! \param realTime True to modify terrain immediately. False to modify
//! terrain when xRenderWorld() in next called
//! \ingroup terrcommands
XDECL void			XCALL xModifyTerrain(Entity entity, int x, int y, float height, bool realTime = false);

//! \brief Moves an entity relative to its current position and orientation. 
//! \details What this means is that an entity will move in whatever
//! direction it is facing. So for example if you have an game character
//! is upright when first loaded and it remains upright (i.e. turns left or
//! right only), then moving it by a z amount will always see it move
//! forward or backward, moving it by a y amount will always see it move up
//! or down, and moving it by an x amount will always see it strafe.
//! \param entity Entity handle
//! \param x x amount that entity will be moved by
//! \param y y amount that entity will be moved by
//! \param z z amount that entity will be moved by
//! \param isGlobal Unused
//! \ingroup emcommands
XDECL void			XCALL xMoveEntity(Entity entity, float x, float y, float z, bool isGlobal = false);

//! \brief Draws an oval shape at the screen coordinates of your choice.
//! \details You can make the oval solid or hollow. 
//! \param x x coordinate on the screen to draw the oval
//! \param y y coordinate on the screen to draw the oval
//! \param width How wide to make the oval
//! \param height How high to make the oval
//! \param solid True to make the oval solid, false for hollow
//! \ingroup maincommands
XDECL void			XCALL xOval(int x, int y, int width, int height, bool solid = false);

//! \brief Sets a texture for particle system
//! \param entity Particle system handle
//! \param texture Texture handle
//! \param frame Texture frame
//! \ingroup psyscommands
XDECL void			XCALL xParticleSystemTexture(Entity entity, Texture texture, int frame = 0);

//! \brief Points one entity at another. 
//! \details The roll parameter allows you to specify a roll angle as
//! pointing an entity only sets pitch and yaw angles. 
//! If you wish for an entity to point at a certain position rather than
//! another entity, simply create a pivot entity at your desired position,
//! point the entity at this and then free the pivot.
//! \param entity1 First entity handle
//! \param entity2 Second entity handle
//! \param roll Roll angle of entity
//! \ingroup emcommands
XDECL void			XCALL xPointEntity(Entity  entity1, Entity entity2, float roll = 0.0f);

//! \brief Positions an entity at an absolute position in 3D space. 
//! \details Entities are positioned using an x, y, z coordinate system.
//! x, y and z each have their own axis, and each axis has its own set of
//! values. By specifying a value for each axis, you can position an entity
//! anywhere in 3D space. 0, 0, 0 is the centre of 3D space, and if the
//! camera is pointing in the default positive z direction, then positioning
//! an entity with a z value of above 0 will make it appear in front of the
//! camera, whereas a negative z value would see it disappear behind the
//! camera. Changing the x value would see it moving sideways, and changing
//! the y value would see it moving up/down. 
//! Of course, the direction in which entities appear to move is relative
//! to the position and orientation of the camera.
//! \param entity Entity handle
//! \param x x co-ordinate that entity will be positioned at
//! \param y y co-ordinate that entity will be positioned at
//! \param z z co-ordinate that entity will be positioned at
//! \param isGlobal True if the position should be relative to 0, 0, 0
//! rather than a parent entity's position
//! \ingroup emcommands
XDECL void			XCALL xPositionEntity(Entity entity, float x, float y, float z, bool isGlobal = false);

//! \brief Reads a color value from either the current buffer or the
//! specified buffer. 
//! \details The returned colour value is in the form of an integer that
//! contains the alpha, red, green and blue values of the pixel. 
//! You can use this command on a locked buffer for a slight speed-up. See
//! xLockBuffer()
//! \param x x coordinate of pixel
//! \param y y coordinate of pixel
//! \param buffer Handle of buffer to read colour value from
//! \ingroup maincommands
XDECL int			XCALL xReadPixel(int x, int y, Buffer buffer = -1);

//! \brief Reads a color value from either the current buffer or the
//! specified buffer. 
//! \details The returned colour value is in the form of an integer that
//! contains the alpha, red, green and blue values of the pixel. 
//! You must use this command on a locked buffer, otherwise the command
//! will fail. See xLockBuffer()
//! \param x x coordinate of pixel
//! \param y y coordinate of pixel
//! \param buffer Handle of buffer to read colour value from
//! \ingroup maincommands
XDECL int			XCALL xReadPixelFast(int x, int y, Buffer buffer = -1);

//! \brief Draws a rectangle in the current drawing color starting at the
//! location specified.
//! \param x x coordinate to begin drawing the rectangle
//! \param y y coordinate to begin drawing the rectangle
//! \param width How wide to make the rectangle in pixels
//! \param height How tall to make the rectangle in pixels
//! \param solid False for unfilled and true for filled
//! \ingroup maincommands
XDECL void			XCALL xRect(int x, int y, int width, int height, bool solid = false);

//! \brief Renders the current scene to the current rendering buffer onto
//! the rectangle defined by each cameras xCameraViewport().
//! \details Every camera not hidden by xHideEntity() or with a 
//! xCameraProjMode() of 0 is rendered. Xors3D supported directly rendering
//! into images and textures.
//! \param twen Unused
//! \ingroup maincommands
XDECL void			XCALL xRenderWorld(float twen = 1.0f);

//! \brief Rotates an entity so that it is at an absolute orientation. 
//! \param entity Entity handle
//! \param x Angle in degrees of pitch rotation
//! \param y Angle in degrees of yaw rotation
//! \param z Angle in degrees of roll rotation
//! \param isGlobal True if the angle rotated should be relative to 0, 0, 0
//! rather than a parent entity's orientation
//! \ingroup emcommands
XDECL void			XCALL xRotateEntity(Entity entity, float x, float y, float z, bool isGlobal = false);

//! \brief Saves an image or one of its frames to hard drive.
//! \param image Image handle
//! \param path String with filename to save to
//! \param frame Which frame of the image to save
//! \ingroup imagecommands
XDECL void			XCALL xSaveImage(Image image, const char * path, int frame = 0);

//! \brief Scales an entity so that it is of an absolute size. 
//! \details Scale values of 1, 1, 1 are the default size when creating
//! /loading entities. 
//! Scale values of 2, 2, 2 will double the size of an entity. 
//! Scale values of 0, 0, 0 will make an entity disappear. 
//! Scale values of less than 0, 0, 0 will invert an entity and make it bigger.
//! \param entity Entity handle
//! \param x x size of entity
//! \param y y size of entity
//! \param z z size of entity
//! \param isGlobal Unused
//! \ingroup emcommands
XDECL void			XCALL xScaleEntity(Entity entity, float x, float y, float z, bool isGlobal = false);

//! \brief Sets entity's animation speed
//! \param entity Entity handle
//! \param speed New animation speed
//! \param rootBone Root bone. See xAnimate() for more infromation
//! \ingroup eacommands
XDECL void			XCALL xSetAnimSpeed(Entity entity, float speed, const char * rootBone = "");

//! \brief Sets entity's animation time
//! \param entity Entity handle
//! \param time New animation time
//! \param rootBone Root bone. See xAnimate() for more infromation
//! \ingroup eacommands
XDECL void			XCALL xSetAnimTime(Entity entity, float time, const char * rootBone = "");

//! \brief Selects a cube face for direct rendering to a texture. 
//! \details This command should only be used when you wish to draw
//! directly to a cubemap texture in real-time. Otherwise, just loading a
//! pre-rendered cubemap with a flag of FLAGS_CUBICENVMAP will suffice. 
//! To understand how this command works exactly it is important to
//! recognise that Xors3D treats cubemap textures slightly differently to
//! how it treats other textures. Here's how it works... 
//! A cubemap texture in Xors3D actually consists of six images, each of
//! which must be square 'power' of two size - e.g. 32, 64, 128 etc. Each
//! corresponds to a particular cube face. These images are stored
//! internally by Xors3D, and the texture handle that is returned by
//! xLoadTexture()/xCreateTexture() when specifying the cubemap flag, only
//! provides access to one of these six images at once (by default the first
//! one, or 'FACE_FORWARD' face).
//! How do we get access to the other five images? This is where 
//! xSetCubeFace() comes in. It will tell Xors3D that whenever you next
//! draw to a cubemap texture, to draw to the particular image representing
//! the face you have specified with the face parameter. 
//! \param texture Texture handle
//! \param face Cubemap face. See '\ref facetypes' for more infromation
//! \ingroup texcommands
XDECL void			XCALL xSetCubeFace(Texture texture, int face = FACE_FORWARD);

//! \brief Sets a texture constant for entity shader
//! \param entity Entity handle
//! \param name Constant name
//! \param texture Texture handle
//! \param frame Texture frame
//! \ingroup shadercommands
XDECL void			XCALL xSetEffectTexture(Entity entity, const char * name, Texture texture, int frame = 0);

//! \brief Sets a texture constant for surface shader
//! \param surface Surface handle
//! \param name Constant name
//! \param texture Texture handle
//! \param frame Texture frame
//! \ingroup shadercommands
XDECL void			XCALL xSurfaceEffectTexture(Surface surface, const char * name, Texture texture, int frame = 0);

//! \brief Sets a vector constant for entity shader
//! \param entity Entity handle
//! \param name Constant name
//! \param x Vertor's x component
//! \param y Vertor's y component
//! \param z Vertor's z component
//! \param w Vertor's w component
//! \ingroup shadercommands
XDECL void			XCALL xSetEffectVector(Entity entity, const char * name, float x, float y, float z, float w = 1.0f);

//! \brief Sets a vector constant for surface shader
//! \param surface Surface handle
//! \param name Constant name
//! \param x Vertor's x component
//! \param y Vertor's y component
//! \param z Vertor's z component
//! \param w Vertor's w component
//! \ingroup shadercommands
XDECL void			XCALL xSurfaceEffectVector(Surface surface, const char * name, float x, float y, float z, float w = 1.0f);

//! \deprecated
XDECL void			XCALL xSetPixelShaderEntityTexture(Entity entity, const char * name, int index = 0);

//! \brief Sets shader texture constant by entity's texture
//! \details You can use it for automatic passing textures loaded with mesh
//! or applied by xEntityTexture() to a shader.
//! \param entity Entity handle
//! \param name Constant name
//! \param index Textre layer index. Must be in range 0-7
//! \ingroup shadercommands
XDECL void			XCALL xSetEffectEntityTexture(Entity entity, const char * name, int index = 0);

//! \brief Sets surface shader texture constant by entity's texture
//! \details You can use it for automatic passing textures loaded with mesh
//! or applied by xEntityTexture() to a shader.
//! \param surface Surface handle
//! \param name Constant name
//! \param index Textre layer index. Must be in range 0-7
//! \ingroup shadercommands
XDECL void			XCALL xSurfaceEffectEntityTexture(Surface surface, const char * name, int index = 0);

//! \deprecated
XDECL void			XCALL xSetPixelShaderTexture(Entity entity, const char * name, Texture texture, int frame = 0);

//! \deprecated
XDECL void			XCALL xSetPixelShaderVector(Entity entity, const char * name, float x, float y, float z, float w);

//! \deprecated
XDECL void			XCALL xSetPostEffect(PostEffect postEffect, int index = 0);

//! \deprecated
XDECL void			XCALL xSetPostEffectVector(PostEffect postEffect, const char * name, float x, float y, float z, float w);

//! \brief Sets skinning type
//! \details If mesh is loaded by xLoadAnimMesh() it will use skinning
//! method set by this function. You can switch skinning method before
//! loading mesh if you need different skinning types for diffeent meshes
//! \param method Skinning method. See '\ref skinmethods' for more infromation
//! \ingroup maincommands
XDECL void			XCALL xSetSkinningMethod(int method = SKIN_SOFTWARE);

//! \deprecated
XDECL void			XCALL xSetVertexShaderVector(Entity entity, const char * name, float x, float y, float z, float w);

//! \brief Prints a string at the designated screen coordinates.
//! \param x Starting x coordinate to print text
//! \param y Starting y coordinate to print text
//! \param text Text to print 
//! \param centerx True to center horizontally
//! \param centery True to center vertically
//! \ingroup textcommands
XDECL void			XCALL xText(int x, int y, const char * text, bool centerx = false, bool centery = false);

//! \brief Returns the handle of a texture's drawing buffer. 
//! \details This can be used with xSetBuffer() to perform direct drawing
//! operations to the texture
//! \param texture Texture handle
//! \param frame Texture frame
//! \ingroup texcommands
XDECL Buffer		XCALL xTextureBuffer(Texture texture, int frame = 0);

//! \brief Tiles image on the screen
//! \param image Image handle
//! \param x Starting x location of the tile
//! \param y Starting y location of the tile
//! \param frame Image frame
//! \ingroup imagecommands
XDECL void			XCALL xTileImage(Image image, int x, int y, int frame = 0);

//! \brief Translates an entity relative to its current position and not
//! its orientation. 
//! \details What this means is that an entity will move in a certain
//! direction despite where it may be facing. Imagine that you have a game
//! character that you want to make jump in the air at the same time as
//! doing a triple somersault. Translating the character by a positive y
//! amount will mean the character will always travel directly up in their
//! air, regardless of where it may be facing due to the somersault action. 
//! \param entity Entity handle
//! \param x x amount that entity will be translated by
//! \param y y amount that entity will be translated by
//! \param z z amount that entity will be translated by
//! \param isGlobal Unused
//! \ingroup emcommands
XDECL void			XCALL xTranslateEntity(Entity entity, float x, float y, float z, bool isGlobal = false);

//! \brief Turns an entity relative to its current orientation. 
//! \param entity Entity handle
//! \param x Angle in degrees that entity will be pitched
//! \param y Angle in degrees that entity will be yawed
//! \param z Angle in degrees that entity will be rolled
//! \param isGlobal True to turn in global coordinate system, false - for
//! local
//! \ingroup emcommands
XDECL void			XCALL xTurnEntity(Entity entity, float x, float y, float z, bool isGlobal = false);

//! \brief Unlocks locked buffer
//! \param buffer Buffer handle
//! \ingroup maincommands
XDECL void			XCALL xUnlockBuffer(Buffer buffer = -1);

//! \brief Animates all entities in the world, and performs collision checking. 
//! The speed parameter allows you affect the animation speed of all
//! entities at once. A value of 1.0 will animate entities at their usual
//! animation speed, a value of 2.0 will animate entities at double their
//! animation speed, and so on. 
//! For best results use this command once per main loop, just before
//! calling xRenderWorld(). 
//! \param speed A master control for animation speed
//! \ingroup maincommands
XDECL void			XCALL xUpdateWorld(float speed = 1.0f);

//! \brief Sets the color of an existing vertex
//! \param surface Surface handle
//! \param vertex Vertex index
//! \param red Red value of vertex color
//! \param green Green value of vertex color
//! \param blue Blue value of vertex color
//! \param alpha Alpha value of vertex color
//! \ingroup surfcommands
XDECL void			XCALL xVertexColor(Surface surface, int vertex, int red, int green, int blue, float alpha = 1.0f);

//! \brief Sets the texture coordinates of an existing vertex
//! \param surface Surface handle
//! \param vertex Vertex index
//! \param u u coordinate of vertex
//! \param v v coordinate of vertex
//! \param w w coordinate of vertex
//! \param coodSet Texture coodrinates set. Should be set to 0 or 1.
//! \ingroup surfcommands
XDECL void			XCALL xVertexTexCoords(Surface surface, int vertex, float u, float v, float w = 1.0f, int coodSet = 0);

//! \brief Returns the texture u coordinate of a vertex
//! \param surface Surface handle
//! \param vertex Vertex index
//! \param coodSet Texture coodrinates set. Should be set to 0 or 1.
//! \ingroup surfcommands
XDECL float			XCALL xVertexU(Surface surface, int vertex, int coodSet = 0);

//! \brief Returns the texture v coordinate of a vertex
//! \param surface Surface handle
//! \param vertex Vertex index
//! \param coodSet Texture coodrinates set. Should be set to 0 or 1.
//! \ingroup surfcommands
XDECL float			XCALL xVertexV(Surface surface, int vertex, int coodSet = 0);

//! \brief Returns the texture w coordinate of a vertex
//! \param surface Surface handle
//! \param vertex Vertex index
//! \param coodSet Texture coodrinates set. Should be set to 0 or 1.
//! \ingroup surfcommands
XDECL float			XCALL xVertexW(Surface surface, int vertex, int coodSet = 0);

//! \brief Writes a color value to either the current buffer or the 
//! specified buffer. 
//! \details You can use this command on a locked buffer for a slight
//! speed-up. 
//! \param x x-coordinate of pixel
//! \param y y-coordinate of pixel
//! \param argb ARGB color value of pixel (alpha, red, green, blue)
//! \param buffer Buffer handle
//! \ingroup maincommands
XDECL void			XCALL xWritePixel(int x, int y, int argb, Buffer buffer = -1);

//! \brief Writes a color value to either the current buffer or the 
//! specified buffer. 
//! \details You must use this command on a locked buffer, otherwise the
//! command will fail 
//! \param x x-coordinate of pixel
//! \param y y-coordinate of pixel
//! \param argb ARGB color value of pixel (alpha, red, green, blue)
//! \param buffer Buffer handle
//! \ingroup maincommands
XDECL void			XCALL xWritePixelFast(int x, int y, int argb, Buffer buffer = -1);

//! \brief Sets shader effect for surface
//! \param surface Surafce handle
//! \param effect Effect handle
//! \ingroup shadercommands
XDECL void XCALL xSetSurfaceEffect(Surface surface, Effect effect);

//! \brief Sets shader constant name for passing bones for skinning computing
//! \details If you use hadrware skinning, you should pass bone transformation
//! matrices to shader and transform vertices in it. This command
//! specify constant name for each surface in mesh separatly.
//! \param surface Surface handle
//! \param baName Name of shader constant for passing bone matrices
//! \ingroup shadercommands
XDECL void XCALL xSurfaceBonesArrayName(Surface surface, const char * baName);

#endif
